/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christoph Wellner <chwellner@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef MMBOX_EPGSEARCHRESULT_HPP
#define MMBOX_EPGSEARCHRESULT_HPP MMBOX_EPGSEARCHRESULT_HPP

// the app
#include "../../main/application.hpp"
#include "../../main/mmboxstate.hpp"

#include "nmm/interfaces/video/osd/manager/ITextView.hpp"
#include "nmm/interfaces/misc/tvguide/ITVGuide.hpp"
#include "nmm/interfaces/misc/tvguide/ITVGuidePeerObjectImpl.hpp"
#include "nmm/interfaces/misc/tvtimer/ITVTimer.hpp"
#include "nmm/plugins/video/osd/manager/OSDElements/listunit.hpp"

#include "nmm/utils/thread/Thread.hpp"

#include "nmm/misc/tvguide/TVGuide.hpp"


namespace NMM{

  class EPGSearchResult : public MMBoxState, 
                          public ITVGuidePeerObjectImpl
  {

  private:
    list<ProgramDescription> queryResult;
    int currentShortResult;
    ThreadMutex sortMutex;

  public:
    EPGSearchResult(string, MMBoxApplication*);
    ~EPGSearchResult();

    typedef struct {
      unsigned int  xpos;
      unsigned int  ypos;
      unsigned int  width;
      unsigned int  height;
      unsigned int  win_active_color;
      unsigned int  win_inactive_color;
      unsigned int  sel_active_color;
      unsigned int  sel_active_color2;
      unsigned int  sel_inactive_color;
    } win_properties_t;

    enum wintype_e {SHORTRESULT =0, EXTRESULT};

    void setWinProperties( int, win_properties_t* );

    Result Up();
    Result Down();
    Result Left();
    Result Right();
    Result Return();
    Result Back();
    Result Record();
    
    Result colorKeyRed();
    Result colorKeyGreen();
    Result colorKeyYellow();
    Result colorKeyBlue();
    
    Result doCreate(string); 
    Result doKill();
    Result doInitialize(string);
    Result doUninitialize();

    Result showOSD();
    Result hideOSD();
    Result updateOSD();

    void startSearch(ProgramFilter filter);

  protected:
    Result peerResult (const ProgramDescriptionList& pdlist, 
                       const int& id, 
                       const list<Interface*>& /*history*/);

    Result peerChannelResult (const ChannelDescriptionList& /*cdlist*/, 
                              const list<Interface*>& /*history*/){return SUCCESS;}

  private:
    ITVGuide_var itvgserver;
    ITVTimer_var itvtimer;
    IOSDManager* manager;

    /*Windows for results */
    ListUnit* shortResult;
    ListUnit* extResult;

    /** properties of the windows */
    win_properties_t  winprops[2];

    /** window with input focus */
    wintype_e win_focus;


    int currentQueryID;

    /** delete created windows */
    void deleteWindows();
    /** create windows for the guide */
    void createWindow(wintype_e w, string h);
    void updateExtResult();

    string formatTimeString(string s, bool timeOnly);
    bool lineMarked(list<unsigned int> l, unsigned int i);
  };

}

#endif
