/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include"automenu.hpp"

#include "../../main/application.hpp"

#include "nmm/base/ProxyObject.hpp"

using namespace NMM;

AutoMenu::AutoMenu( string id, MMBoxApplication* _app ) : MMBoxState( id, _app ) {
  manager = app->getOsdManager();
  factory = manager->getParentObject()->getCheckedInterface<IWidgetFactory>();
  
  buttons = factory->createRadioButtons();
  
  first_init = true;
  win = NULL;
}

AutoMenu::~AutoMenu() {
  if (buttons) {
    manager->deleteWidget(*buttons);
    delete buttons;
    buttons = NULL;
  }

  if ( win ) {
    manager->deleteWindow(*win);
    delete win;
    win = NULL;
  }

  if (factory) {
    delete factory;
    factory = NULL;
  }
}

void AutoMenu::addButton( int buttonid, string label, int x, int y, string inactive_id, string active_id, string select_id ) {
  buttons->addButton( buttonid, label, x, y, inactive_id, active_id, select_id );
}

void AutoMenu::setStep( unsigned int s ) {
  buttons->setStep( s );
}

void AutoMenu::setDimension( unsigned int w, unsigned int h ) {
  buttons->setDimension( w, h );
}

Result AutoMenu::doCreate( string ) {
  GLOBAL_DEBUG_STREAM("AutoMenu: initialize: " << stateid << endl);
  
  if ( isBackgroundFree() ) {
    GLOBAL_DEBUG_STREAM("AutoMenu: creating default graph" << endl);
    if (FAILURE == app->createDefaultGraph( background )) {
      return FAILURE;
    }
    GLOBAL_DEBUG_STREAM("AutoMenu: created default graph" << endl);
    
    if (FAILURE == app->startDefaultGraph()) {
      return FAILURE;
    }
  }

  win = manager->createWindow();
  win->setDimension( 720, 576 );
  win->setPosition( 0, 0 );
  win->setContent( *buttons );
  
  if ( first_init ) {
    /* select first button only if it is the first init */
    buttons->selectButton( 1 );
    first_init = false;
  }
  
  win->update();

  manager->update();
  
  return( SUCCESS );
}

Result AutoMenu::doKill() {
  if ( win ) {
    /* delete Window from OSD display list */
    GLOBAL_DEBUG_STREAM("deleting Window" << endl);
    manager->deleteWindow( *win );
    delete win;
    win = NULL;
    GLOBAL_DEBUG_STREAM("deleted Window" << endl);
  }
  
  if ( isBackgroundFree() ) {
    GLOBAL_DEBUG_STREAM("disconnecting video graph" << endl);
    app->disconnectVideoGraph();
    GLOBAL_DEBUG_STREAM("disconnected video graph" << endl);
  }
  
  return( SUCCESS );
}

Result AutoMenu::doInitialize( string ) {
  doCreate("");

  return( SUCCESS );
}

Result AutoMenu::doUninitialize() {
  doKill();
  
  return( SUCCESS );
}


Result AutoMenu::Down() {
  GLOBAL_DEBUG_STREAM( "AutoMenu: Down" << endl );
  
  buttons->selectDown();
  
  win->update();
  manager->update();
  
  return(SUCCESS);
}

Result AutoMenu::Up() {
  GLOBAL_DEBUG_STREAM( "AutoMenu: Up" << endl );

  buttons->selectUp();

  win->update();
  manager->update();

  return(SUCCESS);
}

Result AutoMenu::Left() {
  GLOBAL_DEBUG_STREAM( "AutoMenu: Left" << endl );

  buttons->selectLeft();

  win->update();
  manager->update();

  return(SUCCESS);
}

Result AutoMenu::Right() {
  GLOBAL_DEBUG_STREAM( "AutoMenu: Right" << endl );

  buttons->selectRight();

  win->update();
  manager->update();

  return(SUCCESS);
}

Result AutoMenu::Back() {
  GLOBAL_DEBUG_STREAM( "AutoMenu: Back" << endl );
  GLOBAL_DEBUG_STREAM( "Selected Menu entry ID:" << lastid << endl );

  deactivate();

  return( goToLastState() );
}

Result AutoMenu::Return() {
  GLOBAL_DEBUG_STREAM( "AutoMenu: Return" << endl );
  GLOBAL_DEBUG_STREAM( "Selected Menu entry ID:" << stateid << endl );

  deactivate();

  GLOBAL_DEBUG_STREAM("AutoMenu: Getting selected button label" << endl);
  string stateid = buttons->getSelectedButtonLabel();
  GLOBAL_DEBUG_STREAM("AutoMenu: Got selected button label: " << stateid << ", changing state" << endl);
  if ( changeState( stateid ) == FAILURE ) {
    GLOBAL_DEBUG_STREAM( "Menuentry with id: " << stateid << " not found" << endl );
    return( FAILURE );
  }
  GLOBAL_DEBUG_STREAM("AutoMenu: changed state" << endl);

  return(SUCCESS);
}

