/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef MMBOX_MP3_HPP
#define MMBOX_MP3_HPP MMBOX_MP3_HPP

#include "../../main/application.hpp"
#include "nmm/utils/DirReader.hpp"

// include used interfaces
#include "nmm/interfaces/general/progress/IProgress.hpp"
#include "nmm/interfaces/general/progress/ITimeProgress.hpp"
#include "nmm/interfaces/general/progress/ITimeProgressListener.hpp"
#include "nmm/interfaces/video/osd/manager/ITextView.hpp"
#include "nmm/interfaces/video/osd/manager/IPlaneDecorator.hpp"
#include "nmm/interfaces/video/osd/manager/IScrollDecorator.hpp"
#include "nmm/interfaces/video/osd/manager/IBorderDecorator.hpp"
#include "nmm/interfaces/video/osd/manager/IHeaderDecorator.hpp"
#include "nmm/interfaces/video/osd/manager/IWidgetFactory.hpp"
#include "nmm/interfaces/file/IFileHandler.hpp"
#include "nmm/interfaces/file/ISeekable.hpp"
#include <stack>

namespace NMM {
/** MP3State
  * @bug No seeking symbols during seek process/
  * @bug marker in playlist-window stays on the forst song selected, when playing the next song
  */
  class MP3State : public MMBoxState {
  private:
    // Variables
    enum MP3Status { PLAYING, STOPPED, PAUSED };
    MP3Status state;
  
    DirReader	    dirreader;
    DirReader	    dirreader2;
    vector<DirReader::directory_t> dir;
    vector<DirReader::directory_t> dir2;
    stack<int> dir_position;

    // nodes owned by the application
    INode* osd_node;
    INode* audio_play;
  
    // own nodes
    INode* readfile;
    INode* mp3dec;
    INode* copy;
  
    // widgets
    ITextView* text;
    IPlaneDecorator* plane;
    IBorderDecorator* border;
    IScrollDecorator* scroll;
    IHeaderDecorator* header;
  
    // window
    IWindow* win;
  
    // other interfaces
    IProgress* progress;
    ITimeProgress* time_progress;
    IFileHandler* file_source;
    ISeekable* iseek;
    IOSDManager* manager;
    IWidgetFactory* factory;
  
    string mp3rootpath;
    unsigned int actual_playing_line;
  
    const string mp3suffix;
    const string MP3suffix;
  
    string playing_titel;

    bool first_file;
    bool loaded_m3u;

    ///! the last values received as progress
    Rational current_progress;

    TEDObject0<MP3State> endTrack_listener;
    TEDObject0<MP3State> syncReset_listener;
    TEDObject2<MP3State, u_int64_t, u_int64_t> setProgress_listener;
    TEDObject1<MP3State, Time> setTimeProgress_listener;
    TEDObject1<MP3State, Interval> trackDuration_listener;

    /** duration of the current track (in seconds). */
    u_int64_t track_duration;
    
    Result playLine( unsigned int );
  
    // Functions
    void playmp3(string filename);
    void stopmp3();
    void pausemp3();
    void unpausemp3();
    void loadm3u( string filename );
  
    void replaceToken(string* textStr, string token, string replacement);
    void replaceTokens(string* textStr);
  
    Result initSound();
    void deInitSound();
  
    void createWindow();
    void deleteWindow();
  
    void dirToText();
    void readDir();
  
  public:
    MP3State( string id, MMBoxApplication* _app );
    ~MP3State();
  
    // event listeners
    Result endTrack();
    Result syncReset();
    Result setProgress( u_int64_t& numerator, u_int64_t& demoninator );
    Result setTimeProgress(Time& current_media_time);
    Result trackDuration(Interval& duration);
  
    // some helper functions
    Result doCreate( string );
    Result doKill();
    Result doInitialize( string );
    Result doUninitialize();
    Result seek(int seektime);
    Result selectEntry(string &eventvalue);
  
    // here all used "events" must be entered
    Result Back();
    Result Up();
    Result Down();
    Result Left();
    Result Right();
    Result Return();
    Result Pause();
    Result Rewind();
    Result Forward();
    Result Stop();
  };

}

#endif //MMBOX_MP3_HPP
