/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef __PLAYLIST__
#define __PLAYLIST__

#include <vector>
#include <string>

#include"../../main/application.hpp"
#include "../../main/mmboxstate.hpp"
#include "../../main/PlaylistTemplateFunctions.hpp"

#include "nmm/plugins/video/osd/manager/OSDElements/OSDElementComposit.hpp"
#include "nmm/base/graph/GraphBuilder2.hpp"

#include "nmm/interfaces/file/ISeekable.hpp"

#include "../../gui/WindowSelectorPlaylist.hpp"

namespace NMM {
  
  /** Playlist
   * @bug No seeking symbols during seek process
   */
  class Playlist : public MMBoxState,
                   public PlaylistTemplateFunctions {

  public: 
    /**
     * constructor needs a string id that identifies the class
     * \param id the menu id string
     */
    Playlist( string, MMBoxApplication* );
    ~Playlist();

    Result Up();
    Result Down();
    Result Left();
    Result Right();
    Result Return();
    Result Back();
    Result doSelectTrackWindow(string url);
    Result doSelectPlaylistWindow(string url);
    void   doChangeState(string state);

    Result colorKeyRed();
    Result colorKeyGreen();
    Result colorKeyYellow();
    Result colorKeyBlue();
    
    Result Pause();
    Result Rewind();
    Result Forward();
    Result Stop();
    
    /* show/hide on-screen display */
    Result OSDKey();
    
    Result doCreate( string );
    Result doKill();
    Result doInitialize( string );
    Result doUninitialize();
    Result receiveMessage( string message );

    Result endTrack();
    Result setProgress(u_int64_t&, u_int64_t&);
    Result setTimeProgress(Time& current_media_time);
    Result trackDuration(Interval& duration);
    
    void setWinProperties( int nr, OSDElementComposit::win_properties_t* sadflk );
    
    /** play the given url 
    */
    Result playUrl( string );

  protected:

    WindowSelectorPlaylist* playlist_gui;
    /** state of nodes */
    enum playlistStatus { PLAYING, STOPPED, PAUSED };
    playlistStatus state;

    /** titel of actual playing track */
    string       playing_titel;

    /** x position of progress bar */
    unsigned int progress_x;

    /** y position of progress bar */
    unsigned int progress_y;

    /** indicates if composite node has video output */
    bool with_video;

    /** indicates if composite node has audio output */
    bool with_audio;

    /** mmbox rootpath */
    string       rootpath;

    /** duration of the current track (in seconds). */
    u_int64_t track_duration;

    TEDObject0<Playlist> endTrack_listener;
    TEDObject2<Playlist, u_int64_t, u_int64_t> setProgress_listener;
    TEDObject1<Playlist, Interval> trackDuration_listener;
    TEDObject1<Playlist, Time> setTimeProgress_listener;
    
    /** OSD manager node */
    INode* osd_node;
    
    /** OSD manager interface */
    IOSDManager* manager;
    
    /** audio node */
    INode* audio_node;
    
    /** display node */
    INode* display;

    /** indicates if window should be hidden */
    bool              hide_win;
    bool              show_play;

    /** indicates if track is played directly from track window */
    bool              play_single_url;
    GraphBuilder2     gb;
    CompositeNode*    cn;
    INode*            cn_node;
    ISeekable*        seeker;

    bool use_image_scaler;
    int image_rotation;

    void registerProgressListener();
    void deRegisterProgressListener();
    

    /* show on-screenn display */
    Result showOSD();

    /* hide on-screen display */
    Result hideOSD();
    
    /* stop all nodes */
    void stopNodes();
    
    /* flush all nodes */
    void flushAll();

    /* disconnect all nodes */
    void   disconnectNodes();

    /* delete composite node */
    void   deleteCnNode();

    /* reset on-screen display (reset progressbar information) */
    Result resetOSD();

    /* get the name of the file that stores the rotation of the current image file */
    const string getRotationFileName(const string& url);
  };
}


#endif //__PLAYLIST__
