/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef __SLIDESHOWSTATE_HPP__
#define __SLIDESHOWSTATE_HPP__

#include <vector>
#include <string>

#include "../../main/application.hpp"
#include "../../main/mmboxstate.hpp"
#include "nmm/utils/DirReader.hpp"
#include "nmm/interfaces/file/IFileHandler.hpp"
#include "nmm/base/registry/GraphDescription.hpp"


namespace NMM {
class SlideShowState : public MMBoxState {

public:
  SlideShowState( string, MMBoxApplication* );
  ~SlideShowState();

  /**
   * This method gets called when the 'down' button (down-arrow) is pressed:
   * for the slide show, this means: load the next image in the
   * directory
   */
  Result Down();

  /**
   * This method gets called when the 'up' button (up-arrow) is pressed:
   * for the slide show, this means: load the previous image in the
   * directory
   */
  Result Up();

  /**
   * This method gets called when the 'back' button (q) is pressed:
   * for the slide show, this means: simply call the goToLastState()
   */
  Result Back();

  /**
   * This method is called when the state is created ( selected in the
   * menu ), all NMM nodes are instantiated here and connected.
   */
  Result doCreate( string );

  /**
   * This method is called when the state is destroyed ( replaced by
   * another state, that uses the video background), all NMM nodes
   * are release her
   */
  Result doKill();

  /**
   * This method is called when the state is re-activated again 
   * ( after being put to background )
   */
  Result doInitialize( string );

  /**
   * This method is called when the state is put to background
   * Since no on-screen items are used in this simple implementation
   * nothing has to be done here.
   */
  Result doUninitialize();

protected:

  /**
   * Loads the next image
   */
  string loadNextImage(int jump_by);

  /**
   * Description of the used flow graph
   */
  GraphDescription graph_descr;

  /**
   * Handle to the own node for reading images
   */
  INode* read_node;

  /**
   * Handle to the own node that gets connected to the osd
   */
  INode* last_video_node;

  /**
   * The root path for reading images 
   * (so far, all images have to be in this path)
   */
  string rootpath;

  /**
   * Utility class for navigating in directories
   */
  DirReader dirreader;

  /**
   * Current index in current directory
   */
  int current_index;

  /**
   * Utility class that stores files in current directory
   */
  vector<DirReader::directory_t> dir;

  /**
   * Tells the state whether to use a video scaler for all 
   * pictures or not
   */
  bool use_scaler;

};
}

#endif
