/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002, 2003
 *                    Network-Multimedia Workgroup,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        David Maass (dama@graphics.cs.uni-sb.de) 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef __TRANSCODINGGROUP__
#define __TRANSCODINGGROUP__

#include "nmm/utils/gdparse/GDParse.hpp"
#include "nmm/utils/utils.hpp"
#include "nmm/base/sync/Clock.hpp"
#include "nmm/base/registry/GraphDescription.hpp"
#include "nmm/base/registry/ClientRegistry.hpp"
#include "apps/mmbox/main/application.hpp"
#include "apps/mmbox/main/TranscodeTemplateFunctions.hpp"
#include "TranscodingJob.hpp"

namespace NMM {
  class JobManager;

  class TranscodingGroup {
    
  public:
    TranscodingGroup(GraphDescription* , list<NodeConfiguration>* , const string& , ClientRegistry& , IPluginManager& , JobManager*,TranscodeTemplateFunctions* );
    ~TranscodingGroup();
    
    void addInputFiles(const vector<GraphURL>&);
    void addInputFileNames(const vector<string>&);
    void addInputFile(const string&);
    void addInputFileName(const string&);
    void setDescription(const string&);
    string getDescription();
    void setDistribute(const bool&);
    void setPriorize(const bool&);
    void setParallel(const int);
    void useRegistrys (const list<pair<string,int> >&);
    void useLocalhost(const bool&);
    void setWorkingDirectory (const string&);
    list<list<string> > getStatus();
    list<string> getJobStatus(const int&);
    list<string> getGroupStatus();
    void getJobProgress(const int& jobid, int& val, int& max);
    void setup();
    void processJobs();
    void deleteJob(const int&);
    void pauseJob(const int&);
    string mapFormat(const string&);

    void jobDone();
    bool allDone();

    void useScript(const string& script) {_script = script;}
   
    //methods forwarded to the TranscodingJob
    bool spymode(const int& i);
    Result spymodeRaw(const int& i, INode* node, const string& stream);
    Result spymodeTrans(const int& i, INode* node, const string& stream);
    void endSpymode(const int& i);

  protected:
    GraphDescription*           _graph;
    list<NodeConfiguration>*    _node_configuration;
    string                      _protocol;
    ClientRegistry&             _registry;
    IPluginManager&             _pm;
    JobManager*                 _jobmanager;
    TranscodeTemplateFunctions* _state_template;
    int                         _running_jobs, _jobs_done, _jobs_error, _jobs_sum;
    vector<string>              _input_files;
    vector<string>              _input_file_names;
    string                      _description;
    string                      _working_directory;
    bool                        _distribute;
    bool                        _priorize;
    bool                        _use_localhost;
    int                         _parallel;
    list<pair<string,int> >     _registry_list;
    vector<TranscodingJob*>     _transcoding_job_list;
    string                      _script;
    Clock                       _clock;
    NMM::Time                   _start_time, _end_time;
    ThreadMutex                 _mutex;

  };
}

#endif
