/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002, 2003
 *                    Network-Multimedia Workgroup,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        David Maass (dama@graphics.cs.uni-sb.de) 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef __TRANSCODINGJOB__
#define __TRANSCODINGJOB__

#include "nmm/utils/gdparse/GDParse.hpp"
#include "nmm/base/registry/GraphDescription.hpp"
#include "nmm/base/registry/ClientRegistry.hpp"
#include "../../main/application.hpp"
//#include "transcode.hpp"
#include "JobManager.hpp"

namespace NMM {
  class TranscodingGroup;

  class TranscodingJob {
    
    /**
     * this class is used to manage the running transcoding jobs in MMBox transcodingstate *
     */
  public:
    /**
     * constructor needs a URL to a GraphDescription, a Registry and a JobManager
     * \param _graph, the URL to the graphdescpription to be managed
     * \param _registry, the ClientRegistry to be used for loading plugins
     * \param _job, the JobManager used to handle eof Events
     */
    TranscodingJob(const string&, ClientRegistry&, IPluginManager&, JobManager*);
    ~TranscodingJob();

    /** indicates the status of the job **/
    enum Status { waiting, running, paused, done, error };
    enum Spymode { NONE, RAW, TRANS };

    void setGraphDescription(GraphDescription* gd) { _graph = new GraphDescription(*gd); }
    void setNodeConfiguration(list<NodeConfiguration>* l) {_node_configuration = new list<NodeConfiguration>(*l); }

    /** In/Output and Configuration handling **/
    void   setInputFile(const string& file) { input_file=file; }
    void   setFileName(const string& name) { file_name=name;}
    void   setWorkingDirectory(const string& dir) {working_directory = dir;}
    void   setDescription(const string& desc) {_description = desc;}
    void   setDistribute(const bool& i) {_distribute=i;}
    void   setPriorize(const bool& i) {_priori=i;}
    void   useRegistrys(const list< pair<string,int> >& reg_list);
    void   useLocalhost(const bool& use) {_use_localhost = use;}
    void   setTranscodingGroup(TranscodingGroup* g) {_group=g;}

    /** Return some status-information **/
    string getHost() {return opt_host;};
    string getActiveInputFile() { if (file_name!="")  return file_name; else return input_file.substr(input_file.find_last_of("/")+1,input_file.size()); }
    string getActiveOutputFile(){ return output_file.substr(output_file.find_last_of("/")+1,output_file.size()); }
    string getDescription();
    string getStatus();
    string getRemaining();
    void   getProgress(int&,int&);

    void pause();
    void eof();

    void useScript(const string& s) {_script=s;}
    
    /** configure Graphdescription, set Options, request Graph and start job **/
    Result setup();

    /** look how it's working hard ;) **/
    Result spymodeRaw(INode* node, const string& stream);
    Result spymodeTrans(INode* node, const string& stream);
    bool spymode() {return ((trans_video_node != 0) && (_status==running));}
    void endSpymode();

  protected:
    /** catch eof-Events here and tell JobManager to react **/
    Result eofHandler();
    Result progressHandler(u_int64_t &, u_int64_t &);

  private:
    GraphDescription* _graph;
    GraphDescription* _graph_copy;
    GraphDescription* _raw_graph;
    GraphDescription* _trans_graph;
    ClientRegistry& _registry;
    IPluginManager& _pm;
    int act_progress,max_progress;
    string _description;
    string input_file;
    string file_name;
    string output_file;
    string working_directory;
    string opt_host;
    Status _status;
    Spymode _spymode;
    list<NodeConfiguration>* _node_configuration;
    bool _distribute, _priori, _use_localhost;
    list< pair<string,int> > registry_list;
    JobManager* _jobmanager;
    TranscodingGroup* _group;
    string protocol;
    list<int> sink_list;
    unsigned int eof_counter;
    int first_video_edge,trans_video_edge;
    int first_video_node, trans_video_node;
    string first_video_tag, trans_video_tag;
    int id;
    string _script;
    Clock clock;
    NMM::Time _start_time;
    int _processing_time;
    bool raw_init,trans_init;
  };
}

#endif
