/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef MMBOX_TVTIMER_HPP
#define MMBOX_TVTIMER_HPP MMBOX_TVTIMER_HPP

// the app
#include "../../main/application.hpp"

// stored Interfaces
#include "nmm/interfaces/video/tv/ITVCard.hpp"
#include "nmm/interfaces/file/ITimeshifting.hpp"
#include "nmm/interfaces/video/osd/manager/ITextView.hpp"

#include "nmm/misc/dvb/channel.hpp"

#include "nmm/plugins/video/osd/manager/OSDElements/doublelistunit.hpp"
#include "nmm/plugins/video/osd/manager/OSDElements/listunit.hpp"

#include "nmm/interfaces/video/tv/TVChannelList.hpp"

#include "nmm/utils/thread/ThreadedObject.hpp"

#include "nmm/misc/tvguide/TVGuide.hpp"
#include "nmm/misc/tvtimer/TVTimerControl.hpp"

namespace NMM {

  
  /** TV Timer State.
   * MMBox state for programming a timer to record from the TV card.
   */

  class TvTimer 
    : public MMBoxState,
      public ThreadedObject {

  public:


    enum winType_e { LIST=0, TIMER, DETAILS, CONFLICT, CONFLICTDETAILS};

    typedef struct {
      unsigned int  xpos;
      unsigned int  ypos;
      unsigned int  width;
      unsigned int  height;
      unsigned int  win_active_color;
      unsigned int  win_inactive_color;
      unsigned int  sel_active_color;
      unsigned int  sel_active_color2;
      unsigned int  sel_inactive_color;
    } win_properties_t;

    void setWinProperties( int , win_properties_t* );

    typedef list<TVTimerEntry> rec_table_t;
  private:

    ITVTimer_var recorder;

    ThreadMutex mutex;

    unsigned int stringPos;
    int dvbChannel;

    char contentTmp;
    bool editing;
    bool timerChange;
    string timerLocation;

    bool displayRecorded;
    bool displaySleeping;
    bool displayRecording;

    string tz; //for queries to the TVGuide, our Timezone
    TVChannelList* channelConfig;

    TVTimerEntry conflictEntry;
  public:

    /**
     * constructor needs a string id that identifies the class
     * \param id the menu id string
     */
    TvTimer(string, MMBoxApplication*);
    ~TvTimer();

    Result Up();
    Result Down();
    Result Left();
    Result Right();
    Result Return();
    Result Back();
    Result Record();
    Result specialKey01();
    Result specialKey02();
    Result colorKeyRed();
    Result colorKeyGreen();
    Result colorKeyYellow();
    Result colorKeyBlue();    

    Result numKey0();
    Result numKey1(); 
    Result numKey2();
    Result numKey3();
    Result numKey4();
    Result numKey5();
    Result numKey6();
    Result numKey7();
    Result numKey8();
    Result numKey9();
    Result doCreate(string line = "1");
    Result doKill();
    Result doInitialize(string);
    Result doUninitialize();
    Result receiveMessage(string);

    Result programCurrentSelection(bool change)throw (const ActionNotAllowedException&);
    Result deleteTimer();

    int programTimer(ProgramDescription p);
    int programTimer(const TVTimerEntry &e);
    list<TVTimerEntry> getTimers(string day, string channel);

  protected:
    map<string, int>  month;
    map<string, int>  channel;
    bool              timer_load;

    TVChannelList     dvb;

    rec_table_t           rec_table;
    rec_table_t::iterator rec_cur;

    bool              timertext;
    bool              active;
    string            rootpath;

    winType_e         winStatus;

    /** properties of the windows */
    win_properties_t  winprops[3];


    DoubleListUnit*   timerList;
    ListUnit*   timerDetails;

    void updateHeader();
    /**
     * converts time strings to time_t value
     * \param time string of type hour:minute (example: 12:15)
     * \param day string that represents the day of month (1-31)
     * \param month string that represents the month of the year (January-December or 1-12)
     * \param year string that represents the year
     */
    time_t stringsToTime(string time, string day, string month, string year);

    /**
     * convert string to time_t value
     * \param s string of type year-month-day hour:month (example: 2002-12-01 12:15)
     */
    time_t stringToTime(string s);

    string TimeToString(time_t ltime);

    /**
     * convert time-String from format yyyymmddhhmm to dd-mm-yyyy hh:mm
     */
    string transformTime(string s);

    Result newTimer();
    Result nextTimer();
    Result previousTimer();

    TEDObject0<TvTimer> timerUpdate_listener;
    Result timerUpdate();

  private:
    void run();

    void updateColorKeyBar();
    
    void showOSD();
    void hideOSD();
    void connectTVTimerServer(string s);

    void numKey(string num);

    void deleteWindow();
    void createWindow(winType_e w, string h);
    
    void showTimerList(int markLine = 1);

    void showDetails();
    TCPAddress parseHostString(const string s);

  };

}

#endif //MMBOX_TVTIMER_HPP
