
#include "nmm/base/ProxyApplication.hpp"
#include "nmm/base/ProxyObject.hpp"

#include "nmm/interfaces/file/IFileHandler.hpp"
#include "nmm/interfaces/video/tv/IChannelInfo.hpp"

#include "nmm/utils/NMMConfig.hpp"
#include "nmm/utils/utils.hpp"

#include "nmm/misc/tvmanager/TVManager.hpp"

#include "nmm/base/utils/GraphURL.hpp"

#include "nmm/base/serialize/StdStream.hpp"
#include "nmm/base/serialize/MagicNumberValueStream.hpp"

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <getopt.h>
#include <iostream>
#include <fstream>
#include <sstream>

#include <map>

using namespace NMM;


int usage( char *name ) {
  cerr << endl
       << "Usage:" << endl
       << name << " [options] [filename]" << endl
       << endl
       << "    -f --file            set an output-file (default: $HOME/.nmm/tv_channels" << endl
       << "    -c --config          set mmboxrc" << endl
       << "    -v --verbose         increase verbosity (may be given multiple times)" << endl
       << "    -h --help            Shows this help and exits." << endl;
  return 1;
};

int main(int argc, char ** argv) {
  /* Global options */
  string fileURL;
  unsigned int verbose = 0;
  NMMConfig *config = NULL;
  string filename = (string(getenv("HOME")) + "/.nmm/tv_channels");

  const char *optstring = "f:c:vh";
  struct option longopts[] = {
    {"file",           1, NULL, 'f'},
    {"config",         1, NULL, 'c'},
    {"verbose",        0, NULL, 'v'},
    {"help",           0, NULL, 'h'},
    {NULL,             0, NULL, 0  } /* last row must be zero */
  };

  int ch, optionindex;
  while(EOF != (ch=getopt_long(argc,argv,optstring,longopts,&optionindex))){
    // which option did we get?
    switch (ch) {
    case 'f':
      filename = string(optarg);
      break;
    case 'c':
      config = new NMMConfig(optarg);
      break;
    case 'v':
      verbose++;
      break;
    default: //includes 'h'
      return usage(argv[0]);
    }
  }

  /* handle *Stream output */
  if (verbose < 1) {
    NamedObject::getGlobalInstance().setErrorStream(0, NamedObject::ALL_LEVELS);
  }
  if (verbose < 2) {
    NamedObject::getGlobalInstance().setWarningStream(0, NamedObject::ALL_LEVELS);
  }
  if (verbose < 3) {
    NamedObject::getGlobalInstance().setMessageStream(0, NamedObject::ALL_LEVELS);
  }
  if (verbose < 4) {
    NamedObject::getGlobalInstance().setDebugStream(0, NamedObject::ALL_LEVELS);
  }


  try{

    NMMApplication *app = ProxyApplication::getApplication(argc, argv);

    if(config == NULL) {
      string path = string(getenv("HOME")) + "/.mmboxrc";
      config = new NMMConfig(path);
    }
    int tvhostNumber = 0;
    TVChannelList mmboxlist;

    IChannelInfo_var ci(NULL);

    if(config->getValue("tvhosts_number" , tvhostNumber)){

      for(int i = 0; i < tvhostNumber ; i++){

        GraphDescription gd;
        string tvHostURL;
        
        //cerr << "checking " << "tvhost_" << i << endl;

        if(config->getValue("tvhost_" + itoa(i), tvHostURL)){
          
          GraphURL url(tvHostURL);
          
          NodeDescription dvb;
          
          if(url.getProtocol() == GraphURL::FILE){
            dvb = url.getNodeDescription();
            dvb.setNodeName("DVBSimulatorNode");
            dvb.addNodeEvent(Event(IFileHandler::setFilename_event, new TInValue<string>(url.getURIPath())), INode::CONSTRUCTED);
            dvb.setSharingType(NodeDescription::SHARED_OR_EXCLUSIVE_THEN_SHARED);

          }else{
            dvb = url.getNodeDescription();
          }
        
          if(!url.getOption("channelConfig").empty()){
            dvb.addNodeEvent(Event(IChannelInfo::setChannelConfigFile_event,
                                   new TInValue<string>(url.getOption("channelConfig"))), INode::CONSTRUCTED);
          }
          
          NodeDescription devnull("DevNullNode");
          devnull.setSharingType(NodeDescription::EXCLUSIVE);

          gd.addEdge(dvb, devnull);
          
          try{
            app->getRegistry().requestGraph(gd);
            
            gd.realizeGraph();
          }catch(const Exception &e){
            cerr << "requesting failed " << e << endl;
            continue;
          }
          //read the channels
          
          
          ci.reset(gd.getINode(dvb)->getParentObject()->getCheckedInterface<IChannelInfo>());
          if(ci.get()){
            
            TVChannelList l = ci->getChannelList();
            list<TVChannel> c = l.getChannels();
            for(list<TVChannel>::iterator i = c.begin(); i != c.end(); i++){
              TVChannel chan;
              try{
                chan = mmboxlist.getChannelByName(i->getChannelName());              
              }catch(const ChannelNotFoundException &e){
                bool channelFound = false;
                list<string> channelAliases = i->getChannelAliases();
                for(list<string>::iterator j = channelAliases.begin(); j != channelAliases.end(); j++){
                  try{
                    chan = mmboxlist.getChannelByName(*j);
                    channelFound = true;
                    chan.addAlias(i->getChannelName());
                    break;
                  }catch(const Exception &e){
                    continue;
                  }
                }
                if(!channelFound)
                  chan = TVChannel(i->getChannelName());
              }
              
              chan.addCard(tvHostURL, i->getChannelNumber());
              list<string> channelAliases = i->getChannelAliases();
              for(list<string>::iterator j = channelAliases.begin(); j != channelAliases.end(); j++){
                chan.addAlias(*j);
              }
              mmboxlist.addChannel(chan);
            }
            
          }else{
            cerr << "CANT GET CHANNELINFO" << endl;
          }
        }else{
          cerr << "all tv_hosts parsed" << endl;
          
          try{
            app->getRegistry().releaseGraph(gd);
          }catch(const Exception &e){
          }
          
          break;
        }
        
        try{
          app->getRegistry().releaseGraph(gd);
        }catch(const Exception &e){
        }

      }
    }

    mmboxlist.write(filename);

  }catch(const Exception &e){
    cerr << "Error: " << e << endl;
  }catch(...){
    cerr << "error!!!" << endl;
  }
  
  return 0;
}
