/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        David Maass <dama@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "nmm/interfaces/device/dvd/IDVD.hpp"

#include "WindowSelectorTranscode.hpp"

#include "nmm/base/NMMApplication.hpp"
#include "nmm/base/ProxyObject.hpp"

#include "nmm/utils/utils.hpp"

namespace NMM {
  WindowSelectorTranscode::WindowSelectorTranscode(IOSDManager* manager, IWidgetFactory* factory, PlaylistTemplateFunctions* state):
    WindowSelectorPlaylist(manager, factory, state)
  {
    playlistfile="mmbox_transcode.pls";
  }

  WindowSelectorTranscode::~WindowSelectorTranscode()
  {}

  Result WindowSelectorTranscode::colorKeyGreen() {
    if (WindowSelectorPlaylist::colorKeyGreen() == FAILURE) {
      if ((focus == 1) && (prefix == "dvd://")) {
        unsigned int nr = track->getTextWidget()->getSelectedLine();
        string url(prefix + string("?title=") + ((int) nr) + "&chapter=1&angle=1");
        playlist->getTextWidget()->insertLine( playlist->getTextWidget()->getSelectedLine(), track->getTextWidget()->getLineContent(nr), url );
        playlist->getTextWidget()->selectDown();
      }
      elements[focus]->update();
      manager->update();
    }
    return SUCCESS;
  }
  
  Result WindowSelectorTranscode::colorKeyBlue() {
    select();
    return SUCCESS;
  }

  void WindowSelectorTranscode::createWindows() {
    WindowSelectorPlaylist::createWindows();
    selection->addLine("DVD");
  }

  Result WindowSelectorTranscode::select() {
    if (WindowSelectorPlaylist::select() == SUCCESS) {
      return SUCCESS;
    }
    else      
      if ((focus == 0) && (selection->getTextWidget()->getLineContent(selection->getTextWidget()->getSelectedLine()) == "DVD")) {
      prefix = "dvd://";
      track->getTextWidget()->deleteAllLines();
      
      if (app) {
        try {
          ClientRegistry& registry = app->getNMMApplication()->getRegistry();
          INode_var dvd;
          {
            RegistryLock lock(registry);
            list<Response> dvd_response = registry.initRequest(NodeDescription("DVDReadNode"));
            
            if (dvd_response.empty()) {
              GLOBAL_ERROR_STREAM("Size of dvd_response " << dvd_response.size() << endl);
              throw Exception(string("A required node is not available\n"));
            }
            
            dvd.reset(registry.requestNode(dvd_response.front()));
          }
          
          IDVD_var dvdi(dvd->getParentObject()->getInterface<IDVD>());          
          
          dvd->reachActivated();
          
          // get number of titles and number of chapters for each title
          int num_titles = dvdi->getTitles();
          track->deleteAllLines();
          for(int i=1; i<=num_titles; i++) {
            int num =  dvdi->getChapters(i);
            track->addLine(string("Title ") + i + " (" + num + " chapters)");
          }
          
        }catch (const Exception& e) {
          manager->displayMessage("Error","Could not find a valid DVD","OK");
          state->doChangeState("OSDInputState");
          manager->update();
          return FAILURE;
        }
        track->getTextWidget()->selectLine( 1 );
        selectWindow(1); //select Track-Window
      }
      } else if ((focus == 1) && (prefix=="dvd://")) {
        unsigned int nr = track->getTextWidget()->getSelectedLine();
        actual_file = track->getTextWidget()->getLineContent(nr);
        return state->doSelectTrackWindow(prefix + string("?title=") + ((int) nr) + "&chapter=1&angle=1");
      }
    return SUCCESS;
  }

  void WindowSelectorTranscode::updateColorBar() {
    switch (focus) {
    case 0: {
      if (selection) {
        string sel = selection->getTextWidget()->getLineContent(selection->getTextWidget()->getSelectedLine() );
        if (sel == "DataCD")
          manager->setColorBarText(mounted ? "Unmount" : "Mount", "", "", "");
        else 
          manager->setColorBarText("", "", "", "");
      }
      else 
        manager->setColorBarText("", "", "", "");
      break;
    }
    case 1:
      if (prefix == "file://" && track->getTextWidget()->getSelectedLine()) {
        if (!dir[ track->getTextWidget()->getSelectedLine() -1 ].isdir)
          manager->setColorBarText("Delete", "Add", "", "Transcode");
        else 
          manager->setColorBarText("Delete", "Add", "", "Open dir");
      }
      else if (prefix =="audiocd://" && track->getTextWidget()->getSelectedLine()) {
        manager->setColorBarText("","Add","","Transcode");
      }
      else {
	manager->setColorBarText("", "", "", "Transcode");
      }
      break;
    case 2:
      if (playlist->getTextWidget()->getSelectedLine()) {
	if (playlist->getTextWidget()->getSelectedLine() == playlist->getTextWidget()->getSelectedLine2()) {
	  manager->setColorBarText("", playlist_mode == NORMAL ? "Drag" : "Drop", "Del. all", "Trans. all");
	}
	else {
	  manager->setColorBarText("Delete", playlist_mode == NORMAL ? "Drag" : "Drop", "Del. all", "Trans. all");
	}
      }
      else {
	manager->setColorBarText("", "", "", "");
      }
      break;
    default:
      manager->setColorBarText("", "", "", "");
    }
    manager->update();
  }

}
