/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de> 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "DistMMBoxState.hpp"
#include "application.hpp"
#include "nmm/base/registry/ClientRegistry.hpp"
#include "nmm/base/NMMApplication.hpp"

namespace NMM {
  DistMMBoxState::DistMMBoxState( string id, MMBoxApplication* app) : MMBoxState(id, app)
  {
  } 
 
  DistMMBoxState::~DistMMBoxState()
  {
  } 

  Result DistMMBoxState::Back() 
  {
    deactivate();
    return goToLastState();
  }
  
  Result DistMMBoxState::doInitialize( string ) 
  {
    doCreate("");
    return SUCCESS;
  }

  Result DistMMBoxState::doUninitialize() 
  {
    doKill();
    return SUCCESS;
  }

  Result DistMMBoxState::initialize(const string& message)
  {
    return MMBoxState::initialize(message);
  }

  Result DistMMBoxState::uninitialize()
  {
    return MMBoxState::uninitialize();
  }

  Result DistMMBoxState::create(const string& message)
  {
    return MMBoxState::create(message);
  }

  Result DistMMBoxState::kill()
  {
    return MMBoxState::kill();
  }

  INode* DistMMBoxState::createNode(const NodeDescription& ndescr)
  {
    GLOBAL_DEBUG_STREAM("DistMMBoxState: createNode" << endl);
    /*Begin> Get the ClientRegistry and request Nodes */
    ClientRegistry& registry = app->getNMMApplication()->getRegistry();
    {
      RegistryLock lock(registry);

      list<Response> response = registry.initRequest(ndescr);
      if(response.empty()) {
	GLOBAL_ERROR_STREAM("DistMMBoxState: Cannot create wanted node" << endl);
	return 0;
      }
      else {
	INode* n = registry.requestNode(response.front());
	GLOBAL_DEBUG_STREAM("DistMMBoxState: got node " << n << endl);
	return n;
      } 
    }
  }

  INode* DistMMBoxState::getAudioNode()
  {
    return app->getAudioNode();
  }

  INode* DistMMBoxState::getOsdNode()
  {
    return app->getOsdNode();
  }

  INode* DistMMBoxState::getDisplayNode()
  {
    return app->getDisplayNode();
  }

  void DistMMBoxState::setNeedVideo(const bool& b)
  {
    MMBoxState::setNeedVideo(b);
  }

  void DistMMBoxState::setNeedAudio(const bool& b)
  {
    MMBoxState::setNeedAudio(b);
  }


}
