/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef MMBOXSTATE_HPP
#define MMBOXSTATE_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/registry/GraphDescription.hpp"

namespace NMM {

  class MMBoxApplication;
  class IMMBoxApplication;

  /**
   * the methods of these class are virtual, all MMBox applications like dvdplayer...
   * must be derived from it, so that the MMBoxApplication class can control the function
   * calls depending on the current state
   */
  class MMBoxState {
  private:
    struct ltstr
    {
      bool operator()(const pair<string, string>& s1, const pair<string, string>& s2) const
      {
	int i = strcmp(s1.first.c_str(), s2.first.c_str());
	if(i == 0) {
	  return (strcmp(s1.second.c_str(), s2.second.c_str()) < 0);
	}
	else {
	  return (i < 0);
	}
      }
    };

    
    typedef map< pair<string, string>, pair<GraphDescription, IMMBoxApplication*>, ltstr > SharedStateMap_t;
    SharedStateMap_t shared_states;

  protected:
    MMBoxApplication* app;
    Result changeState(string stateid, string message = "");

    /* here we save our own id, should be done in the constructor */
    string stateid;

    /* needed to jump back to the last state */
    string lastid;

    /* filename of background image, empty if no background needed */
    string background;

    /* indicates if the state needs a background */
    bool lazy_background;

    /* indicates if the state needs a audio node */
    bool need_audio;

    /* indicates if we can connect to the osd node (stream1) */
    bool background_free;

    /* indicates if we can connect to the audio node */
    bool audio_free;

    /* indicates that state has input/view focus */
    bool has_focus;

    /* current graph description for adding remote hosts to state */
    GraphDescription current_graph_description;

    string session_location;

  public:
    MMBoxState(string, MMBoxApplication* app);
    virtual ~MMBoxState();

    virtual Result Up();
    virtual Result Down();
    virtual Result Left();
    virtual Result Right();
    virtual Result Return();
    virtual Result Back();
    virtual Result Pause();
    virtual Result Forward();
    virtual Result Rewind();
    virtual Result Record();
    virtual Result Stop();
    virtual Result OSDKey();
    virtual Result specialKey01();
    virtual Result specialKey02();
    virtual Result colorKeyRed();
    virtual Result colorKeyGreen();
    virtual Result colorKeyYellow();
    virtual Result colorKeyBlue();
    virtual Result numKey0();
    virtual Result numKey1();
    virtual Result numKey2();
    virtual Result numKey3();
    virtual Result numKey4();
    virtual Result numKey5();
    virtual Result numKey6();
    virtual Result numKey7();
    virtual Result numKey8();
    virtual Result numKey9();
    virtual Result REVEAL();
    virtual Result receiveMessage(string);
    virtual Result doInitialize(string);
    virtual Result doUninitialize();
    virtual Result doCreate(string);
    virtual Result doKill();

    const GraphDescription& getCurrentGraphDescription() const;
    void setCurrentGraphDescription(const GraphDescription& current_graph_description, string session = "");
    bool hasCurrentGraphDescription();
    
    bool hasSharedState(const string& host, const string& state) const;
    GraphDescription getSharedGraphDescription(const string& host, const string& state) const;
    void addSharedState(const string& host, const string& state,
			const GraphDescription& graph, IMMBoxApplication* dist_mmbox = 0);
    Result releaseSharedState(const string& host, const string& state);

    Result initialize(string message = "");
    Result uninitialize();
    Result create(string message = "");
    Result kill();
    Result goToLastState(string message = "");

    /* deactivate actual state */
    void deactivate();

    /* set background file */
    void setBackground(string);

    /* get background file */
    string getBackground();

    /* indicates if the state needs a background */
    void setLazyBackground(bool);

    /* indicates if the state needs a background */
    void setNeedVideo(bool);

    /* indicates if the state needs a audio node */
    void setNeedAudio(bool);

    /* check if state needs background */
    bool isLazyBackground();

    /* check if state needs background */
    bool isNeedVideo();

    /* check if state needs audio node */
    bool isNeedAudio();

    void setBackgroundFree(bool);
    void setAudioFree(bool);
    bool isBackgroundFree();
    bool isAudioFree();

    /* set focus */
    void setFocus(bool);

    /* check if state has focus */
    bool hasFocus();

    /**
     * \returns state identification string
     */
    string getID();

    virtual string getSessionLocation();

  };

}

#endif //MMBOXSTATE_HPP
