/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "qgraphlist.h"
#include "qgraphwidget.h"
#include "nmm/interfaces/base/INode.hpp"

#include <qlayout.h>
#include <qlistview.h>
#include <qpopupmenu.h>

QGraphList::QGraphList(QWidget *parent, const char *name , const std::list<QString>& header_list)
  : QWidget(parent,name), __header_list(header_list)
{
  allocate();
  updateView();
  initConnect();
}

QGraphList::~QGraphList(){

}

void  QGraphList::updateView() {
  __list_view->clear();
  __graph_map.clear();
  __node_map.clear();

  for (InternalIt it = getWidgetList().begin(); it != getWidgetList().end(); ++it) {
    QListViewItem*  item = new QListViewItem(__list_view);
    int i = 0;
    for (list<QString>::const_iterator int_it = (*it)->getHeader().begin(); int_it !=(*it)->getHeader().end(); ++int_it, ++i) {
      item->setText(i, *int_it);
    }
    __graph_map.insert(pair<QListViewItem*, QGraphWidget*> (item, *it));
    updateGraphWidget(item, *it);
  }
}

void QGraphList::updateGraphWidget(QListViewItem* item, QGraphWidget* widget) {
  for (list<NMM::INode*>::iterator it = widget->getNodeList().begin(); it != widget->getNodeList().end(); ++it) {
    QListViewItem* child_item = new QListViewItem(item, (*it)->getInterfaceType().c_str());
    __node_map.insert(pair<QListViewItem*, NMM::INode*> (child_item, *it));
  }
}

void  QGraphList::allocate() {
  __top_layout = new QGridLayout(this, 3, 3);
  __list_view  = new QListView(this);
  for (std::list<QString>::iterator it = __header_list.begin(); it != __header_list.end(); ++it)
    __list_view->addColumn(*it);

  __top_layout->addWidget(__list_view, 0, 0);

  __popup_menu = new QPopupMenu(this);
}

void  QGraphList::deallocate() {
  for (InternalIt it = getWidgetList().begin(); it != getWidgetList().end(); ++it)
    delete *it;
}

void  QGraphList::initConnect() {
  connect(__list_view, SIGNAL(rightButtonClicked( QListViewItem *, const QPoint &, int )), this, SLOT(showGraphMenu(QListViewItem *, const QPoint &, int )));
  connect(__list_view, SIGNAL(clicked(QListViewItem*)), this, SLOT(emitClicked(QListViewItem*)));
}

void QGraphList::emitClicked(QListViewItem* item) {
  if (!item) {
    return;
  }
  map<QListViewItem*, QGraphWidget*>::iterator it = __graph_map.find(item);
  if (it != __graph_map.end()) {
    emit clicked(it->second);
    return;
  }
  map<QListViewItem*, NMM::INode*>::iterator it2 = __node_map.find(item);
  if (it2 != __node_map.end()) {
    emit clicked(it2->second);
    return;
  }
}

void QGraphList::showGraphMenu(QListViewItem* , const QPoint & pos, int ) {
  __popup_menu->popup(pos);
}
