/*
 * Qwt Widget Library (http://qwt.sf.net)
 * Copyright (C) 1997   Josef Wilgen
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, 
 * USA.
 */


#ifndef QWT_SCLDIV_H
#define QWT_SCLDIV_H

#include <qmemarray.h>

class QwtScaleDiv
{
private:
    
    double d_lBound;
    double d_hBound;
    double d_majStep;
    bool d_log;


    QMemArray<double> d_majMarks;
    QMemArray<double> d_minMarks;

    void copy(const QwtScaleDiv &s);
    
public:

    QwtScaleDiv ();
    virtual ~QwtScaleDiv();
    QwtScaleDiv(const QwtScaleDiv& s);

    QwtScaleDiv& operator= (const QwtScaleDiv &s);
    int operator== (const QwtScaleDiv &s) const;
    int operator!= (const QwtScaleDiv &s) const;
    
    double lBound() const { return d_lBound; }
    double hBound() const { return d_hBound; }
    int minCnt() const { return d_minMarks.size(); }
    int majCnt() const { return d_majMarks.size(); }
    bool logScale() const { return d_log; }
    double majMark(int i) const { return d_majMarks[i]; }
    double minMark(int i) const { return d_minMarks[i]; }
    double majStep() const { return d_majStep; }
    void reset();

private:

    bool buildLinDiv(int maxMajMark, int maxMinMark, double step = 0.0);
    bool buildLogDiv(int maxMajMark, int maxMinMark, double step = 0.0);

public:

    bool rebuild(double lBound, double hBound, int maxMaj, int maxMin,
			 bool log, double step = 0.0, bool ascend = true);
    
};

#endif
