/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christian Gerstner <gerstner@graphics.cs.uni-sb.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef _NMM_VCORE_GUIWIDGET_HPP_
#define _NMM_VCORE_GUIWIDGET_HPP_ _NMM_VCORE_GUIWIDGET_HPP_

#include <qscrollbar.h>
#include <qlcdnumber.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qmainwindow.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include "MyLineEdit.hpp"

namespace NMM {
  class NMMApplication;
  class MasterSlaveSourceSynchronizer;
  class ClientRegistry;
}

class GuiWidget : public QMainWindow
{
  Q_OBJECT
private:
  NMM::NMMApplication& app;

  QLabel* labelOutFileName;
  QLabel* labelVNCHostAndPort;
  QLabel* labelVNCPassword;  
  QLabel* labelVideoBitrate;
  QLabel* labelAudioBitrate;
  QLabel* labelWidth;
  QLabel* labelHeight;

  QLineEdit* editOutFileName;
  QLineEdit* editVNCHostAndPort;
  QLineEdit* editVNCPassword;
    
  MyLineEdit* lcdVBR;

  QPushButton* outFileName;
  QPushButton* start;
  QPushButton* stop;
  QPushButton* preview;
  QPushButton* quit;

  QCheckBox* checkBoxDisplay;
  QCheckBox* checkBoxVisca;
  QCheckBox* checkBoxDeinterlacer;

  QSpinBox* widthBox;
  QSpinBox* heightBox;

  QScrollBar* sliderVBR;
  QScrollBar* sliderABR;
public:
  /**
   * The main widget for the application holding all needed
   * buttons, editlines, sliders, switchboxes, ...
   */
  GuiWidget(NMM::NMMApplication& app_);
  ~GuiWidget();

  /**
   * set value of outputfilename and VNC Host&Port
   */
  void setDefaultValues(QString, QString);

  /**
   * provide a handle to the start button, so it can be hidden from the outside
   * (if the start button is pressed, it is hidden and the stop button is shown)
   */
  QPushButton* getButtonStart();
  /**
   * provide a handle to the stop button, so it can be hidden from the outside
   * (if the stop button is pressed, it is hidden and the start button is shown)
   */
  QPushButton* getButtonStop();
  /**
   * provide a handle to the preview button to e.g. connect its signals somewhere outside
   */
  QPushButton* getButtonPreview();
  /**
   * provide a handle to the "..." button to e.g. connect its signals somewhere outside
   */
  QPushButton* getButtonOutFileName();
  /**
   * provide a handle to the quit button to e.g. connect its signals somewhere outside
   */
  QPushButton* getButtonQuit();
  /**
   * provide a handle to the display checkbox to e.g. connect its signals somewhere outside
   */
  QCheckBox* getCheckBoxDisplay();
  /**
   * provide a handle to the VISCA checkbox to e.g. connect its signals somewhere outside
   */
  QCheckBox* getCheckBoxVisca();
  /**
   * provide a handle to the deinterlacer checkbox to e.g. connect its signals somewhere outside
   */
  QCheckBox* getCheckBoxDeinterlacer();
  /**
   * provide a handle to the editline of the output filename to e.g. connect its signals somewhere outside
   */
  QLineEdit* getEditOutFileName();
  /**
   * provide a handle to the editline of the VNC host and port to e.g. connect its signals somewhere outside
   */
  QLineEdit* getEditVNCHostAndPort();
  /**
   * provide a handle to the VNC password field to e.g. connect its signals somewhere outside
   */
  QLineEdit* getEditVNCPassword();
  /**
   * provide a handle to the spinbox for the width to e.g. connect its signals somewhere outside
   */
  QSpinBox* getWidthBox();
  /**
   * provide a handle to the spinbox for the height to e.g. connect its signals somewhere outside
   */
  QSpinBox* getHeightBox();
  /**
   * provide a handle to the slider of the video bitrate to e.g. connect its signals somewhere outside
   */
  QScrollBar* getSliderVBR();
  /**
   * provide a handle to the slider of the audio bitrate to e.g. connect its signals somewhere outside
   */
  QScrollBar* getSliderABR();
};
#endif //_NMM_VCORE_GUIWIDGET_HPP_
