/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christian Gerstner <gerstner@graphics.cs.uni-sb.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "MDIWindow.hpp"
#include <qtoolbutton.h>
#include <qcombobox.h>
#include <qbuttongroup.h>
#include <iostream>

#include "./icons/backward.xpm"
#include "./icons/black.xpm"
#include "./icons/blue.xpm"
#include "./icons/eraser.xpm"
#include "./icons/forward.xpm"
#include "./icons/freehand.xpm"
#include "./icons/green.xpm"
#include "./icons/custom.xpm"
#include "./icons/line.xpm"
#include "./icons/pointer.xpm"
#include "./icons/rectangle.xpm"
#include "./icons/red.xpm"

using namespace std;

MDIWindow::MDIWindow(NMM::NMMApplication& app_, QWidget *parent, const char* name, WFlags f) : 
  QMainWindow(parent, name, f),
  app(app_),
  workspace(NULL){

  workspace = new QWorkspace(this,"wspace");
  workspace->setGeometry(1,1,800,600);
  setCentralWidget(workspace);
  setFocusPolicy(StrongFocus);
  vcoremode = new QToolBar( this, "VCORE-Mode" );
  vcoremode->setLabel( "Mode" );

  mode = new QComboBox( FALSE, vcoremode, "Mode" ); 
  mode->insertItem("annotate");
  mode->insertItem("present");
  mode->insertItem("write");
  mode->insertItem("prepare");

  vncmenu = new QToolBar( this, "SVG-Controls" );
  vncmenu->setLabel( "SVG-Controls" );

  
  
  tbundo = new QToolButton( QIconSet(QPixmap((const char **)backward_xpm)), QString("Undo"), QString::null,
					this, SLOT(doNothing()), vncmenu, "Undo" );
  tbredo = new QToolButton( QIconSet(QPixmap((const char **)forward_xpm)), QString("Redo"), QString::null,
					this, SLOT(doNothing()), vncmenu, "Redo" );
  QIconSet * clearicon = new QIconSet(QPixmap((const char **)eraser_xpm));
  tbclear = new QToolButton( *clearicon, QString("Clear"), QString::null,
					 this, SLOT(doNothing()), vncmenu, "Clear" );
  
  vncmenu->addSeparator();
  drawtypegroup = new QButtonGroup(vncmenu);
  drawtypegroup->setExclusive(true);
  drawtypegroup->hide();
  tbpointer = new QToolButton( QIconSet(QPixmap((const char **)pointer_xpm)), QString("Pointer"), QString::null,
					this, SLOT(doNothing()), vncmenu, "Pointer" );
    tbfreehand = new QToolButton( QIconSet(QPixmap((const char **)freehand_xpm)), QString("Freehand"), QString::null,
					 this, SLOT(doNothing()), vncmenu, "Freehand" );
    tbrectangle = new QToolButton( QIconSet(QPixmap((const char **)rectangle_xpm)), QString("Rectangle"), QString::null,
				       this, SLOT(doNothing()), vncmenu, "Rectangle" );
    tbline = new QToolButton( QIconSet(QPixmap((const char **)line_xpm)), QString("Line"), QString::null,
					 this, SLOT(doNothing()), vncmenu, "Line" );
    
  tbpointer->setToggleButton(true);
  tbpointer->setOn(true);
  tbfreehand->setToggleButton(true);
  tbrectangle->setToggleButton(true);
  tbline->setToggleButton(true);
  drawtypegroup->insert(tbpointer);
  drawtypegroup->insert(tbfreehand);
  drawtypegroup->insert(tbrectangle);
  drawtypegroup->insert(tbline);  

  vncmenu->addSeparator();
  colorgroup = new QButtonGroup(vncmenu);
  colorgroup->setExclusive(true);
  colorgroup->hide();
  tbblue = new QToolButton( QIconSet(QPixmap((const char **)blue_xpm)), QString("Blue"), QString::null,
					this, SLOT(doNothing()), vncmenu, "Blue" );
  tbgreen = new QToolButton( QIconSet(QPixmap((const char **)green_xpm)), QString("Green"), QString::null,
					 this, SLOT(doNothing()), vncmenu, "Green" );
  tbred = new QToolButton( QIconSet(QPixmap((const char **)red_xpm)), QString("Red"), QString::null,
				       this, SLOT(doNothing()), vncmenu, "Red" );
  tbblack = new QToolButton( QIconSet(QPixmap((const char **)black_xpm)), QString("Black"), QString::null,
					 this, SLOT(doNothing()), vncmenu, "Black" );
  tbcustom = new QToolButton( QIconSet(QPixmap((const char **)custom_xpm)), QString("Custom Colors"), QString::null,
					 this, SLOT(doNothing()), vncmenu, "Custom Colors" );
  
  tbblue->setToggleButton(true);
  tbblue->setOn(true);
  tbblue->setDisabled(true);
  tbgreen->setToggleButton(true);
  tbgreen->setDisabled(true);
  tbred->setToggleButton(true);
  tbred->setDisabled(true);
  tbblack->setToggleButton(true);
  tbblack->setDisabled(true);
  tbcustom->setDisabled(true);
  colorgroup->insert(tbblue);
  colorgroup->insert(tbgreen);
  colorgroup->insert(tbred);
  colorgroup->insert(tbblack);
  colorgroup->insert(tbcustom);
}

MDIWindow::~MDIWindow() {
    if(workspace) delete workspace;
    if(mode) delete mode;
    if(vcoremode) delete vcoremode;
    if(tbundo) delete tbundo;
    if(tbredo) delete tbredo;
    if(tbclear) delete tbclear;
    if(colorgroup) delete colorgroup;
    if(tbblue) delete tbblue;
    if(tbgreen) delete tbgreen;
    if(tbred) delete tbred;
    if(tbblack) delete tbblack;
    if(drawtypegroup) delete drawtypegroup;
    if(tbpointer) delete tbpointer;
    if(tbfreehand) delete tbfreehand;
    if(tbrectangle) delete tbrectangle;
    if(tbline) delete tbline; 
    if(vncmenu) delete vncmenu;
}

QToolBar* MDIWindow::getToolBarVNCMenu(){
  return vncmenu;
}

QButtonGroup* MDIWindow::getButtonGroupColor(){
  return colorgroup;
}

QButtonGroup* MDIWindow::getButtonGroupDrawType(){
  return drawtypegroup;
}

QComboBox* MDIWindow::getComboBoxMode(){
  return mode;
}

QToolButton* MDIWindow::getToolButtonBlue(){
  return tbblue;
}

QToolButton* MDIWindow::getToolButtonRed(){
  return tbred;
}

QToolButton* MDIWindow::getToolButtonGreen(){
  return tbgreen;
}

QToolButton* MDIWindow::getToolButtonBlack(){
  return tbblack;
}

QToolButton* MDIWindow::getToolButtonCustom(){
  return tbcustom;
}

QToolButton* MDIWindow::getToolButtonUndo(){
  return tbundo;
}

QToolButton* MDIWindow::getToolButtonRedo(){
  return tbredo;
}

QToolButton* MDIWindow::getToolButtonClear(){
  return tbclear;
}

void MDIWindow::closeEvent( QCloseEvent* ce ){      
  ce->accept();
  emit mdiWindowClosed();
}
