/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "InterfaceFactory.hpp"

namespace NMM {

void InterfaceFactory::registerInterface(const IFactoryObject& factory_object, const string& interface_type) {
  //  DEBUG_STREAM("Register Interface " << interface_type << endl);
  InternalIt it = __factory_map.find(interface_type);
  if (it != __factory_map.end()) {
    ERROR_STREAM("An interface with type " << interface_type << " is already registered" << endl);
    return;
  }

  __factory_map.insert(pair<string, IFactoryObject*>(interface_type, factory_object.copy()));
}

Interface* InterfaceFactory::createInterface(const string& interface_type, NMMObject& receiver) {
  InternalIt it = __factory_map.find(interface_type);
  if (it == __factory_map.end())  {
    ERROR_STREAM("An interface with type " << interface_type << " is not available" << endl);
    return 0;
  }

  return it->second->createInstance(receiver);
}

bool InterfaceFactory::isInterfaceAvailable(const string& i_type) const {
  if (__factory_map.find(i_type) == __factory_map.end())
    return false;

  return true;
}

InterfaceFactory::~InterfaceFactory() {
  for (InternalIt it = __factory_map.begin(); it != __factory_map.end(); ++it)
    delete it->second;

  __factory_map.clear();
}


}//end namespace
