/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "ProxyApplication.hpp"
#include "nmm/base/proxy/tcp/TCPStrategy.hpp"
#include "nmm/utils/net/NetReactor.hpp"
#include "nmm/utils/utils.hpp"
#include "nmm/base/registry/PluginManager.hpp"
#include "nmm/base/registry/Registry.hpp"
#include "nmm/interfaces/net/IObjectAcceptor.hpp"
#include "nmm/base/proxy/tcp/TCPAcceptor.hpp"
#include "nmm/base/ProxyObject.hpp"

#ifdef ARCH_ARM
#include "nmm/ext/base/registry/StaticServerRegistry.hpp"
#endif

namespace NMM {

  ProxyApplication::ProxyApplication(int argc, char** argv, string dist_hostname, int port, bool enable_acceptor)
  : NMMApplication (argc, argv, false, enable_acceptor), __default_host(dist_hostname), __default_port(port)
{
  __client_registry.reset(new ClientRegistry());
  string hostname = getHostname();

  #ifdef ARCH_ARM
  /* If NMM is used on ARM-System, use the StaticRegistry */
  IServerRegistryImpl* ipaq_registry = new StaticServerRegistry();
  setServerRegistry(ipaq_registry);
  __client_registry->addRegistry(LocalAddress(ipaq_registry->getReference()), false);
  #endif
  #ifndef ARCH_ARM
  TCPAddress address;
  address.setPort(__default_port);

  if (__default_host.empty())
    address.setHostname(hostname);
  else
    address.setHostname(__default_host);

  /* Try to connect the ServerRegistry with the given address */
  __client_registry->addRegistry(address, true);
  #endif

  __i_plugin_manager.reset(__client_registry->getRegistryList().find(RegistryName(hostname, __default_port))->second.plugin_manager->getParentObject()->getInterface<IPluginManager>());
}

ProxyApplication::~ProxyApplication() {

}

ClientRegistry& ProxyApplication::getRegistry() {
  return *__client_registry;
}
const ClientRegistry& ProxyApplication::getRegistry() const {
  return *__client_registry;
}

IPluginManager& ProxyApplication::getPluginManager(){
  return *__i_plugin_manager.get();
}

const IPluginManager& ProxyApplication::getPluginManager() const {
  return *__i_plugin_manager.get();
}

NMMApplication* ProxyApplication::getApplication(int argc, char** argv, string host , int port, bool enable_acceptor) {
  try {
    return new ProxyApplication(argc, argv, host, port, enable_acceptor);
  } catch (const Exception& e) {
    GLOBAL_ERROR_STREAM("Connection to locally running serverregistry failed, will use internal registry instead" << endl) ;
    return new NMMApplication(argc, argv);
  }
}

}//end namespace

