/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_PROXYAPPLICATION_HPP
#define NMM_PROXYAPPLICATION_HPP

#include "nmm/base/NMMApplication.hpp"

namespace NMM {
class TCPStrategy;
class TCPAcceptor;

/** The ProxyApplication tries to connect to the Server-Registry that runs on this system.
  * If no Server-Registry is available, an exception is thrown in the construtor.
  */
class ProxyApplication : public NMMApplication {
public:
/** @name Constructor & Destructor
  */
//@{
/** Constructor
  */
  ProxyApplication(int argc, char** argv, string="", int port = ClientRegistry::default_port, bool enable_acceptor = true);

/** Destructor
  */
  virtual ~ProxyApplication();
//@}

/** @name ClientRegistry & PluginManager
  */
//@{
/** This method returns the ClientRegistry, which can be used to search and request Nodes
  */
  virtual       ClientRegistry& getRegistry();
/** This method returns the ClientRegistry, which can be used to search and request Nodes
  */
  virtual const ClientRegistry& getRegistry() const;

/** This method returns the PluginManager, which can be used to load plugins
  */
  virtual       IPluginManager& getPluginManager();

/** This method returns the PluginManager, which can be used to load plugins
  */
  virtual const IPluginManager& getPluginManager() const;
//@}

/** @name Helper methods
  */
//@{
  /* This method can be used to detect automatically wheter an NMMApplication or a ProxyApplication can be used.
   * The caller of this method is responsible to delete this object
   */
  static NMMApplication* getApplication(int argc, char** argv, string="", int port = ClientRegistry::default_port, bool enable_acceptor = true);
//@}
private:
  auto_ptr<ClientRegistry>        __client_registry;        ///< pointer to the ClientRegistry
  IPluginManager_var              __i_plugin_manager;       ///< pointer to the IPluginManager
  string                          __default_host;           ///< uses this host to connect to the ServerRegistry
  int                             __default_port;           ///< uses this port to connect to the ServerRegistry
};


}

#endif

