/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "IExternalBufferManager.hpp"
#include "Buffer.hpp"

namespace NMM {
  IExternalBufferManager::IExternalBufferManager() {
    buffer_manager = 0;
  }

  IExternalBufferManager::~IExternalBufferManager() {
    if (buffer_manager) buffer_manager->release();
  }

  Buffer* IExternalBufferManager::getNewBuffer(const size_t size) {
    //DEBUG_STREAM2("getNewBuffer with size " << size << endl);
    if (buffer_manager == 0) {
      NamedObject *obj_ptr = dynamic_cast<NamedObject *>(this);
      if(obj_ptr) {
	string allocator_name = obj_ptr->getName() + " B_Allocator";
#ifdef ARCH_ARM
	buffer_manager = new B_Allocator(size, 1, 4, allocator_name.c_str()); 
#else
	buffer_manager = new B_Allocator(size, 4, 10, allocator_name.c_str()); 
#endif
      }
      else {
#ifdef ARCH_ARM
	buffer_manager = new B_Allocator(size, 1, 4); 
#else
	buffer_manager = new B_Allocator(size, 4, 10); 
#endif
      }
    }
    return buffer_manager->requestBuffer(size);
  }

  Buffer* IExternalBufferManager::getNewBuffer(const size_t size, 
                                                       const Buffer *buffer) {
    Buffer *new_buffer = getNewBuffer(size);
    if (new_buffer != NULL) {                    // copy the header
      // only copy timestamp if it was set before (otherwise, it should be all zero anyway)
      if(buffer->getTimestamp().is_set) {
	new_buffer->setTimestamp(buffer->getTimestamp());
      }
    }
    return new_buffer;
  }

  void IExternalBufferManager::setBufferManager(BufferManager *manager) {
    if (buffer_manager)
      buffer_manager->release();
    buffer_manager = manager;
  }

  const BufferManager* IExternalBufferManager::getBufferManager() const {
    return buffer_manager;
  }

  BufferManager* IExternalBufferManager::requestBufferManager()
  {
    if (buffer_manager == 0) {
      NamedObject *obj_ptr = dynamic_cast<NamedObject *>(this);
      if(obj_ptr) {
	string allocator_name = obj_ptr->getName() + " B_Allocator";
#ifdef ARCH_ARM
	buffer_manager = new B_Allocator(0, 0, 4, allocator_name.c_str()); 
#else
	buffer_manager = new B_Allocator(0, 0, 10, allocator_name.c_str()); 
#endif
      }
      else {
#ifdef ARCH_ARM
	buffer_manager = new B_Allocator(0, 0, 4); 
#else
	buffer_manager = new B_Allocator(0, 0, 10); 
#endif
      }
    }
    
    return buffer_manager->request();
  }

}
