/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Florian Winter <fw@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "LiveThread.hpp"
#include "BufferedSource.hpp"


namespace NMM {
  
  LiveThread::LiveThread(UsageEnvironment& env, MediaSource& source, MediaSink& sink)
    : __env(env),
      __source(source),
      __sink(sink),
      __live_stop(0)
  {}
  
  void LiveThread::start() {
    GLOBAL_DEBUG_STREAM("Starting LIVE thread" << endl);
    __live_stop = 0;
    activateThread();
  }
  
  void LiveThread::stop() {    
    GLOBAL_DEBUG_STREAM("Joining LIVE thread" << endl);
    
    __live_stop = 1;
    __sink.stopPlaying();
    
    deactivateThread();
  }
  
  void LiveThread::run() {
    // XXX: This is ugly! LIVE is an object oriented library, so you should pass an object instead of a function...
    __sink.startPlaying(__source, &afterPlaying, this);
    __env.taskScheduler().doEventLoop(&__live_stop);
  }
  
  void LiveThread::afterPlaying(void* /*obj*/) {
    // XXX: Notify the RTPStrategy ?
  }
  
} // namespace NMM

