/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *                    NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef _NMM_FORAMTSIZECALCULATOR_HPP_
#define _NMM_FORAMTSIZECALCULATOR_HPP_

#include <list>
#include "nmm/base/format/Format.hpp"

namespace NMM {

class FormatSizeCalculator {
  struct Resolution {
    uint x_resolution;
    uint y_resolution;

    Resolution(uint x,uint y) : x_resolution(x), y_resolution(y) {};
  };

  list<Resolution> resolutions_4_3;
  list<Resolution> resolutions_16_9;

  /**
   * \returns how many bits are needed to represent a pixel by the given format
   * or 0 if there is no relationship
   */
  uint getBitsPerPixel(const Format&);

  public:
    FormatSizeCalculator();
    ~FormatSizeCalculator();

    /**
     * This functions computes how much data represents
     * one second if using the given format
     */
    long computeBytesPerSecond(const Format&);

    /**
     * This function computes the optimal resolution for a given
     * format and data-rate
     * \returns 0,0 if there was no match
     */
    Format computeResolution(const Format&, unsigned long bytes_per_second);

    /**
     * \params bytes_per_second is the maximum allowed data rate
     * \returns a default YV12 format using 4:3 aspect ratio
     */
    Format getRawVideoFormat(const long bytes_per_second);
};

}

#endif
