#include "Resources.hpp"

namespace NMM {

Resources::Resources() :  NMM::SerializedValue("Resources", "NMM") {
  registerValues();
  processing_time = list<float>();
  bandwidth       = list<float>();
  min_processing_time = 0;
  avg_processing_time = 0;
  max_processing_time = 0;
  min_bandwidth       = 0;
  avg_bandwidth       = 0;
  max_bandwidth       = 0;
}

Resources::Resources(const Resources& r) :  NMM::SerializedValue(r) {
  registerValues();
  *this = r;
}

Resources::~Resources() {
  processing_time.clear();
  bandwidth.clear();
}

const float Resources::getMaximumProcessingTime() const {
  return max_processing_time.getRValue();
}

const float Resources::getAverageProcessingTime() const {
  return getAverage(processing_time);
}

const float Resources::getMinimumProcessingTime() const {
  return min_processing_time.getRValue();
}

const float Resources::getMaximumBandwidth() const {
  return max_bandwidth.getRValue();
}

const float Resources::getAverageBandwidth() const {
  return getAverage(bandwidth);
}

const float Resources::getMinimumBandwidth() const {
  return min_bandwidth.getRValue();
}

void Resources::setProcessingTime(float t) {
  if (t < min_processing_time.getRValue()) {
    min_processing_time = t;
  }
  if (t > max_processing_time.getRValue()) {
    max_processing_time = t;
  }

  processing_time.push_back(t);
}

void Resources::setBandwidth(float t) {
  if (t < min_bandwidth.getRValue()) {
    min_bandwidth = t;
  }
  if (t > max_bandwidth.getRValue()) {
    max_bandwidth = t;
  }
  bandwidth.push_back(t);
}

const float Resources::getAverage(const list<float>& l) const {
  double sum = 0;
  float num = 0;
  for (list<float>::const_iterator i = l.begin(); i != l.end(); ++i) {
    sum += *i;
    num++;
  }
  float ret_val = (float) (sum / num);
  return ret_val;
}

void Resources::registerValues() {
  insertArgument(&min_processing_time);
  insertArgument(&avg_processing_time);
  insertArgument(&max_processing_time);
  insertArgument(&min_bandwidth);
  insertArgument(&avg_bandwidth);
  insertArgument(&max_bandwidth);
}

void Resources::updateAttributes() {
  processing_time.push_back(avg_processing_time.getRValue());
  bandwidth.push_back(avg_bandwidth.getRValue());
}

void Resources::prepareAttributes() {
  avg_processing_time = getAverageProcessingTime();
  avg_bandwidth       = getAverageBandwidth();
}

}
