/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "StdStream.hpp"

namespace NMM {

StdOStream::StdOStream(std::ostream* stream)
  : __stream(stream), __fin(0x20)
{
}

StdOStream::~StdOStream() {
  if (__stream)
    __stream->flush();

  __stream = 0;
}

StdOStream& StdOStream::operator<<(const u_int16_t& value) {
  *__stream << value << __fin;
  return *this;
}
StdOStream& StdOStream::operator<<(const int16_t& value   ) {
  *__stream << value << __fin;
  return *this;
}
StdOStream& StdOStream::operator<<(const u_int32_t& value) {
  *__stream << value << __fin;
  return *this;
}
StdOStream& StdOStream::operator<<(const int32_t& value   ) {
  *__stream << value << __fin;
  return *this;
}

StdOStream& StdOStream::operator<<(const u_int64_t& value ) {
  *__stream << value << __fin;
  return *this;
}
StdOStream& StdOStream::operator<<(const int64_t& value ) {
  *__stream << value << __fin;
  return *this;
}
StdOStream& StdOStream::operator<<(const bool& value ) {
  *__stream << value << __fin;
  return *this;
}
StdOStream& StdOStream::operator<<(const unsigned long& value) {
  *__stream << value << __fin;
  return *this;
}

StdOStream& StdOStream::operator<<(const long& value ) {
  *__stream << value << __fin;
  return *this;
}

StdOStream& StdOStream::operator<<(const double& value ) {
  *__stream << value << __fin;
  return *this;
}
StdOStream& StdOStream::operator<<(const float& value ) {
  *__stream << value << __fin;
  return *this;
}
StdOStream& StdOStream::operator<<(const char& value ) {
  *__stream << value << __fin;
  return *this;
}
StdOStream& StdOStream::operator<<(const unsigned char& value) {
  *__stream << value << __fin;
  return *this;
}
StdOStream& StdOStream::operator<<(const string& value) {
  *__stream << value.size() << __fin << value << __fin;
  return *this;
}

void StdOStream::writeBuffer(const char* value, unsigned int size)  {
  __stream->write(value,size);
  __stream->write(&__fin,1);
}

  void StdOStream::writeBuffer(const unsigned char* value, unsigned int size){
    __stream->write(reinterpret_cast<const char*>(value),size);
    __stream->write(reinterpret_cast<char*>(&__fin),1);
  }

void StdOStream::writeData() {
  if (__stream)
    __stream->flush();
}

StdIStream::StdIStream(std::istream* stream)
   : __stream(stream)
{

}


StdIStream& StdIStream::operator>>(u_int16_t& value)        {
  *__stream >> value;
  return *this;
}
StdIStream& StdIStream::operator>>(int16_t& value){
  *__stream >> value;
  return *this;
}
StdIStream& StdIStream::operator>>(u_int32_t& value  ){
  *__stream >> value;
  return *this;
}
StdIStream& StdIStream::operator>>(int32_t& value  ){
  *__stream >> value;
  return *this;
}

StdIStream& StdIStream::operator>>(u_int64_t& value  ){
  *__stream >> value;
  return *this;
}
StdIStream& StdIStream::operator>>(int64_t& value  ){
  *__stream >> value;
  return *this;
}
StdIStream& StdIStream::operator>>(bool& value  ){
  *__stream >> value;
  return *this;
}
StdIStream& StdIStream::operator>>(unsigned long& value){
  *__stream >> value;
  return *this;
}

StdIStream& StdIStream::operator>>(long& value ){
  *__stream >> value;
  return *this;
}

StdIStream& StdIStream::operator>>(double& value ){
  *__stream >> value;
  return *this;
}
StdIStream& StdIStream::operator>>(float& value ){
  *__stream >> value;
  return *this;
}
StdIStream& StdIStream::operator>>(char& value ){
  *__stream >> value;
  return *this;
}
StdIStream& StdIStream::operator>>(unsigned char& value){
  *__stream >> value;
  return *this;
}
StdIStream& StdIStream::operator>>(string& value){
  size_t size = 0;
  *__stream >> size;
  value.resize(size);
  __stream->get(); //skip the blank
  if (size > 0)
    __stream->read(&value[0], size);
  return *this;
}
void StdIStream::readBuffer(char* value, unsigned int size)           {
  __stream->get(); //skip the blank
  __stream->read(value, size);
}
void StdIStream::readBuffer(unsigned char* value, unsigned int size){
  __stream->get(); //skip the blank
  __stream->read(reinterpret_cast<char*>(value), size);
}     
     
bool StdIStream::isEmpty() const {
  return true;
}

}


