/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "UnbufferedNetStream.hpp"

#include "nmm/utils/net/Sock.hpp"

#include <errno.h>       // for errnos
#include <sys/socket.h>  // for inet_ntoa
#include <sys/types.h>
#include <fstream>

namespace NMM {
  
  UnbufferedNetStream::UnbufferedNetStream(Sock& sock)
    : NetStream(sock)
  {}
  
  UnbufferedNetStream::UnbufferedNetStream(const UnbufferedNetStream& other)
    : NetStream(other)
  {}
  
  UnbufferedNetStream::~UnbufferedNetStream()
  {}
  
  UnbufferedNetStream* UnbufferedNetStream::copy() const {
    return new UnbufferedNetStream(*this);
  }
  
  void UnbufferedNetStream::writeValue(const void* value, size_t size) {
    /* write bytes to the socket */
    ssize_t retval = getSock().send(static_cast<const char*>(value), size, MSG_NOSIGNAL);
    if(retval == 0) {
      throw ConnectionClosed(EPIPE);
    }
    
    /* count bytes */
    countWriteBytes(retval);
    
    /* if not all values are written, write the rest */
    if(static_cast<size_t>(retval) != size) {
      const char* new_offset = static_cast<const char*>(value) + retval;
      writeValue(new_offset, size - retval);
    }
  }
  
  void UnbufferedNetStream::readValue(void* value, size_t size) {
    /* read bytes from the socket */
    ssize_t retval = getSock().recv(static_cast<char*>(value), size, MSG_NOSIGNAL);
    if(retval == 0) {
      throw ConnectionClosed(EPIPE);
    }
    
    /* count bytes */
    countReadBytes(retval);
    
    /* if not all values are read, call this method again to read the rest */
    if(static_cast<size_t>(retval) != size) {
      char* new_offset = static_cast<char*>(value) + retval;
      readValue(new_offset, size - retval);
    }
  }
  
}





