/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_UNBUFFERED_NET_STREAM_HPP
#define NMM_UNBUFFERED_NET_STREAM_HPP

#include "NetStream.hpp"

namespace NMM {
  
  class Sock;
  
  /**
   *  @brief Unbuffered network stream
   *
   *  This class implements a simple, unbuffered network stream. Data is written to the network
   *  immediately. The read operation blocks until the requested number of bytes has been read.
   */
  class UnbufferedNetStream : public NetStream {
  public:
    /** @name Constructor & Destructor
     */
    //@{
    /** Constructor
     */
    UnbufferedNetStream(Sock& sock);
    
    /** Copy constructor
     */
    UnbufferedNetStream(const UnbufferedNetStream& other);
    
    /** Destructor
     */
    virtual ~UnbufferedNetStream();
    
    /** virtual Copy constructor
     */
    virtual UnbufferedNetStream* copy() const;
    //@}

    /** Returns true if the stream does not contain any data
      */
    virtual bool isEmpty() const { return true; }

  protected:
    /** Internal method to write the data to the network
     */
    virtual void writeValue(const void*, size_t);

    /** Internal method to read the data from the network
     */
    virtual void readValue(void*, size_t);
  };
  
}

#endif
