
#include "INode.hpp"
#include "nmm/base/InterfaceFactory.hpp"

#include "nmm/base/format/Property.hpp"

namespace NMM
{
  
  
  void INode::reachState(const INode::State& state) 
      throw(const Exception&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    TInValue<INode::State > state_value(state, false);
    Event event("INode::reachState", &state_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void INode::reachConstructed() 
      throw(const Exception&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    Event event("INode::reachConstructed");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void INode::reachInitialized() 
      throw(const Exception&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    Event event("INode::reachInitialized");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void INode::reachOutputInitialized() 
      throw(const Exception&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    Event event("INode::reachOutputInitialized");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void INode::reachActivated() 
      throw(const Exception&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    Event event("INode::reachActivated");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void INode::reachStarted() 
      throw(const Exception&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    Event event("INode::reachStarted");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void INode::setPrefetchQueueSize(const unsigned int& pre_fetch_queue_size) /* limited(INode::CONSTRUCTED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TInValue<unsigned int > pre_fetch_queue_size_value(pre_fetch_queue_size, false);
    Event event("INode::setPrefetchQueueSize", &pre_fetch_queue_size_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  unsigned int INode::getPrefetchQueueSize()
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    Event event("INode::getPrefetchQueueSize");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void INode::waitPrefetchFinished() /* limited(INode::ACTIVATED) */
      throw(const Exception&, const IllegalStateException&)
  {
    Event event("INode::waitPrefetchFinished");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void INode::init() /* limited(INode::CONSTRUCTED) */ 
      throw(const Exception&, const IllegalStateException&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    Event event("INode::init");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void INode::initOutput() /* limited(INode::INITIALIZED) */ 
      throw(const Exception&, const IllegalStateException&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    Event event("INode::initOutput");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void INode::activate() /* limited(INode::OUTPUT_INITIALIZED) */ 
      throw(const Exception&, const IllegalStateException&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    Event event("INode::activate");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  Result INode::flush() /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<Result > return_value;
    Event event("INode::flush");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void INode::start() /* limited(INode::ACTIVATED) */ 
      throw(const Exception&, const IllegalStateException&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    Event event("INode::start");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void INode::stop() /* limited(INode::STARTED) */ 
      throw(const Exception&, const IllegalStateException&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    Event event("INode::stop");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void INode::deactivate() /* limited(INode::ACTIVATED) */ 
      throw(const Exception&, const IllegalStateException&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    Event event("INode::deactivate");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void INode::deinitOutput() /* limited(INode::OUTPUT_INITIALIZED) */ 
      throw(const Exception&, const IllegalStateException&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    Event event("INode::deinitOutput");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void INode::deinit() /* limited(INode::INITIALIZED) */ 
      throw(const Exception&, const IllegalStateException&, const StateTransitionFailedException&, const UnsuccessfulRequiredCallException&)
  {
    Event event("INode::deinit");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        if(StateTransitionFailedException* ex = dynamic_cast<StateTransitionFailedException*>(event.getException())) {
          throw StateTransitionFailedException(*ex);
        }
        if(UnsuccessfulRequiredCallException* ex = dynamic_cast<UnsuccessfulRequiredCallException*>(event.getException())) {
          throw UnsuccessfulRequiredCallException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  bool INode::isConstructed() const
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("INode::isConstructed");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  bool INode::isInitialized() const
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("INode::isInitialized");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  bool INode::isOutputInitialized() const
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("INode::isOutputInitialized");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  bool INode::isActivated() const
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("INode::isActivated");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  bool INode::isStarted() const
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("INode::isStarted");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  INode* INode::getSuccessorNode(const string& out_jack)
      throw(const Exception&)
  {
    TOutValue<INode* > return_value;
    TInValue<string > out_jack_value(out_jack, false);
    Event event("INode::getSuccessorNode", &out_jack_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  INode* INode::getPredecessorNode(const string& in_jack)
      throw(const Exception&)
  {
    TOutValue<INode* > return_value;
    TInValue<string > in_jack_value(in_jack, false);
    Event event("INode::getPredecessorNode", &in_jack_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IInputJack* INode::getInputJack(const Format& format, const string& jack_tag) /* limited(INode::INITIALIZED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<IInputJack* > return_value;
    TInValue<Format > format_value(format, false);
    TInValue<string > jack_tag_value(jack_tag, false);
    Event event("INode::getInputJack", &format_value, &jack_tag_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IInputJack* INode::getInputStream(const string& jack_tag) /* limited(INode::INITIALIZED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<IInputJack* > return_value;
    TInValue<string > jack_tag_value(jack_tag, false);
    Event event("INode::getInputStream", &jack_tag_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<string >  INode::getInputStreamTags() const
      throw(const Exception&)
  {
    TOutValue<list<string >  > return_value;
    Event event("INode::getInputStreamTags");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  bool INode::isInputConnected(const string& jack_tag /* = "default" */) const
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    TInValue<string > jack_tag_value(jack_tag, false);
    Event event("INode::isInputConnected", &jack_tag_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Format* INode::getInputStreamFormat(const string& jack_tag /* = "default" */)
      throw(const Exception&)
  {
    TOutValue<Format* > return_value;
    TInValue<string > jack_tag_value(jack_tag, false);
    Event event("INode::getInputStreamFormat", &jack_tag_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IOutputJack* INode::getOutputJack(const Format& format, const string& jack_tag) /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<IOutputJack* > return_value;
    TInValue<Format > format_value(format, false);
    TInValue<string > jack_tag_value(jack_tag, false);
    Event event("INode::getOutputJack", &format_value, &jack_tag_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IOutputJack* INode::getUnintializedOutputJack(const Format& format, const string& jack_tag) /* limited(INode::INITIALIZED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<IOutputJack* > return_value;
    TInValue<Format > format_value(format, false);
    TInValue<string > jack_tag_value(jack_tag, false);
    Event event("INode::getUnintializedOutputJack", &format_value, &jack_tag_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IOutputJack* INode::getOutputStream(const string& jack_tag) /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<IOutputJack* > return_value;
    TInValue<string > jack_tag_value(jack_tag, false);
    Event event("INode::getOutputStream", &jack_tag_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<string >  INode::getOutputStreamTags() const
      throw(const Exception&)
  {
    TOutValue<list<string >  > return_value;
    Event event("INode::getOutputStreamTags");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  bool INode::isOutputConnected(const string& jack_tag /* = "default" */) const
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    TInValue<string > jack_tag_value(jack_tag, false);
    Event event("INode::isOutputConnected", &jack_tag_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IOutputJack* INode::cloneOutputJack(Format*& intersec_format, const string& jack_tag /* = "default" */) /* limited(INode::OUTPUT_INITIALIZED) */ /* limited(INode::ACTIVATED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<IOutputJack* > return_value;
    TOutValue<Format* > intersec_format_value(intersec_format);
    TInValue<string > jack_tag_value(jack_tag, false);
    Event event("INode::cloneOutputJack", &intersec_format_value, &jack_tag_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    intersec_format = intersec_format_value.getRValue();
    return return_value.getRValue();
  }
  
  Format* INode::getOutputStreamFormat(const string& jack_tag /* = "default" */)
      throw(const Exception&)
  {
    TOutValue<Format* > return_value;
    TInValue<string > jack_tag_value(jack_tag, false);
    Event event("INode::getOutputStreamFormat", &jack_tag_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  float INode::getCost(const string& jack_tag) const
      throw(const Exception&)
  {
    TOutValue<float > return_value;
    TInValue<string > jack_tag_value(jack_tag, false);
    Event event("INode::getCost", &jack_tag_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  INode::Type INode::getType() const
      throw(const Exception&)
  {
    TOutValue<INode::Type > return_value;
    Event event("INode::getType");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  INode::State INode::getState() const
      throw(const Exception&)
  {
    TOutValue<INode::State > return_value;
    Event event("INode::getState");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Property* INode::getInputProperty()
      throw(const Exception&)
  {
    TOutValue<Property* > return_value;
    Event event("INode::getInputProperty");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Property* INode::getWorkingInputProperty()
      throw(const Exception&)
  {
    TOutValue<Property* > return_value;
    Event event("INode::getWorkingInputProperty");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Property* INode::getOutputProperty()
      throw(const Exception&)
  {
    TOutValue<Property* > return_value;
    Event event("INode::getOutputProperty");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Property* INode::getWorkingOutputProperty()
      throw(const Exception&)
  {
    TOutValue<Property* > return_value;
    Event event("INode::getWorkingOutputProperty");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ComputationConfig* INode::getComputationConfig() const
      throw(const Exception&)
  {
    TOutValue<ComputationConfig* > return_value;
    Event event("INode::getComputationConfig");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ComputationConfig* INode::getWorkingComputationConfig() const
      throw(const Exception&)
  {
    TOutValue<ComputationConfig* > return_value;
    Event event("INode::getWorkingComputationConfig");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void INode::setComputationConfig(const ComputationConfig* cc)
      throw(const Exception&)
  {
    TInValue<ComputationConfig* > cc_value (const_cast<ComputationConfig* >(cc), false);
    Event event("INode::setComputationConfig", &cc_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void INode::setWorkingComputationConfig(const ComputationConfig* cc)
      throw(const Exception&)
  {
    TInValue<ComputationConfig* > cc_value (const_cast<ComputationConfig* >(cc), false);
    Event event("INode::setWorkingComputationConfig", &cc_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  float INode::getInternalCost() const
      throw(const Exception&)
  {
    TOutValue<float > return_value;
    Event event("INode::getInternalCost");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result INode::sendData() /* limited(INode::OUTPUT_INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* in-stream */ 
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<Result > return_value;
    Event event("INode::sendData");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result INode::sendDataFailed() /* limited(INode::INITIALIZED) */ /* in-stream */ 
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<Result > return_value;
    Event event("INode::sendDataFailed");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* INode::INode_type = "INode";
  TIFactoryObject<INode > __INode_prototype("INode");
  TValueConstructor< INode::Type > __INode_Type_prototype(TypeInfo("INode::Type", "NMM"));

  OValueStream& operator<<(OValueStream& ov, const INode::Type& t)
  {
    ov.beginType(TypeInfo("INode::Type", "NMM"));
    ov << reinterpret_cast<const int&>(t);
    ov.endType();
    return ov;
  }

  IValueStream& operator>>(IValueStream& iv, INode::Type& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("INode::Type", "NMM")) {
      throw InvalidTypeException(TypeInfo("INode::Type", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    iv >> reinterpret_cast<int&>(t);
    iv.goUp();
    return iv;
  }

  TValueConstructor< INode::State > __INode_State_prototype(TypeInfo("INode::State", "NMM"));

  OValueStream& operator<<(OValueStream& ov, const INode::State& t)
  {
    ov.beginType(TypeInfo("INode::State", "NMM"));
    ov << reinterpret_cast<const int&>(t);
    ov.endType();
    return ov;
  }

  IValueStream& operator>>(IValueStream& iv, INode::State& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("INode::State", "NMM")) {
      throw InvalidTypeException(TypeInfo("INode::State", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    iv >> reinterpret_cast<int&>(t);
    iv.goUp();
    return iv;
  }



  const char* INode::reachState_event = "INode::reachState";
  const char* INode::reachConstructed_event = "INode::reachConstructed";
  const char* INode::reachInitialized_event = "INode::reachInitialized";
  const char* INode::reachOutputInitialized_event = "INode::reachOutputInitialized";
  const char* INode::reachActivated_event = "INode::reachActivated";
  const char* INode::reachStarted_event = "INode::reachStarted";
  const char* INode::setPrefetchQueueSize_event = "INode::setPrefetchQueueSize";
  const char* INode::getPrefetchQueueSize_event = "INode::getPrefetchQueueSize";
  const char* INode::waitPrefetchFinished_event = "INode::waitPrefetchFinished";
  const char* INode::init_event = "INode::init";
  const char* INode::initOutput_event = "INode::initOutput";
  const char* INode::activate_event = "INode::activate";
  const char* INode::flush_event = "INode::flush";
  const char* INode::start_event = "INode::start";
  const char* INode::stop_event = "INode::stop";
  const char* INode::deactivate_event = "INode::deactivate";
  const char* INode::deinitOutput_event = "INode::deinitOutput";
  const char* INode::deinit_event = "INode::deinit";
  const char* INode::isConstructed_event = "INode::isConstructed";
  const char* INode::isInitialized_event = "INode::isInitialized";
  const char* INode::isOutputInitialized_event = "INode::isOutputInitialized";
  const char* INode::isActivated_event = "INode::isActivated";
  const char* INode::isStarted_event = "INode::isStarted";
  const char* INode::getSuccessorNode_event = "INode::getSuccessorNode";
  const char* INode::getPredecessorNode_event = "INode::getPredecessorNode";
  const char* INode::getInputJack_event = "INode::getInputJack";
  const char* INode::getInputStream_event = "INode::getInputStream";
  const char* INode::getInputStreamTags_event = "INode::getInputStreamTags";
  const char* INode::isInputConnected_event = "INode::isInputConnected";
  const char* INode::getInputStreamFormat_event = "INode::getInputStreamFormat";
  const char* INode::getOutputJack_event = "INode::getOutputJack";
  const char* INode::getUnintializedOutputJack_event = "INode::getUnintializedOutputJack";
  const char* INode::getOutputStream_event = "INode::getOutputStream";
  const char* INode::getOutputStreamTags_event = "INode::getOutputStreamTags";
  const char* INode::isOutputConnected_event = "INode::isOutputConnected";
  const char* INode::cloneOutputJack_event = "INode::cloneOutputJack";
  const char* INode::getOutputStreamFormat_event = "INode::getOutputStreamFormat";
  const char* INode::getCost_event = "INode::getCost";
  const char* INode::getType_event = "INode::getType";
  const char* INode::getState_event = "INode::getState";
  const char* INode::getInputProperty_event = "INode::getInputProperty";
  const char* INode::getWorkingInputProperty_event = "INode::getWorkingInputProperty";
  const char* INode::getOutputProperty_event = "INode::getOutputProperty";
  const char* INode::getWorkingOutputProperty_event = "INode::getWorkingOutputProperty";
  const char* INode::getComputationConfig_event = "INode::getComputationConfig";
  const char* INode::getWorkingComputationConfig_event = "INode::getWorkingComputationConfig";
  const char* INode::setComputationConfig_event = "INode::setComputationConfig";
  const char* INode::setWorkingComputationConfig_event = "INode::setWorkingComputationConfig";
  const char* INode::getInternalCost_event = "INode::getInternalCost";
  const char* INode::sendData_event = "INode::sendData";
  Event* INode::create_sendData()
  {

    return new Event("INode::sendData");
  }

  const char* INode::sendDataFailed_event = "INode::sendDataFailed";
  Event* INode::create_sendDataFailed()
  {

    return new Event("INode::sendDataFailed");
  }



}


