
#include "IOutputJack.hpp"
#include "nmm/base/InterfaceFactory.hpp"

#include "IInputJack.hpp"
#include "nmm/base/CommunicationChannel.hpp"

namespace NMM
{
  
  
  CommunicationChannel* IOutputJack::connectTo(const IInputJack& jack) 
      throw(const Exception&, const BindingException&)
  {
    TOutValue<CommunicationChannel* > return_value;
    TInValue<IInputJack > jack_value(jack, false);
    Event event("IOutputJack::connectTo", &jack_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(BindingException* ex = dynamic_cast<BindingException*>(event.getException())) {
          throw BindingException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IInputJack* IOutputJack::getDataPartner()
      throw(const Exception&)
  {
    TOutValue<IInputJack* > return_value;
    Event event("IOutputJack::getDataPartner");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ITransportStrategy* IOutputJack::getTransportStrategy()
      throw(const Exception&)
  {
    TOutValue<ITransportStrategy* > return_value;
    Event event("IOutputJack::getTransportStrategy");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IOutputJack::IOutputJack_type = "IOutputJack";
  TIFactoryObject<IOutputJack > __IOutputJack_prototype("IOutputJack");


  const char* IOutputJack::connectTo_event = "IOutputJack::connectTo";
  const char* IOutputJack::getDataPartner_event = "IOutputJack::getDataPartner";
  const char* IOutputJack::getTransportStrategy_event = "IOutputJack::getTransportStrategy";


}


