
#ifndef NMM_IUsageImpl_HPP
#define NMM_IUsageImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IUsage.hpp"

#include "nmm/interfaces/base/IUsageNotifyImpl.hpp"

namespace NMM
{

  class IUsageImpl  : public virtual NMMObject { 
  public:
    IUsageImpl() { addInterface(new IUsage(*this)); registerEvents(); };

    virtual ~IUsageImpl() {};

  protected:
    /**
     *Set the Usage of the node.
     *@param usage usage of the node
     *@param user set the IUsageNotify-interface of this user so it will be notified, if its status (master or slave) changes
     */

    virtual void setUsage(const IUsage::usage_t& usage, const IUsageNotify* user) = 0;
    /**
     *Unset the Usage of the node.
     *@param usage usage of the node
     *@param user set the IUsageNotify-interface of this user to identify the user
     */

    virtual void unsetUsage(const IUsage::usage_t& usage, const IUsageNotify* user) = 0;
    /**
     *Get the current Usage of this node
     *@return usage current usage
     */

    virtual IUsage::usage_t getUsage() = 0;
    /**
     *Get the number of users wich have rewuested this node before the given user with the given usage
     *@param usage usage
     *@param user the user wich makes the request
     *@return int rank
     */

    virtual int getRank(const IUsage::usage_t& usage, const IUsageNotify* user) = 0;
    /**
     *Checks, if the given user will be Master with the given usage
     *@param mode usage
     *@param user user-identification
     *@return SUCCESS if the user will be master
     *@return FAILED if the user will be slave
     */

    virtual Result isMaster(const IUsage::usage_t& mode, const IUsageNotify* user) = 0;


  private:
    void registerEvents();
  };

}

#endif
