
#include "ICompositeNode.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  unsigned int ICompositeNode::addNode(const INode& node) /* limited(INode::CONSTRUCTED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<unsigned int > return_value;
    TInValue<INode > node_value(node, false);
    Event event("ICompositeNode::addNode", &node_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int ICompositeNode::addNodeWithInterfaces(const INode& node, const list<Interface* > & interfaces) /* limited(INode::CONSTRUCTED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<unsigned int > return_value;
    TInValue<INode > node_value(node, false);
    TInValue<list<Interface* >  > interfaces_value(interfaces, false);
    Event event("ICompositeNode::addNodeWithInterfaces", &node_value, &interfaces_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ICompositeNode::renameInputJackTag(const string& old_cn_jack_tag, const string& new_cn_jack_tag) /* limited(INode::INITIALIZED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<Result > return_value;
    TInValue<string > old_cn_jack_tag_value(old_cn_jack_tag, false);
    TInValue<string > new_cn_jack_tag_value(new_cn_jack_tag, false);
    Event event("ICompositeNode::renameInputJackTag", &old_cn_jack_tag_value, &new_cn_jack_tag_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ICompositeNode::renameOutputJackTag(const string& old_cn_jack_tag, const string& new_cn_jack_tag) /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TOutValue<Result > return_value;
    TInValue<string > old_cn_jack_tag_value(old_cn_jack_tag, false);
    TInValue<string > new_cn_jack_tag_value(new_cn_jack_tag, false);
    Event event("ICompositeNode::renameOutputJackTag", &old_cn_jack_tag_value, &new_cn_jack_tag_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ICompositeNode::ICompositeNode_type = "ICompositeNode";
  TIFactoryObject<ICompositeNode > __ICompositeNode_prototype("ICompositeNode");


  const char* ICompositeNode::addNode_event = "ICompositeNode::addNode";
  const char* ICompositeNode::addNodeWithInterfaces_event = "ICompositeNode::addNodeWithInterfaces";
  const char* ICompositeNode::renameInputJackTag_event = "ICompositeNode::renameInputJackTag";
  const char* ICompositeNode::renameOutputJackTag_event = "ICompositeNode::renameOutputJackTag";


}


