
#include "IPluginManager.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void IPluginManager::addPluginPath(const string& path)
      throw(const Exception&)
  {
    TInValue<string > path_value(path, false);
    Event event("IPluginManager::addPluginPath", &path_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IPluginManager::loadLibrary(const string& path)
      throw(const Exception&)
  {
    TInValue<string > path_value(path, false);
    Event event("IPluginManager::loadLibrary", &path_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IPluginManager::loadPlugin(const string& plugin_name) 
      throw(const Exception&, const Exception&)
  {
    TInValue<string > plugin_name_value(plugin_name, false);
    Event event("IPluginManager::loadPlugin", &plugin_name_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(Exception* ex = dynamic_cast<Exception*>(event.getException())) {
          throw Exception(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void IPluginManager::loadExternalPlugins(const string& directory) 
      throw(const Exception&, const Exception&)
  {
    TInValue<string > directory_value(directory, false);
    Event event("IPluginManager::loadExternalPlugins", &directory_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(Exception* ex = dynamic_cast<Exception*>(event.getException())) {
          throw Exception(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void IPluginManager::removeExternalPlugins(const string& directory) 
      throw(const Exception&, const Exception&)
  {
    TInValue<string > directory_value(directory, false);
    Event event("IPluginManager::removeExternalPlugins", &directory_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(Exception* ex = dynamic_cast<Exception*>(event.getException())) {
          throw Exception(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void IPluginManager::activateRegistryPart(const string& plugin) 
      throw(const Exception&, const Exception&)
  {
    TInValue<string > plugin_value(plugin, false);
    Event event("IPluginManager::activateRegistryPart", &plugin_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(Exception* ex = dynamic_cast<Exception*>(event.getException())) {
          throw Exception(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void IPluginManager::activateFactoryPart(const string& plugin) 
      throw(const Exception&, const Exception&)
  {
    TInValue<string > plugin_value(plugin, false);
    Event event("IPluginManager::activateFactoryPart", &plugin_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(Exception* ex = dynamic_cast<Exception*>(event.getException())) {
          throw Exception(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void IPluginManager::deactivateRegistryPart(const string& plugin) 
      throw(const Exception&, const Exception&)
  {
    TInValue<string > plugin_value(plugin, false);
    Event event("IPluginManager::deactivateRegistryPart", &plugin_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(Exception* ex = dynamic_cast<Exception*>(event.getException())) {
          throw Exception(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void IPluginManager::deactivateFactoryPart(const string& plugin) 
      throw(const Exception&, const Exception&)
  {
    TInValue<string > plugin_value(plugin, false);
    Event event("IPluginManager::deactivateFactoryPart", &plugin_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(Exception* ex = dynamic_cast<Exception*>(event.getException())) {
          throw Exception(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void IPluginManager::removeLibrary(const string& plugin_name) 
      throw(const Exception&, const Exception&)
  {
    TInValue<string > plugin_name_value(plugin_name, false);
    Event event("IPluginManager::removeLibrary", &plugin_name_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(Exception* ex = dynamic_cast<Exception*>(event.getException())) {
          throw Exception(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  list<ObjectID >  IPluginManager::getPluginID(const string& plugin, const unsigned int& index)
      throw(const Exception&)
  {
    TOutValue<list<ObjectID >  > return_value;
    TInValue<string > plugin_value(plugin, false);
    TInValue<unsigned int > index_value(index, false);
    Event event("IPluginManager::getPluginID", &plugin_value, &index_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<string >  IPluginManager::getPlugins() const
      throw(const Exception&)
  {
    TOutValue<list<string >  > return_value;
    Event event("IPluginManager::getPlugins");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<string >  IPluginManager::getUsedPlugins() const
      throw(const Exception&)
  {
    TOutValue<list<string >  > return_value;
    Event event("IPluginManager::getUsedPlugins");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<string >  IPluginManager::getPluginPath() const
      throw(const Exception&)
  {
    TOutValue<list<string >  > return_value;
    Event event("IPluginManager::getPluginPath");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IPluginManager::IPluginManager_type = "IPluginManager";
  TIFactoryObject<IPluginManager > __IPluginManager_prototype("IPluginManager");


  const char* IPluginManager::addPluginPath_event = "IPluginManager::addPluginPath";
  const char* IPluginManager::loadLibrary_event = "IPluginManager::loadLibrary";
  const char* IPluginManager::loadPlugin_event = "IPluginManager::loadPlugin";
  const char* IPluginManager::loadExternalPlugins_event = "IPluginManager::loadExternalPlugins";
  const char* IPluginManager::removeExternalPlugins_event = "IPluginManager::removeExternalPlugins";
  const char* IPluginManager::activateRegistryPart_event = "IPluginManager::activateRegistryPart";
  const char* IPluginManager::activateFactoryPart_event = "IPluginManager::activateFactoryPart";
  const char* IPluginManager::deactivateRegistryPart_event = "IPluginManager::deactivateRegistryPart";
  const char* IPluginManager::deactivateFactoryPart_event = "IPluginManager::deactivateFactoryPart";
  const char* IPluginManager::removeLibrary_event = "IPluginManager::removeLibrary";
  const char* IPluginManager::getPluginID_event = "IPluginManager::getPluginID";
  const char* IPluginManager::getPlugins_event = "IPluginManager::getPlugins";
  const char* IPluginManager::getUsedPlugins_event = "IPluginManager::getUsedPlugins";
  const char* IPluginManager::getPluginPath_event = "IPluginManager::getPluginPath";


}


