
#ifndef NMM_IPluginManager_HPP
#define NMM_IPluginManager_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{

  class IPluginManager : virtual public Interface { 
  public:
    IPluginManager() {  setTypeInfo(TypeInfo(IPluginManager::IPluginManager_type, "NMM")); };

    IPluginManager(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IPluginManager::IPluginManager_type, "NMM")); setEventReceiver(parent); };

    virtual ~IPluginManager() {};

    static const char* IPluginManager_type;

    virtual  string getInterfaceType() const { return IPluginManager_type; }

    Interface* copy() const { return new IPluginManager(*this); }
    static const char* addPluginPath_event;
    static const char* loadLibrary_event;
    static const char* loadPlugin_event;
    static const char* loadExternalPlugins_event;
    static const char* removeExternalPlugins_event;
    static const char* activateRegistryPart_event;
    static const char* activateFactoryPart_event;
    static const char* deactivateRegistryPart_event;
    static const char* deactivateFactoryPart_event;
    static const char* removeLibrary_event;
    static const char* getPluginID_event;
    static const char* getPlugins_event;
    static const char* getUsedPlugins_event;
    static const char* getPluginPath_event;
/** This method adds a plugin path.
  */

    virtual void addPluginPath(const string& path)
      throw(const Exception&);
/** this wrapper method calls the three methods loadPlugin, activateRegistryPart, activateFactoryPart
  */

    virtual void loadLibrary(const string& path)
      throw(const Exception&);
/** loads a specific plugin
  */

    virtual void loadPlugin(const string& plugin_name)
      throw(const Exception&, const Exception&);
/** This method loads all plugins in a given directory. These plugins must be 'external' plugins which does 
    not provide NMM nodes. Thus, this method does not search and register NMM nodes, even if they exist in 
    a plugin */    

    virtual void loadExternalPlugins(const string& directory)
      throw(const Exception&, const Exception&);
/** This method removes all plugins located in a given directory. These plugins must be 'external' plugins 
    which does not provide NMM nodes. Thus, this method does not unregister NMM nodes, even if they 
    exist */

    virtual void removeExternalPlugins(const string& directory)
      throw(const Exception&, const Exception&);
/** activates the registry part of a plugin
  */

    virtual void activateRegistryPart(const string& plugin)
      throw(const Exception&, const Exception&);
/** activates the factory part of a plugin
  */

    virtual void activateFactoryPart(const string& plugin)
      throw(const Exception&, const Exception&);
/** deactivates the registry partof a a plugin
  */

    virtual void deactivateRegistryPart(const string& plugin)
      throw(const Exception&, const Exception&);
/** deactivates the factoy part of a plugin
  */

    virtual void deactivateFactoryPart(const string& plugin)
      throw(const Exception&, const Exception&);
/** removes an already loaded plugin with given name plugin_name
  */

    virtual void removeLibrary(const string& plugin_name)
      throw(const Exception&, const Exception&);
/** returns the ObjectID's of a specific plugin
  */

    virtual list<ObjectID >  getPluginID(const string& plugin, const unsigned int& index)
      throw(const Exception&);
/** returns the plugins available in the path list
  */

    virtual list<string >  getPlugins() const
      throw(const Exception&);
/** returns the plugins available in the path list
  */

    virtual list<string >  getUsedPlugins() const
      throw(const Exception&);
/** returns the plugins available in the path list
  */

    virtual list<string >  getPluginPath() const
      throw(const Exception&);


  };

  typedef auto_ptr<IPluginManager> IPluginManager_var;




}

#endif
