
#include "IRegistry.hpp"
#include "nmm/base/InterfaceFactory.hpp"

#include "nmm/base/registry/NodeDescription.hpp"

namespace NMM
{
  
  
  INode* IRegistry::requestNode(const RegistryID& reg, const ObjectID& node_id, const UserInfo& info, NodeDescription& nd) 
      throw(const Exception&, const RegistryException&)
  {
    TOutValue<INode* > return_value;
    TInValue<RegistryID > reg_value(reg, false);
    TInValue<ObjectID > node_id_value(node_id, false);
    TInValue<UserInfo > info_value(info, false);
    TInOutValue<NodeDescription > nd_value;
    nd_value.setValue(nd);
    Event event("IRegistry::requestNode", &reg_value, &node_id_value, &info_value, &nd_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(RegistryException* ex = dynamic_cast<RegistryException*>(event.getException())) {
          throw RegistryException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    nd = nd_value.getRValue();
    return return_value.getRValue();
  }
  
  Interface* IRegistry::requestObject(const RegistryID& reg, const ObjectID& node_id, const UserInfo& info) 
      throw(const Exception&, const RegistryException&)
  {
    TOutValue<Interface* > return_value;
    TInValue<RegistryID > reg_value(reg, false);
    TInValue<ObjectID > node_id_value(node_id, false);
    TInValue<UserInfo > info_value(info, false);
    Event event("IRegistry::requestObject", &reg_value, &node_id_value, &info_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(RegistryException* ex = dynamic_cast<RegistryException*>(event.getException())) {
          throw RegistryException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IRegistry::releaseNode(const RegistryID& reg_id, const NMMReference& ref, const ObjectID& node_id, const UserInfo& user_info)
      throw(const Exception&)
  {
    TInValue<RegistryID > reg_id_value(reg_id, false);
    TInValue<NMMReference > ref_value(ref, false);
    TInValue<ObjectID > node_id_value(node_id, false);
    TInValue<UserInfo > user_info_value(user_info, false);
    Event event("IRegistry::releaseNode", &reg_id_value, &ref_value, &node_id_value, &user_info_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IRegistry::releaseObject(const RegistryID& reg_id, const NMMReference& ref, const ObjectID& node_id, const UserInfo& user_info)
      throw(const Exception&)
  {
    TInValue<RegistryID > reg_id_value(reg_id, false);
    TInValue<NMMReference > ref_value(ref, false);
    TInValue<ObjectID > node_id_value(node_id, false);
    TInValue<UserInfo > user_info_value(user_info, false);
    Event event("IRegistry::releaseObject", &reg_id_value, &ref_value, &node_id_value, &user_info_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  list<Response >  IRegistry::initRequest(const NodeDescription& node_desc) const
      throw(const Exception&)
  {
    TOutValue<list<Response >  > return_value;
    TInValue<NodeDescription > node_desc_value(node_desc, false);
    Event event("IRegistry::initRequest", &node_desc_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<Response >  IRegistry::initObjectRequest(const ObjectDescription& node_desc) const
      throw(const Exception&)
  {
    TOutValue<list<Response >  > return_value;
    TInValue<ObjectDescription > node_desc_value(node_desc, false);
    Event event("IRegistry::initObjectRequest", &node_desc_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<ObjectID >  IRegistry::searchNode(const RegistryID& registry_id, const NodeDescription& node_desc) const
      throw(const Exception&)
  {
    TOutValue<list<ObjectID >  > return_value;
    TInValue<RegistryID > registry_id_value(registry_id, false);
    TInValue<NodeDescription > node_desc_value(node_desc, false);
    Event event("IRegistry::searchNode", &registry_id_value, &node_desc_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  FactoryTicket IRegistry::reserveNode(const RegistryID& reg_id, const ObjectID& node_id, const UserInfo& info) 
      throw(const Exception&, const RegistryException&)
  {
    TOutValue<FactoryTicket > return_value;
    TInValue<RegistryID > reg_id_value(reg_id, false);
    TInValue<ObjectID > node_id_value(node_id, false);
    TInValue<UserInfo > info_value(info, false);
    Event event("IRegistry::reserveNode", &reg_id_value, &node_id_value, &info_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(RegistryException* ex = dynamic_cast<RegistryException*>(event.getException())) {
          throw RegistryException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  INode* IRegistry::instantiateNode(const RegistryID& reg_id, const FactoryTicket& ticket, const UserInfo& info) 
      throw(const Exception&, const RegistryException&)
  {
    TOutValue<INode* > return_value;
    TInValue<RegistryID > reg_id_value(reg_id, false);
    TInValue<FactoryTicket > ticket_value(ticket, false);
    TInValue<UserInfo > info_value(info, false);
    Event event("IRegistry::instantiateNode", &reg_id_value, &ticket_value, &info_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(RegistryException* ex = dynamic_cast<RegistryException*>(event.getException())) {
          throw RegistryException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IRegistry::releaseReservation(const RegistryID& reg_id, const ObjectID& node_id, const UserInfo& info)
      throw(const Exception&)
  {
    TInValue<RegistryID > reg_id_value(reg_id, false);
    TInValue<ObjectID > node_id_value(node_id, false);
    TInValue<UserInfo > info_value(info, false);
    Event event("IRegistry::releaseReservation", &reg_id_value, &node_id_value, &info_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IRegistry::destroyNode(const RegistryID& reg_id, const NMMReference& reference, const UserInfo& info)
      throw(const Exception&)
  {
    TInValue<RegistryID > reg_id_value(reg_id, false);
    TInValue<NMMReference > reference_value(reference, false);
    TInValue<UserInfo > info_value(info, false);
    Event event("IRegistry::destroyNode", &reg_id_value, &reference_value, &info_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  int IRegistry::getMaxInstanceCount(const RegistryID& reg_id, const ObjectID& node_id) const
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    TInValue<RegistryID > reg_id_value(reg_id, false);
    TInValue<ObjectID > node_id_value(node_id, false);
    Event event("IRegistry::getMaxInstanceCount", &reg_id_value, &node_id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  int IRegistry::getInstanceCount(const RegistryID& reg_id, const ObjectID& node_id) const
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    TInValue<RegistryID > reg_id_value(reg_id, false);
    TInValue<ObjectID > node_id_value(node_id, false);
    Event event("IRegistry::getInstanceCount", &reg_id_value, &node_id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string IRegistry::getPluginInfo(const RegistryID& reg_id, const ObjectID& node_id) const
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    TInValue<RegistryID > reg_id_value(reg_id, false);
    TInValue<ObjectID > node_id_value(node_id, false);
    Event event("IRegistry::getPluginInfo", &reg_id_value, &node_id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<ObjectID >  IRegistry::getNodeList(const RegistryID& reg_id) const
      throw(const Exception&)
  {
    TOutValue<list<ObjectID >  > return_value;
    TInValue<RegistryID > reg_id_value(reg_id, false);
    Event event("IRegistry::getNodeList", &reg_id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<ObjectID >  IRegistry::getReservedNodeList(const RegistryID& reg_id) const
      throw(const Exception&)
  {
    TOutValue<list<ObjectID >  > return_value;
    TInValue<RegistryID > reg_id_value(reg_id, false);
    Event event("IRegistry::getReservedNodeList", &reg_id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<NodeInfo >  IRegistry::getNodeInfoList(const RegistryID& reg_id) const
      throw(const Exception&)
  {
    TOutValue<list<NodeInfo >  > return_value;
    TInValue<RegistryID > reg_id_value(reg_id, false);
    Event event("IRegistry::getNodeInfoList", &reg_id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  NodeDescription IRegistry::getNodeDescription(const RegistryID& reg_id, const ObjectID& node_id) const
      throw(const Exception&)
  {
    TOutValue<NodeDescription > return_value;
    TInValue<RegistryID > reg_id_value(reg_id, false);
    TInValue<ObjectID > node_id_value(node_id, false);
    Event event("IRegistry::getNodeDescription", &reg_id_value, &node_id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<NodeDescription >  IRegistry::getNodeDescriptionList(const RegistryID& reg_id)
      throw(const Exception&)
  {
    TOutValue<list<NodeDescription >  > return_value;
    TInValue<RegistryID > reg_id_value(reg_id, false);
    Event event("IRegistry::getNodeDescriptionList", &reg_id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  NodeFactoryID IRegistry::getNodeFactoryID(const RegistryID& reg_id, const ObjectID& node_id) const
      throw(const Exception&)
  {
    TOutValue<NodeFactoryID > return_value;
    TInValue<RegistryID > reg_id_value(reg_id, false);
    TInValue<ObjectID > node_id_value(node_id, false);
    Event event("IRegistry::getNodeFactoryID", &reg_id_value, &node_id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IRegistry::resetInternalReservation()
      throw(const Exception&)
  {
    Event event("IRegistry::resetInternalReservation");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IRegistry::resetSpecificInternalReservation(const UserInfo& user_info)
      throw(const Exception&)
  {
    TInValue<UserInfo > user_info_value(user_info, false);
    Event event("IRegistry::resetSpecificInternalReservation", &user_info_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IRegistry::initTransaction(const UserInfo& info) 
      throw(const Exception&, const RegistryException&)
  {
    TInValue<UserInfo > info_value(info, false);
    Event event("IRegistry::initTransaction", &info_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(RegistryException* ex = dynamic_cast<RegistryException*>(event.getException())) {
          throw RegistryException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void IRegistry::closeTransaction(const UserInfo& info)
      throw(const Exception&)
  {
    TInValue<UserInfo > info_value(info, false);
    Event event("IRegistry::closeTransaction", &info_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  RegistryID IRegistry::getRegistryID() const
      throw(const Exception&)
  {
    TOutValue<RegistryID > return_value;
    Event event("IRegistry::getRegistryID");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  list<RegistryID >  IRegistry::getChildList(const RegistryID& registry_id) const
      throw(const Exception&)
  {
    TOutValue<list<RegistryID >  > return_value;
    TInValue<RegistryID > registry_id_value(registry_id, false);
    Event event("IRegistry::getChildList", &registry_id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  bool IRegistry::isSubRegistry(const RegistryID& reg_id) const
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    TInValue<RegistryID > reg_id_value(reg_id, false);
    Event event("IRegistry::isSubRegistry", &reg_id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  RegistryMemento IRegistry::getMemento()
      throw(const Exception&)
  {
    TOutValue<RegistryMemento > return_value;
    Event event("IRegistry::getMemento");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IRegistry::setMemento(const RegistryMemento& rm)
      throw(const Exception&)
  {
    TInValue<RegistryMemento > rm_value(rm, false);
    Event event("IRegistry::setMemento", &rm_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }

  const char* IRegistry::IRegistry_type = "IRegistry";
  TIFactoryObject<IRegistry > __IRegistry_prototype("IRegistry");


  const char* IRegistry::requestNode_event = "IRegistry::requestNode";
  const char* IRegistry::requestObject_event = "IRegistry::requestObject";
  const char* IRegistry::releaseNode_event = "IRegistry::releaseNode";
  const char* IRegistry::releaseObject_event = "IRegistry::releaseObject";
  const char* IRegistry::initRequest_event = "IRegistry::initRequest";
  const char* IRegistry::initObjectRequest_event = "IRegistry::initObjectRequest";
  const char* IRegistry::searchNode_event = "IRegistry::searchNode";
  const char* IRegistry::reserveNode_event = "IRegistry::reserveNode";
  const char* IRegistry::instantiateNode_event = "IRegistry::instantiateNode";
  const char* IRegistry::releaseReservation_event = "IRegistry::releaseReservation";
  const char* IRegistry::destroyNode_event = "IRegistry::destroyNode";
  const char* IRegistry::getMaxInstanceCount_event = "IRegistry::getMaxInstanceCount";
  const char* IRegistry::getInstanceCount_event = "IRegistry::getInstanceCount";
  const char* IRegistry::getPluginInfo_event = "IRegistry::getPluginInfo";
  const char* IRegistry::getNodeList_event = "IRegistry::getNodeList";
  const char* IRegistry::getReservedNodeList_event = "IRegistry::getReservedNodeList";
  const char* IRegistry::getNodeInfoList_event = "IRegistry::getNodeInfoList";
  const char* IRegistry::getNodeDescription_event = "IRegistry::getNodeDescription";
  const char* IRegistry::getNodeDescriptionList_event = "IRegistry::getNodeDescriptionList";
  const char* IRegistry::getNodeFactoryID_event = "IRegistry::getNodeFactoryID";
  const char* IRegistry::resetInternalReservation_event = "IRegistry::resetInternalReservation";
  const char* IRegistry::resetSpecificInternalReservation_event = "IRegistry::resetSpecificInternalReservation";
  const char* IRegistry::initTransaction_event = "IRegistry::initTransaction";
  const char* IRegistry::closeTransaction_event = "IRegistry::closeTransaction";
  const char* IRegistry::getRegistryID_event = "IRegistry::getRegistryID";
  const char* IRegistry::getChildList_event = "IRegistry::getChildList";
  const char* IRegistry::isSubRegistry_event = "IRegistry::isSubRegistry";
  const char* IRegistry::getMemento_event = "IRegistry::getMemento";
  const char* IRegistry::setMemento_event = "IRegistry::setMemento";


}


