
#ifndef NMM_IServerRegistryImpl_HPP
#define NMM_IServerRegistryImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IServerRegistry.hpp"

#include "IRegistryImpl.hpp"
#include "ISessionImpl.hpp"
#include "../resourcemgr/IResourceManagerImpl.hpp"

namespace NMM
{
/** \class IServerRegistry
  * Base class for all Registry Interfaces.
  */

  class IServerRegistryImpl  : virtual public IRegistryImpl { 
  public:
    IServerRegistryImpl() { addInterface(new IServerRegistry(*this)); registerEvents(); };

    virtual ~IServerRegistryImpl() {};

  protected:
/** This method returns the timout for access control
  */

    virtual unsigned int getTransactionTimeout() = 0;
/** This method sets the timout for access control.
  * \param time_out The new timeout for the access control
  */

    virtual void setTransactionTimeout(const unsigned int& time_out) = 0;
/** This method returns the leasing time_out for nodes.
  */

    virtual unsigned int getNodeLeasingTime() = 0;
/** This method sets the leasing time_out for nodes.
  * \param time_out The new timeout for nodes.
  */

    virtual void setNodeLeasingTime(const unsigned int& time_out) = 0;
/** This method refreshes the reservation of a leased nodes.
  */

    virtual unsigned int refreshNodeLeasing(const RegistryID& reg_id, const ObjectID& node_id, const UserInfo& user_info) = 0;
/** This method refreshes the enabled access control.
  */

    virtual unsigned int refreshTransaction(const UserInfo& user_info) = 0;
/** This method starts the ResourceManager
  */

    virtual void startResourceManager() = 0;
/** This method returns the interface to access the ResourceManager
  * which is started if necessary
  */

    virtual IResourceManager* getResourceManager() = 0;
/** Release the ResourceManager
  */

    virtual void releaseResourceManager() = 0;

    virtual void startIPerfServer() = 0;

    virtual void stopIPerfServer() = 0;

    virtual Interface* requestNMMObject(const ObjectDescription& od) = 0;

    virtual void releaseNMMObject(const Interface& i_face) = 0;

    virtual ISession* requestSharedGraph(GraphDescription& request, const UserInfo& u_info) 
      throw(const RegistryException&) = 0;

    virtual void releaseSession(const ISession& session, const UserInfo& u_info) = 0;

    virtual void releaseSharedNode(const INode& node) = 0;
  /* initialize a new session */

    virtual ISession* initSession(GraphDescription& gd, const UserInfo& u_info, const ISession* given_session) = 0;

    virtual void requestGraph(GraphDescription& gd, const UserInfo& user_info) = 0;


  private:
    void registerEvents();
  };

}

#endif
