
#include "ISession.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  GraphDescription* ISession::getGraphDescription() const
      throw(const Exception&)
  {
    TOutValue<GraphDescription* > return_value;
    Event event("ISession::getGraphDescription");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ICompositeNode* ISession::getCompositeNode() const
      throw(const Exception&)
  {
    TOutValue<ICompositeNode* > return_value;
    Event event("ISession::getCompositeNode");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ISourceSynchronizer* ISession::getSourceSynchronizer() const
      throw(const Exception&)
  {
    TOutValue<ISourceSynchronizer* > return_value;
    Event event("ISession::getSourceSynchronizer");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ISinkSynchronizer* ISession::getSinkSynchronizer() const
      throw(const Exception&)
  {
    TOutValue<ISinkSynchronizer* > return_value;
    Event event("ISession::getSinkSynchronizer");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ISession::setSinkSynchronizer(const ISinkSynchronizer& synchronizer)
      throw(const Exception&)
  {
    TInValue<ISinkSynchronizer > synchronizer_value(synchronizer, false);
    Event event("ISession::setSinkSynchronizer", &synchronizer_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISession::setSourceSynchronizer(const ISourceSynchronizer& synchronizer)
      throw(const Exception&)
  {
    TInValue<ISourceSynchronizer > synchronizer_value(synchronizer, false);
    Event event("ISession::setSourceSynchronizer", &synchronizer_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISession::setGraphDescription(const GraphDescription& gd)
      throw(const Exception&)
  {
    TInValue<GraphDescription > gd_value(gd, false);
    Event event("ISession::setGraphDescription", &gd_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISession::setCompositeNode(const ICompositeNode& comp_node)
      throw(const Exception&)
  {
    TInValue<ICompositeNode > comp_node_value(comp_node, false);
    Event event("ISession::setCompositeNode", &comp_node_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  GraphDescription* ISession::getMasterGraph()
      throw(const Exception&)
  {
    TOutValue<GraphDescription* > return_value;
    Event event("ISession::getMasterGraph");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ISession::setMasterGraph(const GraphDescription& master_graph)
      throw(const Exception&)
  {
    TInValue<GraphDescription > master_graph_value(master_graph, false);
    Event event("ISession::setMasterGraph", &master_graph_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISession::realizeGraph()
      throw(const Exception&)
  {
    Event event("ISession::realizeGraph");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISession::activateGraph()
      throw(const Exception&)
  {
    Event event("ISession::activateGraph");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISession::startGraph()
      throw(const Exception&)
  {
    Event event("ISession::startGraph");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISession::stopGraph()
      throw(const Exception&)
  {
    Event event("ISession::stopGraph");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }

  const char* ISession::ISession_type = "ISession";
  TIFactoryObject<ISession > __ISession_prototype("ISession");


  const char* ISession::getGraphDescription_event = "ISession::getGraphDescription";
  const char* ISession::getCompositeNode_event = "ISession::getCompositeNode";
  const char* ISession::getSourceSynchronizer_event = "ISession::getSourceSynchronizer";
  const char* ISession::getSinkSynchronizer_event = "ISession::getSinkSynchronizer";
  const char* ISession::setSinkSynchronizer_event = "ISession::setSinkSynchronizer";
  const char* ISession::setSourceSynchronizer_event = "ISession::setSourceSynchronizer";
  const char* ISession::setGraphDescription_event = "ISession::setGraphDescription";
  const char* ISession::setCompositeNode_event = "ISession::setCompositeNode";
  const char* ISession::getMasterGraph_event = "ISession::getMasterGraph";
  const char* ISession::setMasterGraph_event = "ISession::setMasterGraph";
  const char* ISession::realizeGraph_event = "ISession::realizeGraph";
  const char* ISession::activateGraph_event = "ISession::activateGraph";
  const char* ISession::startGraph_event = "ISession::startGraph";
  const char* ISession::stopGraph_event = "ISession::stopGraph";


}


