
#ifndef NMM_ISession_HPP
#define NMM_ISession_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "nmm/base/registry/GraphDescription.hpp"
#include "nmm/interfaces/base/sync/ISinkSynchronizer.hpp"
#include "nmm/interfaces/base/sync/ISourceSynchronizer.hpp"
#include "nmm/interfaces/base/graphmgr/ICompositeNode.hpp"

namespace NMM
{

  class ISession : virtual public Interface { 
  public:
    ISession() {  setTypeInfo(TypeInfo(ISession::ISession_type, "NMM")); };

    ISession(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(ISession::ISession_type, "NMM")); setEventReceiver(parent); };

    virtual ~ISession() {};

    static const char* ISession_type;

    virtual  string getInterfaceType() const { return ISession_type; }

    Interface* copy() const { return new ISession(*this); }
    static const char* getGraphDescription_event;
    static const char* getCompositeNode_event;
    static const char* getSourceSynchronizer_event;
    static const char* getSinkSynchronizer_event;
    static const char* setSinkSynchronizer_event;
    static const char* setSourceSynchronizer_event;
    static const char* setGraphDescription_event;
    static const char* setCompositeNode_event;
    static const char* getMasterGraph_event;
    static const char* setMasterGraph_event;
    static const char* realizeGraph_event;
    static const char* activateGraph_event;
    static const char* startGraph_event;
    static const char* stopGraph_event;
    /**
     * Returns the graph description of this sesssion.
     */

    virtual GraphDescription* getGraphDescription() const
      throw(const Exception&);
    /**
     * Returns the composite node of this sesssion.
     */

    virtual ICompositeNode* getCompositeNode() const
      throw(const Exception&);
    /**
     * Returns the source synchronizer of this sesssion.
     */

    virtual ISourceSynchronizer* getSourceSynchronizer() const
      throw(const Exception&);
    /**
     * Returns the sink synchronizer of this sesssion.
     */

    virtual ISinkSynchronizer* getSinkSynchronizer() const
      throw(const Exception&);
    /**
     * Sets the sink synchronizer for the session.
     */

    virtual void setSinkSynchronizer(const ISinkSynchronizer& synchronizer)
      throw(const Exception&);
    /**
     * Sets the source synchronizer for the session.
     */

    virtual void setSourceSynchronizer(const ISourceSynchronizer& synchronizer)
      throw(const Exception&);
    /**
     * Sets the graph description for the session.
     */

    virtual void setGraphDescription(const GraphDescription& gd)
      throw(const Exception&);
    /**
     * Sets the composite node for the session.
     */

    virtual void setCompositeNode(const ICompositeNode& comp_node)
      throw(const Exception&);

    virtual GraphDescription* getMasterGraph()
      throw(const Exception&);

    virtual void setMasterGraph(const GraphDescription& master_graph)
      throw(const Exception&);

    virtual void realizeGraph()
      throw(const Exception&);

    virtual void activateGraph()
      throw(const Exception&);

    virtual void startGraph()
      throw(const Exception&);

    virtual void stopGraph()
      throw(const Exception&);


  };

  typedef auto_ptr<ISession> ISession_var;




}

#endif
