
#ifndef NMM_ISessionImpl_HPP
#define NMM_ISessionImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "ISession.hpp"

#include "nmm/interfaces/base/sync/ISinkSynchronizerImpl.hpp"
#include "nmm/interfaces/base/sync/ISourceSynchronizerImpl.hpp"
#include "nmm/interfaces/base/graphmgr/ICompositeNodeImpl.hpp"

namespace NMM
{

  class ISessionImpl  : public virtual NMMObject { 
  public:
    ISessionImpl() { addInterface(new ISession(*this)); registerEvents(); };

    virtual ~ISessionImpl() {};

  protected:
    /**
     * Returns the graph description of this sesssion.
     */

    virtual GraphDescription* getGraphDescription() const = 0;
    /**
     * Returns the composite node of this sesssion.
     */

    virtual ICompositeNode* getCompositeNode() const = 0;
    /**
     * Returns the source synchronizer of this sesssion.
     */

    virtual ISourceSynchronizer* getSourceSynchronizer() const = 0;
    /**
     * Returns the sink synchronizer of this sesssion.
     */

    virtual ISinkSynchronizer* getSinkSynchronizer() const = 0;
    /**
     * Sets the sink synchronizer for the session.
     */

    virtual void setSinkSynchronizer(const ISinkSynchronizer& synchronizer) = 0;
    /**
     * Sets the source synchronizer for the session.
     */

    virtual void setSourceSynchronizer(const ISourceSynchronizer& synchronizer) = 0;
    /**
     * Sets the graph description for the session.
     */

    virtual void setGraphDescription(const GraphDescription& gd) = 0;
    /**
     * Sets the composite node for the session.
     */

    virtual void setCompositeNode(const ICompositeNode& comp_node) = 0;

    virtual GraphDescription* getMasterGraph() = 0;

    virtual void setMasterGraph(const GraphDescription& master_graph) = 0;

    virtual void realizeGraph() = 0;

    virtual void activateGraph() = 0;

    virtual void startGraph() = 0;

    virtual void stopGraph() = 0;


  private:
    void registerEvents();
  };

}

#endif
