
#include "IResourceManager.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void IResourceManager::setNetworkInterfaces(const unsigned int& num)
      throw(const Exception&)
  {
    TInValue<unsigned int > num_value(num, false);
    Event event("IResourceManager::setNetworkInterfaces", &num_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  Result IResourceManager::addNode(const INode* node, unsigned int& id, const unsigned int& interf /* = 0 */)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<INode* > node_value (const_cast<INode* >(node), false);
    TOutValue<unsigned int > id_value(id);
    TInValue<unsigned int > interf_value(interf, false);
    Event event("IResourceManager::addNode", &node_value, &id_value, &interf_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    id = id_value.getRValue();
    return return_value.getRValue();
  }
  
  Result IResourceManager::releaseNode(const INode* node, const unsigned int& id)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<INode* > node_value (const_cast<INode* >(node), false);
    TInValue<unsigned int > id_value(id, false);
    Event event("IResourceManager::releaseNode", &node_value, &id_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IResourceManager::getCPUInfo(string& name, uint& bogo_mips, uint& num, float& free)
      throw(const Exception&)
  {
    TOutValue<string > name_value(name);
    TOutValue<uint > bogo_mips_value(bogo_mips);
    TOutValue<uint > num_value(num);
    TOutValue<float > free_value(free);
    Event event("IResourceManager::getCPUInfo", &name_value, &bogo_mips_value, &num_value, &free_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    name = name_value.getRValue();
    bogo_mips = bogo_mips_value.getRValue();
    num = num_value.getRValue();
    free = free_value.getRValue();
  }
  
  void IResourceManager::getCPUSpeed(uint& bogo_mips, float& free)
      throw(const Exception&)
  {
    TOutValue<uint > bogo_mips_value(bogo_mips);
    TOutValue<float > free_value(free);
    Event event("IResourceManager::getCPUSpeed", &bogo_mips_value, &free_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    bogo_mips = bogo_mips_value.getRValue();
    free = free_value.getRValue();
  }
  
  void IResourceManager::getCPUFree(float& free)
      throw(const Exception&)
  {
    TOutValue<float > free_value(free);
    Event event("IResourceManager::getCPUFree", &free_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    free = free_value.getRValue();
  }
  
  void IResourceManager::setCPUUpdateIntervall(const unsigned int& seconds)
      throw(const Exception&)
  {
    TInValue<unsigned int > seconds_value(seconds, false);
    Event event("IResourceManager::setCPUUpdateIntervall", &seconds_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IResourceManager::getCPULeft(float& left)
      throw(const Exception&)
  {
    TOutValue<float > left_value(left);
    Event event("IResourceManager::getCPULeft", &left_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    left = left_value.getRValue();
  }
  
  void IResourceManager::setReservationPending()
      throw(const Exception&)
  {
    Event event("IResourceManager::setReservationPending");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IResourceManager::commitReservation()
      throw(const Exception&)
  {
    Event event("IResourceManager::commitReservation");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IResourceManager::startServer()
      throw(const Exception&)
  {
    Event event("IResourceManager::startServer");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IResourceManager::stopServer()
      throw(const Exception&)
  {
    Event event("IResourceManager::stopServer");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  float IResourceManager::testConnection(const string& hostname)
      throw(const Exception&)
  {
    TOutValue<float > return_value;
    TInValue<string > hostname_value(hostname, false);
    Event event("IResourceManager::testConnection", &hostname_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IResourceManager::testConnections(const list<string > & hosts, list<float > & speeds)
      throw(const Exception&)
  {
    TInValue<list<string >  > hosts_value(hosts, false);
    TOutValue<list<float >  > speeds_value(speeds);
    Event event("IResourceManager::testConnections", &hosts_value, &speeds_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    speeds = speeds_value.getRValue();
  }
  
  Result IResourceManager::setHosts(const list<string > & hosts)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<list<string >  > hosts_value(hosts, false);
    Event event("IResourceManager::setHosts", &hosts_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IResourceManager::getCPUDescription(string& description)
      throw(const Exception&)
  {
    TOutValue<string > description_value(description);
    Event event("IResourceManager::getCPUDescription", &description_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    description = description_value.getRValue();
  }
  
  Result IResourceManager::updateNodeInfo(const INode* node)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<INode* > node_value (const_cast<INode* >(node), false);
    Event event("IResourceManager::updateNodeInfo", &node_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IResourceManager::IResourceManager_type = "IResourceManager";
  TIFactoryObject<IResourceManager > __IResourceManager_prototype("IResourceManager");


  const char* IResourceManager::setNetworkInterfaces_event = "IResourceManager::setNetworkInterfaces";
  const char* IResourceManager::addNode_event = "IResourceManager::addNode";
  const char* IResourceManager::releaseNode_event = "IResourceManager::releaseNode";
  const char* IResourceManager::getCPUInfo_event = "IResourceManager::getCPUInfo";
  const char* IResourceManager::getCPUSpeed_event = "IResourceManager::getCPUSpeed";
  const char* IResourceManager::getCPUFree_event = "IResourceManager::getCPUFree";
  const char* IResourceManager::setCPUUpdateIntervall_event = "IResourceManager::setCPUUpdateIntervall";
  const char* IResourceManager::getCPULeft_event = "IResourceManager::getCPULeft";
  const char* IResourceManager::setReservationPending_event = "IResourceManager::setReservationPending";
  const char* IResourceManager::commitReservation_event = "IResourceManager::commitReservation";
  const char* IResourceManager::startServer_event = "IResourceManager::startServer";
  const char* IResourceManager::stopServer_event = "IResourceManager::stopServer";
  const char* IResourceManager::testConnection_event = "IResourceManager::testConnection";
  const char* IResourceManager::testConnections_event = "IResourceManager::testConnections";
  const char* IResourceManager::setHosts_event = "IResourceManager::setHosts";
  const char* IResourceManager::getCPUDescription_event = "IResourceManager::getCPUDescription";
  const char* IResourceManager::updateNodeInfo_event = "IResourceManager::updateNodeInfo";


}


