
#ifndef NMM_IResourceManager_HPP
#define NMM_IResourceManager_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "nmm/interfaces/base/INode.hpp"

namespace NMM
{

  class IResourceManager : virtual public Interface { 
  public:
    IResourceManager() {  setTypeInfo(TypeInfo(IResourceManager::IResourceManager_type, "NMM")); };

    IResourceManager(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IResourceManager::IResourceManager_type, "NMM")); setEventReceiver(parent); };

    virtual ~IResourceManager() {};

    static const char* IResourceManager_type;

    virtual  string getInterfaceType() const { return IResourceManager_type; }

    Interface* copy() const { return new IResourceManager(*this); }
    static const char* setNetworkInterfaces_event;
    static const char* addNode_event;
    static const char* releaseNode_event;
    static const char* getCPUInfo_event;
    static const char* getCPUSpeed_event;
    static const char* getCPUFree_event;
    static const char* setCPUUpdateIntervall_event;
    static const char* getCPULeft_event;
    static const char* setReservationPending_event;
    static const char* commitReservation_event;
    static const char* startServer_event;
    static const char* stopServer_event;
    static const char* testConnection_event;
    static const char* testConnections_event;
    static const char* setHosts_event;
    static const char* getCPUDescription_event;
    static const char* updateNodeInfo_event;
    /**
     * set the number of usable network interfaces
     * this clears all previous reservations
     */

    virtual void setNetworkInterfaces(const unsigned int& num)
      throw(const Exception&);
    /**
     * add a connected Node to reserve the needed cpu resources
     */

    virtual Result addNode(const INode* node, unsigned int& id, const unsigned int& interf = 0)
      throw(const Exception&);
    /**
     * release a previously added node (and free its reserved resources)
     */

    virtual Result releaseNode(const INode* node, const unsigned int& id)
      throw(const Exception&);
    /** This method returns the name, number, speed (MHz and BogoMIPS) and % of free computation
      * time of the cpu on the machine the registry runs on
      */

    virtual void getCPUInfo(string& name, uint& bogo_mips, uint& num, float& free)
      throw(const Exception&);
    /** This method returns the speed (BogoMIPS) and % of free computation time of the cpu on the machine the registry runs on
      */

    virtual void getCPUSpeed(uint& bogo_mips, float& free)
      throw(const Exception&);
   /**
    * \returns the amount of free cpu-time within one second, accumulated on all available cpus
    */

    virtual void getCPUFree(float& free)
      throw(const Exception&);
   /**
    * \set the update intervall of cpuInfo-polling
    */

    virtual void setCPUUpdateIntervall(const unsigned int& seconds)
      throw(const Exception&);
   /**
    * \returns the amount of free cpu-time within one second, accumulated on all available cpus
    */

    virtual void getCPULeft(float& left)
      throw(const Exception&);
   /**
    *
    */

    virtual void setReservationPending()
      throw(const Exception&);
    /**
     *
     */

    virtual void commitReservation()
      throw(const Exception&);
   /**
     * start an iperf-server
     */

    virtual void startServer()
      throw(const Exception&);
    /**
     * stop the iperf-server
     */

    virtual void stopServer()
      throw(const Exception&);
    /**
     * test the speed of a connection to hostname
     */

    virtual float testConnection(const string& hostname)
      throw(const Exception&);
    /**
     * test the speeds of the simultaneous connections to all specified hosts
     * \param hosts a list of hosts to test
     * \param speeds an empty list that will be filled with the measured speeds
     */

    virtual void testConnections(const list<string > & hosts, list<float > & speeds)
      throw(const Exception&);

    virtual Result setHosts(const list<string > & hosts)
      throw(const Exception&);

    virtual void getCPUDescription(string& description)
      throw(const Exception&);

    virtual Result updateNodeInfo(const INode* node)
      throw(const Exception&);


  };

  typedef auto_ptr<IResourceManager> IResourceManager_var;




}

#endif
