
#ifndef NMM_IControllerImpl_HPP
#define NMM_IControllerImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IController.hpp"


namespace NMM
{

  class IPingImpl  : public virtual NMMObject { 
  public:
    IPingImpl() { addInterface(new IPing(*this)); registerEvents(); };

    virtual ~IPingImpl() {};

  protected:
    /**
     * Can be used to estimate the delay between method call and execution
     * @return the system time when the method call was executed
     */

    virtual Time ping() = 0;


  private:
    void registerEvents();
  };

  class IControllerImpl  : virtual public IPingImpl { 
  public:
    IControllerImpl() { addInterface(new IController(*this)); registerEvents(); };

    virtual ~IControllerImpl() {};

  protected:

    virtual Result setMode(const IController::Mode& value) = 0;
    /**
     * Permit discarding buffers.
     * Set the flag that allows or disallows discarding buffers.
     * Set by default. In ONLINE mode, the controller may discard buffers if this flag is set.
     */

    virtual Result permitDiscardBuffers(const bool& value) = 0;
    /**
     * The controller is told to pause the node via this method
     */

    virtual Result pause() = 0;
    /**
     * The controller is told to wake up the node via this method
     */

    virtual Result wakeup() = 0;
    /**
     * The controller is told to reset the node via this method
     */

    virtual Result reset() = 0;
    /**
     * Sets whether buffers should always be regarded as invalid (and therefore not presented)
     * or if the usual checks should be performed to check whether a buffer is valid or not.
     * This allows to enable/disable a node.
     * @param status true for marking all buffers as invalid
     */

    virtual void setBuffersInvalid(const bool& status) = 0;
    /**
     * Sets whether buffers should always be regarded as invalid (and therefore not presented)
     * or if the usual checks should be performed to check whether a buffer is valid or not.
     * This allows to enable/disable a node starting from a particular time.
     * @param status true for marking all buffers as invalid
     * @param status_time timestamp from which on the new status should be active
     */

    virtual void setBuffersInvalidTimed(const bool& status, const Timestamp& status_time) = 0;


  private:
    void registerEvents();
  };

}

#endif
