
#include "IMasterSlaveSourceSynchronizer.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  Result IMasterSlaveSourceSynchronizer::setStartDelay(const Interval& interval)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<Interval > interval_value(interval, false);
    Event event("IMasterSlaveSourceSynchronizer::setStartDelay", &interval_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IMasterSlaveSourceSynchronizer::setMasterController(const ISourceController* value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<ISourceController* > value_value (const_cast<ISourceController* >(value), false);
    Event event("IMasterSlaveSourceSynchronizer::setMasterController", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IMasterSlaveSourceSynchronizer::addSlaveController(const ISourceController* value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<ISourceController* > value_value (const_cast<ISourceController* >(value), false);
    Event event("IMasterSlaveSourceSynchronizer::addSlaveController", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IMasterSlaveSourceSynchronizer::removeSlaveController(const ISourceController* value)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<ISourceController* > value_value (const_cast<ISourceController* >(value), false);
    Event event("IMasterSlaveSourceSynchronizer::removeSlaveController", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IMasterSlaveSourceSynchronizer::resetAll()
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    Event event("IMasterSlaveSourceSynchronizer::resetAll");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IMasterSlaveSourceSynchronizer::IMasterSlaveSourceSynchronizer_type = "IMasterSlaveSourceSynchronizer";
  TIFactoryObject<IMasterSlaveSourceSynchronizer > __IMasterSlaveSourceSynchronizer_prototype("IMasterSlaveSourceSynchronizer");


  const char* IMasterSlaveSourceSynchronizer::setStartDelay_event = "IMasterSlaveSourceSynchronizer::setStartDelay";
  const char* IMasterSlaveSourceSynchronizer::setMasterController_event = "IMasterSlaveSourceSynchronizer::setMasterController";
  const char* IMasterSlaveSourceSynchronizer::addSlaveController_event = "IMasterSlaveSourceSynchronizer::addSlaveController";
  const char* IMasterSlaveSourceSynchronizer::removeSlaveController_event = "IMasterSlaveSourceSynchronizer::removeSlaveController";
  const char* IMasterSlaveSourceSynchronizer::resetAll_event = "IMasterSlaveSourceSynchronizer::resetAll";


}


