
#ifndef NMM_IMasterSlaveSourceSynchronizerImpl_HPP
#define NMM_IMasterSlaveSourceSynchronizerImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IMasterSlaveSourceSynchronizer.hpp"

#include "ISourceSynchronizerImpl.hpp"

namespace NMM
{
  /**
   * Master-slave source synchronizer.
   * Interface for a synchronizer which performs inter-stream synchronization between
   * one master and multiple slave sources.
   */

  class IMasterSlaveSourceSynchronizerImpl  : virtual public ISourceSynchronizerImpl { 
  public:
    IMasterSlaveSourceSynchronizerImpl() { addInterface(new IMasterSlaveSourceSynchronizer(*this)); registerEvents(); };

    virtual ~IMasterSlaveSourceSynchronizerImpl() {};

  protected:
    /**
     * Sets the time interval to wait before starting after start() is called.
     * This allows the MasterSlaveSourceSynchronizer to tell its controllers the
     * delay time.
     * With the help of a start delay, the nodes
     * are told to start at the current time + delay to avoid scheduling
     * problems (as one node would always be started some time after the other).
     * This is done by setting calling the setStartTime-Method of the controllers
     * with (current_time + delay).
     */

    virtual Result setStartDelay(const Interval& interval) = 0;
    /**
     * Sets the master (audio) source node.
     * This allows the MasterSlaveSourceSynchronizer to connect to its master controller.
     */

    virtual Result setMasterController(const ISourceController* value) = 0;
    /**
     * Adds a slave source node. (e.g. video)
     * This allows the MasterSlaveSourceSynchronizer to connect to its slave controller(s).
     */

    virtual Result addSlaveController(const ISourceController* value) = 0;
    /**
     * Removes a slave source node. (e.g. video)
     */

    virtual Result removeSlaveController(const ISourceController* value) = 0;
    /**
     * Resets the MasterSlaveSourceSynchronizer.
     * Only internal variables are resetted, you don't need to set the
     * audio and video source after a reset - call.
     * The reset-method is normally not called directly, but by the
     * ISourceControllers.
     * It switches the state between WORKING and RESET.
     */

    virtual Result resetAll() = 0;


  private:
    void registerEvents();
  };

}

#endif
