
#ifndef NMM_ISourceController_HPP
#define NMM_ISourceController_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "IController.hpp"

namespace NMM
{

  class ISourceSynchronizer;
  /**
   * SourceController.
   * Interface for a source controller which helps the
   * SourceSynchronizer performing synchronization by
   * controlling the appropriate nodes
   */

  class ISourceController : virtual public IController { 
  public:
    ISourceController() {  setTypeInfo(TypeInfo(ISourceController::ISourceController_type, "NMM")); };

    ISourceController(NMMObject& parent) : IController(parent) {  setTypeInfo(TypeInfo(ISourceController::ISourceController_type, "NMM")); setEventReceiver(parent); };

    virtual ~ISourceController() {};

    static const char* ISourceController_type;

    virtual  string getInterfaceType() const { return ISourceController_type; }

    Interface* copy() const { return new ISourceController(*this); }
    static const char* setMasterSkew_event;
    static const char* setStartTime_event;
    static const char* setSourceSynchronizer_event;
    static const char* setSyncMaster_event;
    /**
     * This method sets the time skew from the master of synchronization
     */

    virtual void setMasterSkew(const Interval& interval)
      throw(const Exception&);
    /**
     * This method sets the start time for the controller. With the help of a start time, the nodes
     * can be told to start at e.g. 1 seconds after instantiating the Clock. This avoids scheduling
     * problems as one node would always be started some time after the other.
     */

    virtual void setStartTime(const Time& time)
      throw(const Exception&);
    /**
     * This method sets the Synchronizer this Controller shall communicate with
     */

    virtual Result setSourceSynchronizer(const ISourceSynchronizer* value)
      throw(const Exception&);
    /**
     * This method propagates the master flag to the node
     */

    virtual void setSyncMaster(const bool& master)
      throw(const Exception&);


  };

  typedef auto_ptr<ISourceController> ISourceController_var;




}

#endif
