
#ifndef NMM_ISynchronized_HPP
#define NMM_ISynchronized_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "ISyncReset.hpp"

namespace NMM
{

  class ISynchronized : virtual public ISyncReset { 
  public:
    ISynchronized() {  setTypeInfo(TypeInfo(ISynchronized::ISynchronized_type, "NMM")); };

    ISynchronized(NMMObject& parent) : ISyncReset(parent) {  setTypeInfo(TypeInfo(ISynchronized::ISynchronized_type, "NMM")); setEventReceiver(parent); };

    virtual ~ISynchronized() {};

    static const char* ISynchronized_type;

    virtual  string getInterfaceType() const { return ISynchronized_type; }

    Interface* copy() const { return new ISynchronized(*this); }
    static const char* setSynchronized_event;
    static Event* create_setSynchronized(const bool& toggle);
    static const char* isSynchronized_event;
  /* instream (protected) and non-instream (public) */        /**
     * With this method synchronization of the sync element can be switched on and off.
     * \param toggle true  if the object should be synchronized
     *               false if the object simply presents the buffers as
     *                     fast as possible (default mode).
     */

    virtual Result setSynchronized(const bool& toggle) /* in-stream */ 
      throw(const Exception&);
  /* non-instream (public) */        /**
     * Get the status of the object, synchronized, or not synchronized
     */

    virtual bool isSynchronized()
      throw(const Exception&);


  };

  typedef auto_ptr<ISynchronized> ISynchronized_var;




}

#endif
