
#ifndef NMM_ISynchronizedSource_HPP
#define NMM_ISynchronizedSource_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "ISynchronized.hpp"
#include "ISourceController.hpp"

namespace NMM
{

  class ISynchronizedSource : virtual public ISynchronized { 
  public:
    ISynchronizedSource() {  setTypeInfo(TypeInfo(ISynchronizedSource::ISynchronizedSource_type, "NMM")); };

    ISynchronizedSource(NMMObject& parent) : ISynchronized(parent) {  setTypeInfo(TypeInfo(ISynchronizedSource::ISynchronizedSource_type, "NMM")); setEventReceiver(parent); };

    virtual ~ISynchronizedSource() {};

    static const char* ISynchronizedSource_type;

    virtual  string getInterfaceType() const { return ISynchronizedSource_type; }

    Interface* copy() const { return new ISynchronizedSource(*this); }
    static const char* getController_event;
    static const char* setSyncMaster_event;
    static const char* isSyncMaster_event;
    static const char* setConstantRateProducer_event;
    static const char* isConstantRateProducer_event;
    /**
     * Get the controller for this node.
     * The controller interface is needed by the synchronizer to communicate with it
     * and through it with the node
     */

    virtual ISourceController* getController()
      throw(const Exception&);
    /**
     * With this method the node can be told to be the master of synchronization
     *  \param master true  if the node should be master
     *                false if not
     */

    virtual void setSyncMaster(const bool& master)
      throw(const Exception&);
    /**
     * This method returns whether the node is the master of synchronization
     */

    virtual bool isSyncMaster()
      throw(const Exception&);
    /**
     * With this method one can set whether the underlying node produces its data
     * at a constant rate or not.
     *  \param master true  if the node produces its data at a constant rate
     *                false if not
     */

    virtual void setConstantRateProducer(const bool& const_rate)
      throw(const Exception&);
    /**
     * This method returns whether the node produces its data at a constant rate.
     */

    virtual bool isConstantRateProducer()
      throw(const Exception&);


  };

  typedef auto_ptr<ISynchronizedSource> ISynchronizedSource_var;




}

#endif
