
#include "ICDDB.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  Result ICDDB::setCDDBServer(const string& host, const int& port)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<string > host_value(host, false);
    TInValue<int > port_value(port, false);
    Event event("ICDDB::setCDDBServer", &host_value, &port_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string ICDDB::getTrackName(const int& track)
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    TInValue<int > track_value(track, false);
    Event event("ICDDB::getTrackName", &track_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string ICDDB::getDiscTitle()
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    Event event("ICDDB::getDiscTitle");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string ICDDB::getDiscArtist()
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    Event event("ICDDB::getDiscArtist");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ICDDB::setCDDB(const bool& enabled)
      throw(const Exception&)
  {
    TInValue<bool > enabled_value(enabled, false);
    Event event("ICDDB::setCDDB", &enabled_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  bool ICDDB::getCDDB()
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("ICDDB::getCDDB");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ICDDB::ICDDB_type = "ICDDB";
  TIFactoryObject<ICDDB > __ICDDB_prototype("ICDDB");


  const char* ICDDB::setCDDBServer_event = "ICDDB::setCDDBServer";
  const char* ICDDB::getTrackName_event = "ICDDB::getTrackName";
  const char* ICDDB::getDiscTitle_event = "ICDDB::getDiscTitle";
  const char* ICDDB::getDiscArtist_event = "ICDDB::getDiscArtist";
  const char* ICDDB::setCDDB_event = "ICDDB::setCDDB";
  const char* ICDDB::getCDDB_event = "ICDDB::getCDDB";


}


