
#include "IDVD.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  Result IDVD::select(const unsigned int& title, const unsigned int& chapter, const unsigned int& angle)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<unsigned int > title_value(title, false);
    TInValue<unsigned int > chapter_value(chapter, false);
    TInValue<unsigned int > angle_value(angle, false);
    Event event("IDVD::select", &title_value, &chapter_value, &angle_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int IDVD::getCurrentTitle() const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    Event event("IDVD::getCurrentTitle");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int IDVD::getCurrentChapter() const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    Event event("IDVD::getCurrentChapter");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int IDVD::getTitles() const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    Event event("IDVD::getTitles");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int IDVD::getChapters(const unsigned int& title) const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    TInValue<unsigned int > title_value(title, false);
    Event event("IDVD::getChapters", &title_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int IDVD::getAngles(const unsigned int& title) const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    TInValue<unsigned int > title_value(title, false);
    Event event("IDVD::getAngles", &title_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IDVD::sendFlush()
      throw(const Exception&)
  {
    Event event("IDVD::sendFlush");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IDVD::reset()
      throw(const Exception&)
  {
    Event event("IDVD::reset");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }

  const char* IDVD::IDVD_type = "IDVD";
  TIFactoryObject<IDVD > __IDVD_prototype("IDVD");


  const char* IDVD::select_event = "IDVD::select";
  const char* IDVD::getCurrentTitle_event = "IDVD::getCurrentTitle";
  const char* IDVD::getCurrentChapter_event = "IDVD::getCurrentChapter";
  const char* IDVD::getTitles_event = "IDVD::getTitles";
  const char* IDVD::getChapters_event = "IDVD::getChapters";
  const char* IDVD::getAngles_event = "IDVD::getAngles";
  const char* IDVD::sendFlush_event = "IDVD::sendFlush";
  const char* IDVD::reset_event = "IDVD::reset";


}


