
#ifndef NMM_IFileModeHandler_HPP
#define NMM_IFileModeHandler_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{

  class IFileModeHandler : virtual public Interface { 
  public:
  enum FileMode { 
    FILE_PER_FRAME = 0 , 
    FILE_PER_STREAM  };

  enum FileFormat { 
    DATA_ONLY = 0 , 
    SIZE_DATA , 
    SIZE_TIMESTAMP_DATA  };

  enum FilenameMode { 
    FILENAME_ONLY = 0 , 
    FILENAME_BUFCOUNT , 
    TIMESTAMP_AS_NAME  };

    IFileModeHandler() {  setTypeInfo(TypeInfo(IFileModeHandler::IFileModeHandler_type, "NMM")); };

    IFileModeHandler(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IFileModeHandler::IFileModeHandler_type, "NMM")); setEventReceiver(parent); };

    virtual ~IFileModeHandler() {};

    static const char* IFileModeHandler_type;

    virtual  string getInterfaceType() const { return IFileModeHandler_type; }

    Interface* copy() const { return new IFileModeHandler(*this); }
    static const char* setMode_event;
    static const char* getMode_event;
    static const char* setFormat_event;
    static const char* getFormat_event;
    static const char* setFilenameMode_event;
    static const char* getFilenameMode_event;
    static const char* setLoop_event;

    virtual void setMode(const IFileModeHandler::FileMode& mode)
      throw(const Exception&);

    virtual IFileModeHandler::FileMode getMode()
      throw(const Exception&);

    virtual void setFormat(const IFileModeHandler::FileFormat& format)
      throw(const Exception&);

    virtual IFileModeHandler::FileFormat getFormat()
      throw(const Exception&);

    virtual void setFilenameMode(const IFileModeHandler::FilenameMode& mode)
      throw(const Exception&);

    virtual IFileModeHandler::FilenameMode getFilenameMode()
      throw(const Exception&);

    virtual void setLoop(const bool& d)
      throw(const Exception&);


  };

  typedef auto_ptr<IFileModeHandler> IFileModeHandler_var;

  OValueStream& operator<<(OValueStream& ov, const IFileModeHandler::FileMode& t);
  IValueStream& operator>>(IValueStream& iv, IFileModeHandler::FileMode& t);
  OValueStream& operator<<(OValueStream& ov, const IFileModeHandler::FileFormat& t);
  IValueStream& operator>>(IValueStream& iv, IFileModeHandler::FileFormat& t);
  OValueStream& operator<<(OValueStream& ov, const IFileModeHandler::FilenameMode& t);
  IValueStream& operator>>(IValueStream& iv, IFileModeHandler::FilenameMode& t);



}

#endif
