
#ifndef NMM_ISeekable_HPP
#define NMM_ISeekable_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{

  class ISeekable : virtual public Interface { 
  public:
    ISeekable() {  setTypeInfo(TypeInfo(ISeekable::ISeekable_type, "NMM")); };

    ISeekable(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(ISeekable::ISeekable_type, "NMM")); setEventReceiver(parent); };

    virtual ~ISeekable() {};

    static const char* ISeekable_type;

    virtual  string getInterfaceType() const { return ISeekable_type; }

    Interface* copy() const { return new ISeekable(*this); }
    static const char* hasPercentSeek_event;
    static const char* seekPercent_event;
    static const char* seekPercentTo_event;
    static const char* hasBytesSeek_event;
    static const char* seekBytes_event;
    static const char* seekBytesTo_event;
    static const char* hasTimeSeek_event;
    static const char* seekTime_event;
    static const char* seekTimeTo_event;
    /**
     * this method can be used to test if this interface supports
     * seeking by percentage.
     * \returns true iff percantage seeking is supported.
     */

    virtual bool hasPercentSeek() const
      throw(const Exception&);
    /**
     * this method seeks relative from the current position in
     * percent.
     * \param percent The offset to seek in a Rational from -1 to 1.
     */

    virtual void seekPercent(const Rational& percent)
      throw(const Exception&);
    /**
     * this method seeks absolute to the given position in percent.
     * \param percent The position to seek to in a Rational from 0 to 1.
     */

    virtual void seekPercentTo(const Rational& percent)
      throw(const Exception&);
    /**
     * this method can be used to test if this interface supports
     * seeking by bytes.
     * \returns true iff bytewise seeking is supported.
     */

    virtual bool hasBytesSeek() const
      throw(const Exception&);
    /**
     * this method seeks relative from the current position in
     * bytes.
     * \param offset The offset to seek.
     */

    virtual void seekBytes(const int64_t& offset)
      throw(const Exception&);
    /**
     * this method seeks absolute to the given position in percent.
     * \param bytes The position to seek to.
     */

    virtual void seekBytesTo(const u_int64_t& bytes)
      throw(const Exception&);
    /**
     * this method can be used to test if this interface supports
     * seeking by Time.
     * \returns true iff seeking by Time is supported.
     */

    virtual bool hasTimeSeek() const
      throw(const Exception&);
    /**
     * this method seeks relative from the current position in
     * Time.
     * \param interval The Interval to seek.
     */

    virtual void seekTime(const Interval& interval)
      throw(const Exception&);
    /**
     * this method seeks absolute to the given position in Time.
     * \param time The Time to seek to.
     */

    virtual void seekTimeTo(const Time& time)
      throw(const Exception&);


  };

  typedef auto_ptr<ISeekable> ISeekable_var;




}

#endif
