
#include "IBufferDrop.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void IBufferDrop::setBufferToDrop(const unsigned int& buffer_num)
      throw(const Exception&)
  {
    TInValue<unsigned int > buffer_num_value(buffer_num, false);
    Event event("IBufferDrop::setBufferToDrop", &buffer_num_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IBufferDrop::setDropRatio(const unsigned int& numerator, const unsigned int& denominator)
      throw(const Exception&)
  {
    TInValue<unsigned int > numerator_value(numerator, false);
    TInValue<unsigned int > denominator_value(denominator, false);
    Event event("IBufferDrop::setDropRatio", &numerator_value, &denominator_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }

  const char* IBufferDrop::IBufferDrop_type = "IBufferDrop";
  TIFactoryObject<IBufferDrop > __IBufferDrop_prototype("IBufferDrop");


  const char* IBufferDrop::setBufferToDrop_event = "IBufferDrop::setBufferToDrop";
  const char* IBufferDrop::setDropRatio_event = "IBufferDrop::setDropRatio";
  
  
  void ITimedBufferDrop::addInOutPoint(const unsigned int& in_secs, const unsigned int& in_millisecs, const unsigned int& out_secs, const unsigned int& out_millisecs) /* limited(INode::CONSTRUCTED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TInValue<unsigned int > in_secs_value(in_secs, false);
    TInValue<unsigned int > in_millisecs_value(in_millisecs, false);
    TInValue<unsigned int > out_secs_value(out_secs, false);
    TInValue<unsigned int > out_millisecs_value(out_millisecs, false);
    Event event("ITimedBufferDrop::addInOutPoint", &in_secs_value, &in_millisecs_value, &out_secs_value, &out_millisecs_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void ITimedBufferDrop::useRelativeInOutPoints(const bool& relative) /* limited(INode::CONSTRUCTED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TInValue<bool > relative_value(relative, false);
    Event event("ITimedBufferDrop::useRelativeInOutPoints", &relative_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  bool ITimedBufferDrop::isRelativeInOutPoints()
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("ITimedBufferDrop::isRelativeInOutPoints");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ITimedBufferDrop::ITimedBufferDrop_type = "ITimedBufferDrop";
  TIFactoryObject<ITimedBufferDrop > __ITimedBufferDrop_prototype("ITimedBufferDrop");


  const char* ITimedBufferDrop::addInOutPoint_event = "ITimedBufferDrop::addInOutPoint";
  const char* ITimedBufferDrop::useRelativeInOutPoints_event = "ITimedBufferDrop::useRelativeInOutPoints";
  const char* ITimedBufferDrop::isRelativeInOutPoints_event = "ITimedBufferDrop::isRelativeInOutPoints";


}


