
#ifndef NMM_IBufferDrop_HPP
#define NMM_IBufferDrop_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "nmm/interfaces/base/INode.hpp"

namespace NMM
{

  class IBufferDrop : virtual public Interface { 
  public:
    IBufferDrop() {  setTypeInfo(TypeInfo(IBufferDrop::IBufferDrop_type, "NMM")); };

    IBufferDrop(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IBufferDrop::IBufferDrop_type, "NMM")); setEventReceiver(parent); };

    virtual ~IBufferDrop() {};

    static const char* IBufferDrop_type;

    virtual  string getInterfaceType() const { return IBufferDrop_type; }

    Interface* copy() const { return new IBufferDrop(*this); }
    static const char* setBufferToDrop_event;
    static const char* setDropRatio_event;
    /**
     * Sets which buffers should be dropped (e.g. a '2' meaning every second buffer
     * should be dropped; a '3' meaning every third buffer)
     */

    virtual void setBufferToDrop(const unsigned int& buffer_num)
      throw(const Exception&);
    /**
     * Use this method to specify a drop ratio, e.g. dropping of 5 out
     * of 25 frames.
     * \param numerator The numerator of the ratio.
     * \param denominator The denominator of the ratio.
     */

    virtual void setDropRatio(const unsigned int& numerator, const unsigned int& denominator)
      throw(const Exception&);


  };

  typedef auto_ptr<IBufferDrop> IBufferDrop_var;



  class ITimedBufferDrop : virtual public Interface { 
  public:
    ITimedBufferDrop() {  setTypeInfo(TypeInfo(ITimedBufferDrop::ITimedBufferDrop_type, "NMM")); };

    ITimedBufferDrop(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(ITimedBufferDrop::ITimedBufferDrop_type, "NMM")); setEventReceiver(parent); };

    virtual ~ITimedBufferDrop() {};

    static const char* ITimedBufferDrop_type;

    virtual  string getInterfaceType() const { return ITimedBufferDrop_type; }

    Interface* copy() const { return new ITimedBufferDrop(*this); }
    static const char* addInOutPoint_event;
    static const char* useRelativeInOutPoints_event;
    static const char* isRelativeInOutPoints_event;
    /**
     * Adds an in and out point for removing timestamped buffers froms stream
     */

    virtual void addInOutPoint(const unsigned int& in_secs, const unsigned int& in_millisecs, const unsigned int& out_secs, const unsigned int& out_millisecs) /* limited(INode::CONSTRUCTED) */
      throw(const Exception&, const IllegalStateException&);
    /**
     * If true, all inout-points have to be specified relative to first timestamp in stream
     */

    virtual void useRelativeInOutPoints(const bool& relative) /* limited(INode::CONSTRUCTED) */
      throw(const Exception&, const IllegalStateException&);
    /**
     * Gets the status of relative or absolute inout-points
     */

    virtual bool isRelativeInOutPoints()
      throw(const Exception&);


  };

  typedef auto_ptr<ITimedBufferDrop> ITimedBufferDrop_var;




}

#endif
