
#ifndef NMM_ITimeProgressListener_HPP
#define NMM_ITimeProgressListener_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{
  /**
   *  @brief Progress events
   *
   *  Instream events for sending progress information.
   *  Nodes that implement the ITimeProgress interface send events of this interface.
   *  Applications that need progress information typically register listeners for these events.
   */

  class ITimeProgressListener : virtual public Interface { 
  public:
    ITimeProgressListener() {  setTypeInfo(TypeInfo(ITimeProgressListener::ITimeProgressListener_type, "NMM")); };

    ITimeProgressListener(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(ITimeProgressListener::ITimeProgressListener_type, "NMM")); setEventReceiver(parent); };

    virtual ~ITimeProgressListener() {};

    static const char* ITimeProgressListener_type;

    virtual  string getInterfaceType() const { return ITimeProgressListener_type; }

    Interface* copy() const { return new ITimeProgressListener(*this); }
    static const char* setTimeProgress_event;
    static Event* create_setTimeProgress(const Time& current_media_time);
  /* instream (protected) and non-instream (public) */        /**
     *  @brief Progress information.
     *  @param Contains the current playback time.
     *
     *  Instream event containing the current time progress.
     */

    virtual void setTimeProgress(const Time& current_media_time) /* in-stream */ 
      throw(const Exception&);


  };

  typedef auto_ptr<ITimeProgressListener> ITimeProgressListener_var;




}

#endif
