
#include "INet.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void INet::setTransportStrategy(const TSAddress* address) 
      throw(const Exception&, const BindingException&)
  {
    TInValue<TSAddress* > address_value (const_cast<TSAddress* >(address), false);
    Event event("INet::setTransportStrategy", &address_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(BindingException* ex = dynamic_cast<BindingException*>(event.getException())) {
          throw BindingException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void INet::setRTPStrategy(const string& address, const int& rtp_port, const int& rtcp_port) /* limited(INode::CONSTRUCTED) */
      throw(const Exception&, const IllegalStateException&)
  {
    TInValue<string > address_value(address, false);
    TInValue<int > rtp_port_value(rtp_port, false);
    TInValue<int > rtcp_port_value(rtcp_port, false);
    Event event("INet::setRTPStrategy", &address_value, &rtp_port_value, &rtcp_port_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void INet::setPayloadType(const int& type)
      throw(const Exception&)
  {
    TInValue<int > type_value(type, false);
    Event event("INet::setPayloadType", &type_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  int INet::getPayloadType() const
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    Event event("INet::getPayloadType");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* INet::INet_type = "INet";
  TIFactoryObject<INet > __INet_prototype("INet");


  const char* INet::setTransportStrategy_event = "INet::setTransportStrategy";
  const char* INet::setRTPStrategy_event = "INet::setRTPStrategy";
  const char* INet::setPayloadType_event = "INet::setPayloadType";
  const char* INet::getPayloadType_event = "INet::getPayloadType";


}


