
#ifndef NMM_IByteCounter_HPP
#define NMM_IByteCounter_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{
  /**
   * Interface for counting the bytes send/received by a transport strategy.
   */

  class IByteCounter : virtual public Interface { 
  public:
    IByteCounter() {  setTypeInfo(TypeInfo(IByteCounter::IByteCounter_type, "NMM")); };

    IByteCounter(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IByteCounter::IByteCounter_type, "NMM")); setEventReceiver(parent); };

    virtual ~IByteCounter() {};

    static const char* IByteCounter_type;

    virtual  string getInterfaceType() const { return IByteCounter_type; }

    Interface* copy() const { return new IByteCounter(*this); }
    static const char* getSendingCounter_event;
    static const char* getReceivingCounter_event;
    static const char* resetSendingCounter_event;
    static const char* resetReceivingCounter_event;
    /**
     * Get number of bytes send.
     * Returns the current value of the sending byte counter.
     */

    virtual int getSendingCounter()
      throw(const Exception&);
    /**
     * Get number of bytes received.
     * Returns the current value of the receiving byte counter.
     */

    virtual int getReceivingCounter()
      throw(const Exception&);
    /**
     * Reset the sending byte counter.
     * Resets the sending counter to zero bytes processed.
     */

    virtual void resetSendingCounter()
      throw(const Exception&);
    /**
     * Reset the receiving byte counter.
     * Resets the sending counter to zero bytes processed.
     */

    virtual void resetReceivingCounter()
      throw(const Exception&);


  };

  typedef auto_ptr<IByteCounter> IByteCounter_var;




}

#endif
