
#ifndef NMM_IEncodeStats_HPP
#define NMM_IEncodeStats_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{
  /**
   * Encoding statistics.
   * Interface for encoders that provide statistics by sending events
   * to an IEncodeStatsListener.
   */

  class IEncodeStats : virtual public Interface { 
  public:
    IEncodeStats() {  setTypeInfo(TypeInfo(IEncodeStats::IEncodeStats_type, "NMM")); };

    IEncodeStats(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IEncodeStats::IEncodeStats_type, "NMM")); setEventReceiver(parent); };

    virtual ~IEncodeStats() {};

    static const char* IEncodeStats_type;

    virtual  string getInterfaceType() const { return IEncodeStats_type; }

    Interface* copy() const { return new IEncodeStats(*this); }
    static const char* sendFrameCount_event;
    /**
     * Toggle sending frame count events.
     * Turns sending frame count events on or off. A frame count event
     * contains the number of frames encoded so far and the encoding
     * framerate.
     */

    virtual void sendFrameCount(const bool& value)
      throw(const Exception&);


  };

  typedef auto_ptr<IEncodeStats> IEncodeStats_var;


  /**
   * Encoding statistics events.
   * Contains events that are sent downstream by an encoder that implements
   * IEncodeStats.
   */

  class IEncodeStatsListener : virtual public Interface { 
  public:
    IEncodeStatsListener() {  setTypeInfo(TypeInfo(IEncodeStatsListener::IEncodeStatsListener_type, "NMM")); };

    IEncodeStatsListener(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IEncodeStatsListener::IEncodeStatsListener_type, "NMM")); setEventReceiver(parent); };

    virtual ~IEncodeStatsListener() {};

    static const char* IEncodeStatsListener_type;

    virtual  string getInterfaceType() const { return IEncodeStatsListener_type; }

    Interface* copy() const { return new IEncodeStatsListener(*this); }
    static const char* setFrameCount_event;
    static Event* create_setFrameCount(const unsigned int& count, const float& fps);
  /* instream (protected) and non-instream (public) */        /**
     * Frame count event.
     * Statistical event sent by the encoder.
     * @param count number of frames encoded so far
     * @param fps current encoding framerate in frames per second
     */

    virtual void setFrameCount(const unsigned int& count, const float& fps) /* in-stream */ 
      throw(const Exception&);


  };

  typedef auto_ptr<IEncodeStatsListener> IEncodeStatsListener_var;




}

#endif
