
#include "ISVGEncode.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void ISVGEncode::setColor(const int& red, const int& green, const int& blue)
      throw(const Exception&)
  {
    TInValue<int > red_value(red, false);
    TInValue<int > green_value(green, false);
    TInValue<int > blue_value(blue, false);
    Event event("ISVGEncode::setColor", &red_value, &green_value, &blue_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISVGEncode::setDimension(const int& width, const int& height)
      throw(const Exception&)
  {
    TInValue<int > width_value(width, false);
    TInValue<int > height_value(height, false);
    Event event("ISVGEncode::setDimension", &width_value, &height_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  int ISVGEncode::getPixelTaken()
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    Event event("ISVGEncode::getPixelTaken");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ISVGEncode::setPixelTaken(const int& pixel_taken)
      throw(const Exception&)
  {
    TInValue<int > pixel_taken_value(pixel_taken, false);
    Event event("ISVGEncode::setPixelTaken", &pixel_taken_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISVGEncode::setWithFile(const bool& withFile)
      throw(const Exception&)
  {
    TInValue<bool > withFile_value(withFile, false);
    Event event("ISVGEncode::setWithFile", &withFile_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISVGEncode::setStrokeWidth(const int& stroke_width)
      throw(const Exception&)
  {
    TInValue<int > stroke_width_value(stroke_width, false);
    Event event("ISVGEncode::setStrokeWidth", &stroke_width_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISVGEncode::drawSVGLine(const int& x, const int& y, const int& buttonmask)
      throw(const Exception&)
  {
    TInValue<int > x_value(x, false);
    TInValue<int > y_value(y, false);
    TInValue<int > buttonmask_value(buttonmask, false);
    Event event("ISVGEncode::drawSVGLine", &x_value, &y_value, &buttonmask_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISVGEncode::drawSVGRectangle(const int& x, const int& y, const int& buttonmask)
      throw(const Exception&)
  {
    TInValue<int > x_value(x, false);
    TInValue<int > y_value(y, false);
    TInValue<int > buttonmask_value(buttonmask, false);
    Event event("ISVGEncode::drawSVGRectangle", &x_value, &y_value, &buttonmask_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISVGEncode::drawSVGFreehand(const int& x, const int& y, const int& buttonmask)
      throw(const Exception&)
  {
    TInValue<int > x_value(x, false);
    TInValue<int > y_value(y, false);
    TInValue<int > buttonmask_value(buttonmask, false);
    Event event("ISVGEncode::drawSVGFreehand", &x_value, &y_value, &buttonmask_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISVGEncode::drawSVGPointer(const int& x, const int& y, const int& buttonmask)
      throw(const Exception&)
  {
    TInValue<int > x_value(x, false);
    TInValue<int > y_value(y, false);
    TInValue<int > buttonmask_value(buttonmask, false);
    Event event("ISVGEncode::drawSVGPointer", &x_value, &y_value, &buttonmask_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISVGEncode::newAnnotation()
      throw(const Exception&)
  {
    Event event("ISVGEncode::newAnnotation");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISVGEncode::clearAnnotation()
      throw(const Exception&)
  {
    Event event("ISVGEncode::clearAnnotation");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISVGEncode::undo()
      throw(const Exception&)
  {
    Event event("ISVGEncode::undo");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISVGEncode::redo()
      throw(const Exception&)
  {
    Event event("ISVGEncode::redo");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void ISVGEncode::repaint()
      throw(const Exception&)
  {
    Event event("ISVGEncode::repaint");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }

  const char* ISVGEncode::ISVGEncode_type = "ISVGEncode";
  TIFactoryObject<ISVGEncode > __ISVGEncode_prototype("ISVGEncode");
  TValueConstructor< ISVGEncode::SVGDrawMode > __ISVGEncode_SVGDrawMode_prototype(TypeInfo("ISVGEncode::SVGDrawMode", "NMM"));

  OValueStream& operator<<(OValueStream& ov, const ISVGEncode::SVGDrawMode& t)
  {
    ov.beginType(TypeInfo("ISVGEncode::SVGDrawMode", "NMM"));
    ov << reinterpret_cast<const int&>(t);
    ov.endType();
    return ov;
  }

  IValueStream& operator>>(IValueStream& iv, ISVGEncode::SVGDrawMode& t)
  {
    if(iv.getCurrentTypeInfo() != TypeInfo("ISVGEncode::SVGDrawMode", "NMM")) {
      throw InvalidTypeException(TypeInfo("ISVGEncode::SVGDrawMode", "NMM"), iv.getCurrentTypeInfo());
    }
    iv.goDown();
    iv >> reinterpret_cast<int&>(t);
    iv.goUp();
    return iv;
  }



  const char* ISVGEncode::setColor_event = "ISVGEncode::setColor";
  const char* ISVGEncode::setDimension_event = "ISVGEncode::setDimension";
  const char* ISVGEncode::getPixelTaken_event = "ISVGEncode::getPixelTaken";
  const char* ISVGEncode::setPixelTaken_event = "ISVGEncode::setPixelTaken";
  const char* ISVGEncode::setWithFile_event = "ISVGEncode::setWithFile";
  const char* ISVGEncode::setStrokeWidth_event = "ISVGEncode::setStrokeWidth";
  const char* ISVGEncode::drawSVGLine_event = "ISVGEncode::drawSVGLine";
  const char* ISVGEncode::drawSVGRectangle_event = "ISVGEncode::drawSVGRectangle";
  const char* ISVGEncode::drawSVGFreehand_event = "ISVGEncode::drawSVGFreehand";
  const char* ISVGEncode::drawSVGPointer_event = "ISVGEncode::drawSVGPointer";
  const char* ISVGEncode::newAnnotation_event = "ISVGEncode::newAnnotation";
  const char* ISVGEncode::clearAnnotation_event = "ISVGEncode::clearAnnotation";
  const char* ISVGEncode::undo_event = "ISVGEncode::undo";
  const char* ISVGEncode::redo_event = "ISVGEncode::redo";
  const char* ISVGEncode::repaint_event = "ISVGEncode::repaint";


}


