
#include "IFramerateConverter.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void IFramerateConverter::setTimeOffset(const Interval& start)
      throw(const Exception&)
  {
    TInValue<Interval > start_value(start, false);
    Event event("IFramerateConverter::setTimeOffset", &start_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IFramerateConverter::setTimeOffsetMillisecs(const int& millisecs)
      throw(const Exception&)
  {
    TInValue<int > millisecs_value(millisecs, false);
    Event event("IFramerateConverter::setTimeOffsetMillisecs", &millisecs_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IFramerateConverter::setFrameOffset(const float& alpha)
      throw(const Exception&)
  {
    TInValue<float > alpha_value(alpha, false);
    Event event("IFramerateConverter::setFrameOffset", &alpha_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  int IFramerateConverter::getTotalInBuffers() const
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    Event event("IFramerateConverter::getTotalInBuffers");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  int IFramerateConverter::getTotalOutBuffers() const
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    Event event("IFramerateConverter::getTotalOutBuffers");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  int IFramerateConverter::getTotalDroppedBuffers() const
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    Event event("IFramerateConverter::getTotalDroppedBuffers");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  int IFramerateConverter::getTotalCopiedBuffers() const
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    Event event("IFramerateConverter::getTotalCopiedBuffers");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IFramerateConverter::IFramerateConverter_type = "IFramerateConverter";
  TIFactoryObject<IFramerateConverter > __IFramerateConverter_prototype("IFramerateConverter");


  const char* IFramerateConverter::setTimeOffset_event = "IFramerateConverter::setTimeOffset";
  const char* IFramerateConverter::setTimeOffsetMillisecs_event = "IFramerateConverter::setTimeOffsetMillisecs";
  const char* IFramerateConverter::setFrameOffset_event = "IFramerateConverter::setFrameOffset";
  const char* IFramerateConverter::getTotalInBuffers_event = "IFramerateConverter::getTotalInBuffers";
  const char* IFramerateConverter::getTotalOutBuffers_event = "IFramerateConverter::getTotalOutBuffers";
  const char* IFramerateConverter::getTotalDroppedBuffers_event = "IFramerateConverter::getTotalDroppedBuffers";
  const char* IFramerateConverter::getTotalCopiedBuffers_event = "IFramerateConverter::getTotalCopiedBuffers";


}


