
#ifndef NMM_IFramerateConverter_HPP
#define NMM_IFramerateConverter_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{
  /**
   * Framerate converter interface.
   */

  class IFramerateConverter : virtual public Interface { 
  public:
    IFramerateConverter() {  setTypeInfo(TypeInfo(IFramerateConverter::IFramerateConverter_type, "NMM")); };

    IFramerateConverter(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IFramerateConverter::IFramerateConverter_type, "NMM")); setEventReceiver(parent); };

    virtual ~IFramerateConverter() {};

    static const char* IFramerateConverter_type;

    virtual  string getInterfaceType() const { return IFramerateConverter_type; }

    Interface* copy() const { return new IFramerateConverter(*this); }
    static const char* setTimeOffset_event;
    static const char* setTimeOffsetMillisecs_event;
    static const char* setFrameOffset_event;
    static const char* getTotalInBuffers_event;
    static const char* getTotalOutBuffers_event;
    static const char* getTotalDroppedBuffers_event;
    static const char* getTotalCopiedBuffers_event;
    /**
     * Set time offset.
     * Set the time offset for the output stream. The output stream normally begins at the
     * time of the first incoming buffer. This method can be used to shift the timestamp of
     * the first output buffer relative to the time of the first incoming buffer.
     *
     * If this method is called in STARTED, it will take effect after the next syncReset event.
     */

    virtual void setTimeOffset(const Interval& start)
      throw(const Exception&);
    /**
     * Set time offset.
     * Set the time offset for the output stream. The output stream normally begins at the
     * time of the first incoming buffer. This method can be used to shift the timestamp of
     * the first output buffer relative to the time of the first incoming buffer.
     *
     * If this method is called in STARTED, it will take effect after the next syncReset event.
     */

    virtual void setTimeOffsetMillisecs(const int& millisecs)
      throw(const Exception&);
    /**
     * Set frame offset for interpolation.
     * This method controls where in the output video stream an input video frame exactly starts.
     * The value should be between 0 and 1. A value of zero means that an input frame starts at
     * exactly its own input timestamp. A value of 0.5 (which is the default) means that the
     * frame starts at the average between its timestamp and its predecessor's timestamp.
     */

    virtual void setFrameOffset(const float& alpha)
      throw(const Exception&);
    /**
     * Get statistics since last sync reset.
     * Get total number of input buffers.
     */

    virtual int getTotalInBuffers() const
      throw(const Exception&);
    /**
     * Get statistics since last sync reset.
     * Get total number of output buffers.
     */

    virtual int getTotalOutBuffers() const
      throw(const Exception&);
    /**
     * Get statistics since last sync reset.
     * Get total number of buffers dropped.
     */

    virtual int getTotalDroppedBuffers() const
      throw(const Exception&);
    /**
     * Get statistics since last sync reset.
     * Get total number of copied created.
     */

    virtual int getTotalCopiedBuffers() const
      throw(const Exception&);


  };

  typedef auto_ptr<IFramerateConverter> IFramerateConverter_var;




}

#endif
