
#include "IScaler.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void IScaler::setAutoAspect(const bool& value)
      throw(const Exception&)
  {
    TInValue<bool > value_value(value, false);
    Event event("IScaler::setAutoAspect", &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IScaler::setFiltering(const bool& value)
      throw(const Exception&)
  {
    TInValue<bool > value_value(value, false);
    Event event("IScaler::setFiltering", &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IScaler::setOutputSize(const int& w, const int& h)
      throw(const Exception&)
  {
    TInValue<int > w_value(w, false);
    TInValue<int > h_value(h, false);
    Event event("IScaler::setOutputSize", &w_value, &h_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IScaler::getOutputSize(int& w, int& h) const
      throw(const Exception&)
  {
    TOutValue<int > w_value(w);
    TOutValue<int > h_value(h);
    Event event("IScaler::getOutputSize", &w_value, &h_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    w = w_value.getRValue();
    h = h_value.getRValue();
  }
  
  bool IScaler::getAutoAspect() const
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("IScaler::getAutoAspect");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  bool IScaler::getFiltering() const
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("IScaler::getFiltering");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IScaler::IScaler_type = "IScaler";
  TIFactoryObject<IScaler > __IScaler_prototype("IScaler");


  const char* IScaler::setAutoAspect_event = "IScaler::setAutoAspect";
  const char* IScaler::setFiltering_event = "IScaler::setFiltering";
  const char* IScaler::setOutputSize_event = "IScaler::setOutputSize";
  const char* IScaler::getOutputSize_event = "IScaler::getOutputSize";
  const char* IScaler::getAutoAspect_event = "IScaler::getAutoAspect";
  const char* IScaler::getFiltering_event = "IScaler::getFiltering";


}


