
#ifndef NMM_IButtons_HPP
#define NMM_IButtons_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "IImageView.hpp"

namespace NMM
{
  /**
   * Buttons widget interface.
   */

  class IButtons : virtual public IImageView { 
  public:
    IButtons() {  setTypeInfo(TypeInfo(IButtons::IButtons_type, "NMM")); };

    IButtons(NMMObject& parent) : IImageView(parent) {  setTypeInfo(TypeInfo(IButtons::IButtons_type, "NMM")); setEventReceiver(parent); };

    virtual ~IButtons() {};

    static const char* IButtons_type;

    virtual  string getInterfaceType() const { return IButtons_type; }

    Interface* copy() const { return new IButtons(*this); }
    static const char* addButton_event;
    static const char* getButtonLabel_event;
    static const char* getSelectedButtonLabel_event;
    static const char* getButtonState_event;
    static const char* deleteButton_event;
    static const char* deleteAllButtons_event;
    static const char* selectButton_event;
    static const char* setStep_event;
    static const char* deselectButton_event;
    static const char* activateButton_event;
    static const char* deactivateButton_event;
    static const char* deactivateAllButtons_event;
    /**
     * Add button.
     * @param id identifier for this button
     * @param label button label
     * @param x x position of the upper left corner of the button
     * @param y y position of the upper left corner
     * @param inactive_id bitmap identifier for using if inactive button should be drawn
     * @param active_id bitmap identifier for using if active button should be drawn
     * @param select_id bitmap identifier for using if selected button should be drawn
     */

    virtual void addButton(const int& buttonid, const string& label, const int& x, const int& y, const string& inactive_id, const string& active_id, const string& select_id)
      throw(const Exception&);
    /**
     * Get label of a button.
     * @param button_id button identifier
     * @return label of the button with button_id
     */

    virtual string getButtonLabel(const int& button_id)
      throw(const Exception&);
    /**
     * Get label of selected button.
     * @return label of the selected button
     */

    virtual string getSelectedButtonLabel()
      throw(const Exception&);
    /**
     * Get burron state.
     * @param button_id button identifier
     * @return true, if button is active, false otherwise
     */

    virtual bool getButtonState(const int& button_id)
      throw(const Exception&);
    /**
     * Delete button.
     * @param button_id button identifier
     */

    virtual void deleteButton(const int& button_id)
      throw(const Exception&);
    /** Delete all buttons. */

    virtual void deleteAllButtons()
      throw(const Exception&);
    /**
     * Selects the given button.
     * @param buttonid id of button that should be selected
     * @return id of selected button
     */

    virtual int selectButton(const int& buttonid)
      throw(const Exception&);
    /**
     * Set number of buttons per line.
     * Set number of buttons to get into next line (important for selectUp() and selectDown() calls).
     * @param value number of buttons to skip
     */

    virtual void setStep(const unsigned int& value)
      throw(const Exception&);
    /** Deselect selected button. */

    virtual void deselectButton()
      throw(const Exception&);
    /**
     * Activate a button.
     * @param button_id button identifier
     */

    virtual void activateButton(const int& button_id)
      throw(const Exception&);
    /**
     * Deactivate a button.
     * @param button_id button identifier
     */

    virtual void deactivateButton(const int& button_id)
      throw(const Exception&);
    /**
     * Deactivate all buttons.
     */

    virtual void deactivateAllButtons()
      throw(const Exception&);


  };

  typedef auto_ptr<IButtons> IButtons_var;




}

#endif
