
#ifndef NMM_IButtonsImpl_HPP
#define NMM_IButtonsImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IButtons.hpp"

#include "IImageViewImpl.hpp"

namespace NMM
{
  /**
   * Buttons widget interface.
   */

  class IButtonsImpl  : virtual public IImageViewImpl { 
  public:
    IButtonsImpl() { addInterface(new IButtons(*this)); registerEvents(); };

    virtual ~IButtonsImpl() {};

  protected:
    /**
     * Add button.
     * @param id identifier for this button
     * @param label button label
     * @param x x position of the upper left corner of the button
     * @param y y position of the upper left corner
     * @param inactive_id bitmap identifier for using if inactive button should be drawn
     * @param active_id bitmap identifier for using if active button should be drawn
     * @param select_id bitmap identifier for using if selected button should be drawn
     */

    virtual void addButton(const int& buttonid, const string& label, const int& x, const int& y, const string& inactive_id, const string& active_id, const string& select_id) = 0;
    /**
     * Get label of a button.
     * @param button_id button identifier
     * @return label of the button with button_id
     */

    virtual string getButtonLabel(const int& button_id) = 0;
    /**
     * Get label of selected button.
     * @return label of the selected button
     */

    virtual string getSelectedButtonLabel() = 0;
    /**
     * Get burron state.
     * @param button_id button identifier
     * @return true, if button is active, false otherwise
     */

    virtual bool getButtonState(const int& button_id) = 0;
    /**
     * Delete button.
     * @param button_id button identifier
     */

    virtual void deleteButton(const int& button_id) = 0;
    /** Delete all buttons. */

    virtual void deleteAllButtons() = 0;
    /**
     * Selects the given button.
     * @param buttonid id of button that should be selected
     * @return id of selected button
     */

    virtual int selectButton(const int& buttonid) = 0;
    /**
     * Set number of buttons per line.
     * Set number of buttons to get into next line (important for selectUp() and selectDown() calls).
     * @param value number of buttons to skip
     */

    virtual void setStep(const unsigned int& value) = 0;
    /** Deselect selected button. */

    virtual void deselectButton() = 0;
    /**
     * Activate a button.
     * @param button_id button identifier
     */

    virtual void activateButton(const int& button_id) = 0;
    /**
     * Deactivate a button.
     * @param button_id button identifier
     */

    virtual void deactivateButton(const int& button_id) = 0;
    /**
     * Deactivate all buttons.
     */

    virtual void deactivateAllButtons() = 0;


  private:
    void registerEvents();
  };

}

#endif
