
#include "IWidget.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void IWidget::selectRight()
      throw(const Exception&)
  {
    Event event("IWidget::selectRight");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IWidget::selectLeft()
      throw(const Exception&)
  {
    Event event("IWidget::selectLeft");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IWidget::selectUp()
      throw(const Exception&)
  {
    Event event("IWidget::selectUp");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IWidget::selectDown()
      throw(const Exception&)
  {
    Event event("IWidget::selectDown");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IWidget::selectPush()
      throw(const Exception&)
  {
    Event event("IWidget::selectPush");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  int IWidget::getWidgetID() const
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    Event event("IWidget::getWidgetID");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IWidget::setPosition(const unsigned int& x, const unsigned int& y)
      throw(const Exception&)
  {
    TInValue<unsigned int > x_value(x, false);
    TInValue<unsigned int > y_value(y, false);
    Event event("IWidget::setPosition", &x_value, &y_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IWidget::setDimension(const unsigned int& w, const unsigned int& h)
      throw(const Exception&)
  {
    TInValue<unsigned int > w_value(w, false);
    TInValue<unsigned int > h_value(h, false);
    Event event("IWidget::setDimension", &w_value, &h_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IWidget::getPosition(unsigned int& x, unsigned int& y) const
      throw(const Exception&)
  {
    TOutValue<unsigned int > x_value(x);
    TOutValue<unsigned int > y_value(y);
    Event event("IWidget::getPosition", &x_value, &y_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    x = x_value.getRValue();
    y = y_value.getRValue();
  }
  
  void IWidget::getDimension(unsigned int& w, unsigned int& h) const
      throw(const Exception&)
  {
    TOutValue<unsigned int > w_value(w);
    TOutValue<unsigned int > h_value(h);
    Event event("IWidget::getDimension", &w_value, &h_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    w = w_value.getRValue();
    h = h_value.getRValue();
  }

  const char* IWidget::IWidget_type = "IWidget";
  TIFactoryObject<IWidget > __IWidget_prototype("IWidget");


  const char* IWidget::selectRight_event = "IWidget::selectRight";
  const char* IWidget::selectLeft_event = "IWidget::selectLeft";
  const char* IWidget::selectUp_event = "IWidget::selectUp";
  const char* IWidget::selectDown_event = "IWidget::selectDown";
  const char* IWidget::selectPush_event = "IWidget::selectPush";
  const char* IWidget::getWidgetID_event = "IWidget::getWidgetID";
  const char* IWidget::setPosition_event = "IWidget::setPosition";
  const char* IWidget::setDimension_event = "IWidget::setDimension";
  const char* IWidget::getPosition_event = "IWidget::getPosition";
  const char* IWidget::getDimension_event = "IWidget::getDimension";


}


