
#ifndef NMM_IWidget_HPP
#define NMM_IWidget_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "IScrollable.hpp"

namespace NMM
{
  /**
   * Widget interface.
   * Interface for controlling widgets in an OSDManager.
   * The widgets are identified by integers. To get an interface, use the IOSDManager interface.
   */

  class IWidget : virtual public IScrollable { 
  public:
    IWidget() {  setTypeInfo(TypeInfo(IWidget::IWidget_type, "NMM")); };

    IWidget(NMMObject& parent) : IScrollable(parent) {  setTypeInfo(TypeInfo(IWidget::IWidget_type, "NMM")); setEventReceiver(parent); };

    virtual ~IWidget() {};

    static const char* IWidget_type;

    virtual  string getInterfaceType() const { return IWidget_type; }

    Interface* copy() const { return new IWidget(*this); }
    static const char* selectRight_event;
    static const char* selectLeft_event;
    static const char* selectUp_event;
    static const char* selectDown_event;
    static const char* selectPush_event;
    static const char* getWidgetID_event;
    static const char* setPosition_event;
    static const char* setDimension_event;
    static const char* getPosition_event;
    static const char* getDimension_event;
    /** Selects next button (adds 1 to buttonid). */

    virtual void selectRight()
      throw(const Exception&);
    /** Selects privious button (subs 1 to buttonid). */

    virtual void selectLeft()
      throw(const Exception&);
    /** Selects button (in privoius line, subs step to buttonid). */

    virtual void selectUp()
      throw(const Exception&);
    /** Selects button (in next line, adds step to buttonid). */

    virtual void selectDown()
      throw(const Exception&);
    /** Activates the selected button. */

    virtual void selectPush()
      throw(const Exception&);
    /** Get widget ID. */

    virtual int getWidgetID() const
      throw(const Exception&);
    /**
     * Set x and y position of the widget in the window.
     * @param x x position
     * @param y x position
     */

    virtual void setPosition(const unsigned int& x, const unsigned int& y)
      throw(const Exception&);
    /**
     * Set width and height of the widget.
     * @param w width
     * @param h height
     */

    virtual void setDimension(const unsigned int& w, const unsigned int& h)
      throw(const Exception&);
    /**
     * Get x and y position of the widget.
     * @param x x position
     * @param y x position
     */

    virtual void getPosition(unsigned int& x, unsigned int& y) const
      throw(const Exception&);
    /**
     * Get width and height of the widget.
     * @param w width
     * @param h height
     */

    virtual void getDimension(unsigned int& w, unsigned int& h) const
      throw(const Exception&);


  };

  typedef auto_ptr<IWidget> IWidget_var;




}

#endif
