
#ifndef NMM_IWindow_HPP
#define NMM_IWindow_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "IWidget.hpp"

namespace NMM
{

  class IWidget;
  /**
   * Window interface.
   * A window is a container for a widget in an OSDManager.
   * This interface can be retrieved through an IWidget or IOSDManager interface.
   */

  class IWindow : virtual public Interface { 
  public:
    IWindow() {  setTypeInfo(TypeInfo(IWindow::IWindow_type, "NMM")); };

    IWindow(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IWindow::IWindow_type, "NMM")); setEventReceiver(parent); };

    virtual ~IWindow() {};

    static const char* IWindow_type;

    virtual  string getInterfaceType() const { return IWindow_type; }

    Interface* copy() const { return new IWindow(*this); }
    static const char* getWindowID_event;
    static const char* setPosition_event;
    static const char* getPosition_event;
    static const char* setDimension_event;
    static const char* getDimension_event;
    static const char* update_event;
    static const char* getVisible_event;
    static const char* setVisible_event;
    static const char* setScaleFactor_event;
    static const char* setContent_event;
    static const char* getContent_event;
    /** Get window ID. */

    virtual int getWindowID() const
      throw(const Exception&);
    /**
     * Set position of the window.
     * @param x x position of the upper left corner
     * @param y y position of the upper left corner
     */

    virtual void setPosition(const unsigned int& x, const unsigned int& y)
      throw(const Exception&);
    /**
     * Get x and y position of the window.
     * @param x x position of the upper left corner
     * @param y y position of the upper left corner
     */

    virtual void getPosition(unsigned int& x, unsigned int& y) const
      throw(const Exception&);
    /** Set width and height of the window. */

    virtual void setDimension(const unsigned int& w, const unsigned int& h)
      throw(const Exception&);
    /** Get width and height of the window. */

    virtual void getDimension(unsigned int& w, unsigned int& h) const
      throw(const Exception&);
    /**
     * Redraw the window with help of the widget.
     * Sets updateflag to true.
     */

    virtual void update()
      throw(const Exception&);
    /**
     * Check if window is visible
     * @return true if window is visible
     */

    virtual bool getVisible() const
      throw(const Exception&);
    /**
     * Set window visibility
     * @param show true to make window visible
     */

    virtual void setVisible(const bool& value)
      throw(const Exception&);
    /**
     * Set window scale factor.
     */

    virtual void setScaleFactor(const double& x, const double& y)
      throw(const Exception&);
    /**
     * Set content.
     * Set the widget that this window contains.
     */

    virtual void setContent(const IWidget& value)
      throw(const Exception&);
    /**
     * Get content.
     * Get the widget that this window contains.
     */

    virtual IWidget* getContent() const
      throw(const Exception&);


  };

  typedef auto_ptr<IWindow> IWindow_var;




}

#endif
