
#ifndef NMM_IChannelInfo_HPP
#define NMM_IChannelInfo_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "nmm/interfaces/video/tv/TVChannelList.hpp"
#include "nmm/interfaces/base/INode.hpp"

namespace NMM
{
  /**
   *Interface for requesting channel-information from the Node. It is used to build the global channel-list.
   */

  class IChannelInfo : virtual public Interface { 
  public:
    IChannelInfo() {  setTypeInfo(TypeInfo(IChannelInfo::IChannelInfo_type, "NMM")); };

    IChannelInfo(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IChannelInfo::IChannelInfo_type, "NMM")); setEventReceiver(parent); };

    virtual ~IChannelInfo() {};

    static const char* IChannelInfo_type;

    virtual  string getInterfaceType() const { return IChannelInfo_type; }

    Interface* copy() const { return new IChannelInfo(*this); }
    static const char* hasChannel_event;
    static const char* getChannelList_event;
    static const char* setChannelConfigFile_event;
    /**
     *Tests, if the TVCard has the channel with the given name. It writes the corrensponding channel-number to c1
     */

    virtual Result hasChannel(int& c1, const string& c2)
      throw(const Exception&);
    /**
     *Returns a complete List of all available channels
     *@return TVCannelList the channel-list
     */

    virtual TVChannelList getChannelList()
      throw(const Exception&);
    /**
     *Tell the node to read the channel-configuration from the given file. 
     *@param filename complete path to the channel-configuration-file
     */

    virtual void setChannelConfigFile(const string& filename) /* limited(INode::CONSTRUCTED) */
      throw(const Exception&, const IllegalStateException&);


  };

  typedef auto_ptr<IChannelInfo> IChannelInfo_var;




}

#endif
