
#ifndef NMM_IEPGImpl_HPP
#define NMM_IEPGImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IEPG.hpp"

#include "nmm/interfaces/base/INodeImpl.hpp"

namespace NMM
{
  /**
   * Interface for an Electronic Programming Guide
   * to provide additional information to the program shown
   */

  class IEPGImpl  : public virtual NMMObject { 
  public:
    IEPGImpl() { addInterface(new IEPG(*this)); registerEvents(); };

    virtual ~IEPGImpl() {};

  protected:
    /**
     * returns a string wich contains the starttime, 
     * endtime and title of the current show
     * Format: hh:mm - hh:mm : title
     */

    virtual string getCurrentShow() /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /**
     * returns a string wich contains the starttime, 
     * endtime and title of the following show
     * Format: hh:mm - hh:mm : title
     */

    virtual string getFollowingShow() /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /**
     * returns a string containing a detailed description
     * of the curretn show
     */

    virtual string getDetailedDescription() /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /**
     * sets the Directory, where the XML-file with EPG-information is stored
     */

    virtual void setEPGDirectory(const string& dir) = 0;
    /**
     * restores the channel before starting the scanning-function
     */

    virtual void getActivity() /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /**
     * starts the scanning-function of VDR to collect EPG-data
     */

    virtual void scan() /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;


  private:
    void registerEvents();
  };

}

#endif
