/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, 
 * USA.
 */

#ifndef _NMM_IPAQPRODUCER_HPP_
#define _NMM_IPAQPRODUCER_HPP_ _NMM_IPAQPRODUCER_HPP_

#include "nmm/base/GenericProducer.hpp"

#include <linux/vt.h>
#include <linux/kd.h>
#include <termios.h>


namespace NMM {


  class IEventReceiver;


  class iPAQProducer : 
    public InputProducer
  {
  private:

    /* Mutex */
    ThreadMutex keymap_mutex;
    
    /* keycode -> event  */
    vector<string> keymap;

    /* save all keympas found in the xml config file */
    map< const string, vector<string> > keymaps;

    /* name of the current used layout */
    string currentLayout;

    /* Terminal */
    int tty_handle;
    int tty_no;
    int tty_orig;
    vt_mode tty_vtm;
    
    /* Keyboard */
    int kb_oldkbdmode;
    termios kb_oldttysets;
    
    /* open/close virtual terminal */
    bool open_vt();
    bool close_vt();

    /* open/close keyboard */
    bool open_kb();
    bool close_kb();
    
    /**
     * add a key to the keymap
     */
    void addKey(unsigned char key, string value);
    
    /**
     * return the event defined by keycode
     */
    string getKeyMapping(unsigned char key);

  public:
    
   enum {RECORD    = 120, 
         POWER     = 121, 
         CALENDAR  = 122,
         CONTACTS  = 123,
         MAIL      = 124,
         START     = 125,
         UP        = 103,
         LEFT      = 105,
         RIGHT     = 106,
         DOWN      = 108,
         ACTION    =  96,
         RELEASED  = 128};
  


    iPAQProducer();
    ~iPAQProducer();

    /**
     * read the config from a xml file
     */
    Result readConfig(string);

    /**
     * Starts the event-handling in a new thread
     */
    void start();
    
    /**
     * Stops the event-handling (exit thread)
     */
    void stop();

    /**
     * set layout to layout with name <string>
     * the name is set in the xml config file as
     * property of the keyboard tag
     */
    void setLayout(string);

    /**
     * return current layout
     */
    string getLayout();


  protected:
    
    void handleInput();

  }; /* class */

} /* Namespace NMM */

#endif /* _NMM_IPAQPRODUCER_HPP_ */
