/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christoph Wellner <chwellner@graphics.cs.uni-sb.de>
 *                     
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef NMM_XMLTVPLUGIN_HPP
#define NMM_XMLTVPLUGIN_HPP NMM_XMLTVPLUGIN_HPP

#include "nmm/interfaces/misc/tvguide/IXMLTVPluginImpl.hpp"

#include "nmm/misc/tvguide/TVGuide.hpp"
#include "nmm/misc/tvguide/TVGuidePlugin.hpp"

#include "nmm/utils/thread/ThreadedObject.hpp"

#include "nmm/misc/timer/TimerEvents.hpp"

#include "nmm/misc/tvguide/TVGXercescDatabase.hpp"


namespace NMM{

  class XMLTVPlugin : public TVGuidePlugin,
                      public IXMLTVPluginImpl
  {

  private:

    string filename;
    string configFile;
    string locale;
    string language;

    string lastUpdateDate;
    string lastUpdateChannel;

    list<string> xmltvChannels;

    bool termUpdate;
    string temporaryConfigFile;
    bool fillHolesInDB;

    string period;

    TimerProducer* timer;

    list<pair<string,string> > grabber;

    XercesTranscoder *transcoder;
    DOMImplementation *impl;
    DOMBuilder *parser;

  public: // variables

    TEDObject0<XMLTVPlugin> update_timer;

  public: //functions
    
    XMLTVPlugin();
    ~XMLTVPlugin();

  protected:
    /**
     * Set the path and file where XMLTV stores its downloaded data
     *@param file filename of datafile
     */
    void setFilename(const string &file);
    /**
     * get the filename of the temp datafile
     *@return path + datafile
     */
    string getFilename();
    
    /**
     * Set the patch and name of the temporary config-file, where the plugin stores its current position in update
     *@param filepath and filename of configfile
     */
    void setConfigFile(const string &file);
    /**
     * get the path and name of the confgifile
     *@return path and name of the confgifile
     */
    string getConfigFile();
    
    /**
     * Select the XMLTV-Tool by setting the country-information
     *@param loc abreviation or complete tool-name of XMLTV-Tools
     *@return SUCCESS if the grabber for the given setting is available
     *@return FAILURE otherwise
     */
    Result setLocale(const string &loc);
    /**
     * get the currently used Locale-information
     *@return locale
     */
    string getLocale();
    
    void setPeriod(const string &p);
    string getPeriod();
    
    void setFillHolesInDB(const bool &b);
    bool getFillHolesInDB();

    /**
     * starts the update-process
     *@return SUCCESS on success
     *@return FAILURE on failure
     */ 
    Result doStart();
    Result doStop();
    Result doInit();
    Result doDeinit();

    Result restartUpdate();

  private:
    int readXMLTV(const string& file);

    void readPluginConfig();
    void writePluginConfig();

    void readConfigFile();
    void writeConfigFile();

    bool readXMLTVConfig(string file);

  };


}

#endif
