/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christoph Wellner <chwellner@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef TVTIMERLIST_HPP
#define TVTIMERLIST_HPP TVTIMERLIST_HPP

#include "nmm/misc/tvtimer/TVTimerEntry.hpp"
#include "nmm/misc/tvtimer/TVTimerExceptions.hpp"
#include "nmm/utils/Singleton.hpp"

#include "nmm/interfaces/misc/tvtimer/ITVTimer.hpp"

namespace NMM {

  class TVTimerList {
    friend class CreateUsingNew<TVTimerList>;

  private:
    list<TVTimerEntry> timers;
    unsigned int currentID;
    string timerFilename;

  private:
    TVTimerList();
    TVTimerList(const TVTimerList&);
    ~TVTimerList();

  public: 
    Result addTimer(TVTimerEntry &entry);
    Result changeTimer(TVTimerEntry &entry);

    list<TVTimerEntry> getTimers();
    list<TVTimerEntry> getTimers(string day);
    list<TVTimerEntry> getTimers(string day, string channel);
    list<TVTimerEntry> getTimers(TVCardDescription d);

    list<TVTimerEntry> getSleepingTimers();
    list<TVTimerEntry> getRecordingTimers();
    list<TVTimerEntry> getRecordedTimers();
    list<TVTimerEntry> getConflictTimers();
    list<TVTimerEntry> getConflictsWithTimer(const TVTimerEntry &entry);

    TVTimerEntry getTimer(unsigned int id) throw (NoTimerFoundException);

    Result removeTimer(unsigned int id);

    Result loadTimer(string filename, ITVTimer *itimer);
    Result saveTimer(string filename);

  private:
    string TimeToString(time_t ltime);


  };
  typedef Singleton<TVTimerList, CreateUsingNew> STVTimerList;
}
#endif
