/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_MP3READNODE_HPP_
#define _NMM_MP3READNODE_HPP_ _NMM_MP3READNODE_HPP_

#include <string>

#include "nmm/base/GenericSourceNode.hpp"
#include "nmm/interfaces/file/IBufferSizeImpl.hpp"
#include "nmm/interfaces/file/IFileHandlerImpl.hpp"
#include "nmm/interfaces/file/ISeekableImpl.hpp"
#include "nmm/interfaces/general/progress/IProgressImpl.hpp"

namespace NMM {

  class MP3ReadNode :
    public GenericSourceNode,
    public IFileHandlerImpl,
    public ISeekableImpl,
    public IProgressImpl,
    public IBufferSizeImpl
  {
  private:
    enum EventState {
      EVENT_NONE,
      EVENT_SET_TITLE,
      EVENT_TAGINFO,
      EVENT_START_TRACK,
      EVENT_NEXT_TRACK,
      EVENT_SET_FORMAT,
      EVENT_END_TRACK,
      EVENT_TRACK_DURATION
    };
    
    int event_state;
    string filename;
    
    bool         send_progress;
    unsigned int buffers_before_progress;
    unsigned int buffers_progress;

    size_t buffer_size;

    // seeking & file position
    u_int64_t         filesize;
    u_int64_t         filepos;
    int64_t           seekto;

    Interval          track_duration;

    int fd;

    /* curent mp3 frame info */
    char header[4];
    char fmpeg;
    char flayer;
    char fprotbit;
    int  fbitrate;
    int  fsample_rate;
    char fpadding;
    int fnumchannels;

    bool found_tag;
    char title[31];
    char interpret[31];
    char album[31];
    char year[5];
    char comment[31];
    int genre;

    Timestamp timestamp;

    Format old_format;
    Format old_connection_format;
    Format new_simple_format;

    /* private functions */
    bool parseHeader(char*);
    
    Result openFile();
    
    /**
    * check file with filedescriptor fd, if it has a RIFF header
    * \returns 0, if no RIFF header was found
    * \returns position of data section (without data chunk)
    */
    int isWav();

  protected:
    
    virtual Result doInit();
    virtual Result doInitOutput();
    virtual Result doActivate();

    virtual Message* processBuffer(Buffer *);
    
    
    // IFileHandler interface
    virtual void setFilename(const string& name);
    
    // ISeekable interface
    virtual bool hasPercentSeek() const;
    virtual void seekPercent(const Rational& percent);
    virtual void seekPercentTo(const Rational& percent);
    virtual bool hasBytesSeek() const;
    virtual void seekBytes(const int64_t& offset);
    virtual void seekBytesTo(const u_int64_t& bytes);
    virtual bool hasTimeSeek() const;
    virtual void seekTime(const Interval& interval);
    virtual void seekTimeTo(const Time& time);
    
    // IBufferSize interface
    virtual void setBufferSize(const int& size);
    virtual int getBufferSize();
    
    // IProgress interface
    virtual void sendProgressInformation(const bool&);
    virtual void setProgressInterval(const int& buffers);

  public:

    MP3ReadNode(const char *name = "MP3ReadNode",  
		const StreamQueue::Mode queue_mode = StreamQueue::DEFAULT_MODE,
		const int size = StreamQueue::DEFAULT_SIZE);

    ~MP3ReadNode();
  };

}

#endif
