/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_IMAGENODE_HPP_
#define _NMM_IMAGENODE_HPP_

#include "nmm/base/GenericSourceNode.hpp"
#include "nmm/utils/utils.hpp"

namespace NMM {

  class ImageNode : public GenericSourceNode {

  protected:

    char *image_filename;               ///< filename for the logo image
    unsigned char *image_data;          ///< stores the actual image data
    int image_width, image_height;      ///< image width and height

    mutable double image_fps;
    looptimer timer;

    /**
     * Loads and prepares the PNG image.
     */
    Result preparePNGImage(char *filename);

    /**
     * Sets a pixel on the buffer image.
     */
     void setPixel(char *image, int x, int y,
		  unsigned char r, unsigned char g, unsigned char b);

    /**
     * Displays the png image on the buffer image
     */
    void paintPNGImage(char *image);

    /**
     * The standard doInit method from Node. Effects in ImageNode:
     * loads the image.
     */
    Result doInit();

    /**
     * This method returns a frame
     */
    Buffer* processBuffer(Buffer *);

    /**
     * The standard doActivate method from Node. Effects in ImageNode:
     * obtain and check the size of the incoming buffer image.
     */
    Result doActivate();

  public:

    ImageNode(const char* name = "ImageNode",
	      StreamQueue::Mode = StreamQueue::MODE_SUSPEND,
	      int = StreamQueue::DEFAULT_SIZE);
    virtual ~ImageNode();

    /**
     * The standard setParameter method from Node.
     * ImageNode understands the following parameters:
     * - string image_path : the filename of the image
     */
    Result setImage(string& );
  };

}

#endif //_NMM_IMAGENODE_HPP_
