/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_WAVREADNODE_HPP_
#define _NMM_WAVREADNODE_HPP_ _NMM_WAVREADNODE_HPP_

#include "nmm/base/GenericSourceNode.hpp"

#include "nmm/interfaces/file/IFileHandlerImpl.hpp"
#include "nmm/interfaces/file/IBufferSizeImpl.hpp"
#include "nmm/interfaces/file/ISeekableImpl.hpp"
#include "nmm/interfaces/general/progress/IProgressImpl.hpp"

namespace NMM {

  class WavReadNode :
    public GenericSourceNode,
    public IFileHandlerImpl,
    public ISeekableImpl,
    public IBufferSizeImpl,
    public IProgressImpl
  {
  public:
    Format* out_format;
    
    WavReadNode(const char * = "WavReadNode",
		StreamQueue::Mode = StreamQueue::MODE_SUSPEND,
		int = StreamQueue::DEFAULT_SIZE);
    virtual ~WavReadNode();
    
  protected:
    enum localstates { START, DATA, FINISHED };
    
    long int buffer_size;

    string filename;
    fstream in; /* file object */

    long int channels;
    long int bytes_per_sample;
    long int bytes_per_second;

    // seeking & file position
    u_int64_t         filesize;
    u_int64_t         filepos;
    int64_t           seekto;

    // the duration of the current track
    Interval          track_duration;

    localstates state;

    bool   send_format;

    // progress
    bool send_progress; //status flag whether to send progress information
    int             buffers_before_progress;
    int             buffers_progress;

    virtual Result doInit();
    virtual Result doDeinit();
    virtual Result doInitOutput();
    virtual Result doDeinitOutput();
    
    virtual Message* processBuffer(Buffer *);
    
    // IFileHandler interface
    virtual void setFilename(const string& name);
    
    // IBufferSize interface
    virtual void setBufferSize(const int& size);
    virtual int getBufferSize();
    
    // ISeekable interface
    virtual bool hasPercentSeek() const;
    virtual void seekPercent(const Rational& percent);
    virtual void seekPercentTo(const Rational& percent);
    virtual bool hasBytesSeek() const;
    virtual void seekBytes(const int64_t& offset);
    virtual void seekBytesTo(const u_int64_t& bytes);
    virtual bool hasTimeSeek() const;
    virtual void seekTime(const Interval& interval);
    virtual void seekTimeTo(const Time& time);

    // IProgress interface
    virtual void sendProgressInformation(const bool& in);
    virtual void setProgressInterval(const int& buffers);
  };

}

#endif /* _NMM_READWAVEVENTNODE_HPP_ */
