/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "CopyNode.hpp"
#include "nmm/base/memorymgr/Buffer.hpp"
#include "nmm/base/format/Property.hpp"
#include "nmm/utils/utils.hpp"
#include <stdio.h>
#include "nmm/base/registry/Plugin.hpp"

NMM_REGISTER_PLUGIN(NMM::CopyNode, "CopyNode")

namespace NMM {

  CopyNode::CopyNode(const char *name, StreamQueue::Mode qmode, int qsize) :
    GenericDemultiplexerNode(name, qmode, qsize) 
  {
    num_of_outputs = 2;
  }
  
  void CopyNode::setNumberOfOutputs(const int &value) {
    num_of_outputs = value;
  }
  
  Result CopyNode::doInit() {
    Format* any_format_i = new Format("*/*");
    getOwnInputProperty()->addNewFormat(any_format_i);
    
    Format* any_format_o = new Format("*/*");
    getOwnOutputProperty()->addNewFormat(any_format_o);
    
    return SUCCESS;
  }
  
  Result CopyNode::doInitOutput() {
    if(! (num_of_outputs > 0)) {
      ERROR_STREAM("Number of outputs is set to 0, cannot initialize 0 outputs!" << endl);
      return FAILURE;
    }

    /* get input format */
    if (!getInputFormat()) {
      GLOBAL_ERROR_STREAM("No Input Format! FAILURE" << endl);
      return FAILURE;
    }
    Format* i_format = new Format(*getInputFormat());
    getOwnWorkingInputProperty()->addNewFormat(i_format);
    getOwnWorkingInputProperty()->setDefaultFormat(i_format);
    
    for( unsigned short i = 0; i < num_of_outputs; i++ ) {
      string tag = string("") + i;
      DEBUG_STREAM("Adding stream with name: " << tag << " and with format: " << i_format->getFormatType() << endl);
      if(FAILURE == addOutputStream(tag.c_str())) {
	WARNING_STREAM("Could not add stream with name " << tag << endl);
      } else {
        Format* o_format = new Format(*i_format);
        o_format->setJackTag(tag);
        getOwnWorkingOutputProperty()->addNewFormat(o_format);
        getOwnWorkingOutputProperty()->setDefaultFormat(o_format, tag);
      }
    }
    
    return SUCCESS;
  }
  
  Result CopyNode::doDeinitOutput() {
    getOwnWorkingInputProperty()->erase();
    getOwnWorkingOutputProperty()->erase();
    
    for(unsigned short i = 0; i < num_of_outputs; i++) {
      string tag = string("") + i;
      DEBUG_STREAM("Removing stream with name " << tag << endl);
      if(FAILURE == removeOutputStream(tag.c_str())) {
	WARNING_STREAM("Could not remove stream with name " << tag << endl);
      }
    }
    return SUCCESS;
  }
  
  Message *CopyNode::processBuffer(Buffer *in) 
  {
    if(num_of_outputs) {
      return in;
  }
    else {
      in->release();
      return 0;
    }
  }

}
