/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_SWITCHNODE_HPP_
#define _NMM_SWITCHNODE_HPP_

#include "nmm/NMM.hpp"
#include "nmm/base/GenericDemultiplexerNode.hpp"
#include "nmm/base/format/Property.hpp"

namespace NMM {

  /**
   * \class SwitchNode
   */
  class SwitchNode :
    public GenericDemultiplexerNode
  {
  private:
    unsigned short num_of_outputs;
    Buffer *current_in;
    bool* output_connected;
    int switch_from;
    int switch_to;
    bool internal_switch_enabled;
    ThreadMutex switch_mutex;
    
  protected:
    /**
     * ISwitch interface. Re-implemented functionality from GenericNode
     */
    virtual void switchOutputs();
    virtual void setSwitchOutputsEnabled(const bool& enable);
    virtual bool getSwitchOutputsEnabled();

    /**
     * Here all formats are specified.
     */
    virtual Result doInit();
    
    /**
     * Here all output streams are added.
     */
    virtual Result doInitOutput();

    /**
     * Here all output streams are removed.
     */
    virtual Result doDeinitOutput();

    /**
     * The processBuffer method from superclass
     */
    virtual Message *processBuffer (Buffer *in);

  public:

    /**
     * The main constructor.
     */
    SwitchNode(const char *name = "SwitchNode", StreamQueue::Mode = StreamQueue::MODE_SUSPEND, int qsize = 1);
    
    virtual ~SwitchNode();
    
  };

}

#endif //_NMM_VIDEO_FILTER_OVERLAYNODE_HPP_



