/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */



#ifndef NMM_CAMERA1394_HPP
#define NMM_CAMERA1394_HPP



#include "GenericFireWireNode.hpp"
#include "nmm/interfaces/video/camera/ICameraImpl.hpp"

namespace L1394 {
  class Camera;
}

namespace NMM{


/* \class Camera1394
 * This class creates a wrapper-node for a FireWire device.
 *
 * A Camera1394 controls a L1394::Device object.
 */

class Camera1394 : public GenericFireWireNode, public ICameraImpl
{
public:

/**
  * Constructor
  */
  Camera1394(const char* name = "Camera1394");

/**
  * Destructor
  */
  virtual ~Camera1394(){};
protected:
  virtual Result doActivate();

  /* \fn doStart()
   * \brief This function starts the current device
   */
  virtual Result doStart();

  /* \fn doStop()
   * \brief This function stops a started device
   */
  virtual Result doStop();

  virtual Result doInit();


protected: //event methods

  virtual void setBrightness(const int& value);
  virtual void setAutoExposure(const int& value);
  virtual void setSharpness(const int& value);
  virtual void setWhiteBalance(const int& value);
  virtual void setBlueBalance(const int& value);
  virtual void setRedBalance(const int& value);

  virtual void setHue(const int& value)      ;
  virtual void setSaturation(const int& value);
  virtual void setGamma(const int& value)    ;
  virtual void setShutter(const int& value)  ;
  virtual void setGain(const int& value)     ;
  virtual void setIris(const int& value)     ;
  virtual void setFocus(const int& value)    ;
  virtual void setZoom(const int& value)     ;
  virtual void onePush()              ;


  virtual int getBrightness()        const;
  virtual int getAutoExposure()      const;
  virtual int getSharpness()         const;
  virtual int getWhiteBalance()      const;
  virtual int getBlueBalance()       const;
  virtual int getRedBalance()        const;

  virtual int getHue()               const;
  virtual int getSaturation()        const;
  virtual int getGamma()             const;
  virtual int getShutter()           const;
  virtual int getGain()              const;
  virtual int getIris()              const;
  virtual int getFocus()             const;
  virtual int getZoom()              const;


  virtual pair<int, int> getBrightnessRange()           const;
  virtual pair<int, int> getAutoExposureRange()  const;
  virtual pair<int, int> getSharpnessRange()     const;
  virtual pair<int, int> getWhiteBalanceRange()  const;
  virtual pair<int, int> getHueRange()           const;
  virtual pair<int, int> getSaturationRange()    const;
  virtual pair<int, int> getGammaRange()         const;
  virtual pair<int, int> getShutterRange()       const;
  virtual pair<int, int> getGainRange()          const;
  virtual pair<int, int> getIrisRange()          const;
  virtual pair<int, int> getFocusRange()         const;
  virtual pair<int, int> getZoomRange()          const;


  virtual bool hasBrightness()     const;
  virtual bool hasAutoExposure()   const;
  virtual bool hasSharpness()      const;
  virtual bool hasWhiteBalance()   const;
  virtual bool hasHue()            const;
  virtual bool hasSaturation()     const;
  virtual bool hasGamma()          const;
  virtual bool hasShutter()        const;
  virtual bool hasGain()           const;
  virtual bool hasIris()           const;
  virtual bool hasFocus()          const;
  virtual bool hasZoom()           const;


private:
  L1394::Camera* camera;
  bool automatic;
};
}

#endif

