/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    Network-Multimedia Workgroup,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Eric Peters <epeters@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_NETSINKNODE_HPP
#define NMM_NETSINKNODE_HPP

#include "nmm/base/GenericSinkNode.hpp"
#include "nmm/base/TransportStrategy.hpp"
#include "nmm/base/proxy/rtp/RTPStrategy.hpp"
#include "nmm/base/ISignalReceiver.hpp"
#include "nmm/interfaces/net/INetImpl.hpp"
#include "nmm/interfaces/net/IRTPParametersImpl.hpp"

#include <list>
using std::list;

namespace NMM {

  class NetSinkNode :
    public GenericSinkNode,
    public INetImpl,
    public IRTPParametersImpl
  {
  public: 
    NetSinkNode(const char* = "NetSinkNode");
    virtual ~NetSinkNode();
    
    Result enableInputStream();
  
  protected:
    virtual Message* processBuffer(Buffer*);
    
    virtual Result doInit();
    virtual Result doInitOutput();

    /**
     *  @name INet Interface
     */
    //@{

    /**
     *  @brief TSAddress for connecting
     *  @param tsaddress address to connect
     *  
     *  This method sets the @a tsaddress for the NetSinkNode. The type of 
     *  TSAddress (eg. UDPAddress, TCPAddress or RTPAddress) defines the type 
     *  of protocol to use (eg. UDP, TCP or RTP).
     */
    virtual void setTransportStrategy(const TSAddress*) throw(const BindingException&);
    
    virtual void setRTPStrategy(const string&, const int&, const int&);
    
    virtual void setPayloadType(const int&);
    
    virtual int  getPayloadType() const;
    //@}

    /**
     *  @name IRTPParameters Interface
     */
    //@{

    virtual string        getRTPSinkMediaType() const;

    virtual unsigned char getRTPSinkPayloadType() const;

    virtual string        getRTPSinkMapLine() const;

    virtual string        getRTPSinkAuxSDPLine() const;

    virtual unsigned short getRTPSeqNum() const;

    virtual unsigned      getRTPTimestamp() const;

    virtual unsigned      getRTPPort() const;
    
    virtual unsigned      getRTCPPort() const;

    //@}

  private:
    void setTransportStrategy(TransportStrategy*);

    TransportStrategy*   __transport_strategy;
        
    ThreadMutex          __internal_mutex;
    ThreadCondition      __internal_cond;

    int                  __payload_type;
  };

  /**
   * Class for receiving signals.
   */
  class NetSignalReceiver : public ISignalReceiver {
  private:
    NetSinkNode* __netsink;
    
  public:
    NetSignalReceiver(NetSinkNode* _netsink);
    virtual ~NetSignalReceiver();
    
    virtual void receiveSignal(Signal);
  };  
} //end namepsace

#endif
