/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_SINKNETNODE_HPP
#define NMM_SINKNETNODE_HPP

#include "nmm/base/GenericSinkNode.hpp"

#include "nmm/utils/net/INetAddr.hpp"
#include "nmm/utils/net/Sock.hpp"
#include "nmm/utils/net/SockAcceptor.hpp"

#include "nmm/interfaces/net/INetServerImpl.hpp"

namespace NMM {
  class SinkNetNode :
    public GenericSinkNode,
    public INetServerImpl
  {
  public: 
    SinkNetNode(const char* name);
    virtual ~SinkNetNode();
    
    /**
     * INetServer interface.
     */
    virtual void setPort(const int& port);
    virtual int getPort() const;
    virtual string getHostName() const;
    
  protected:
    virtual Buffer* processBuffer(Buffer* buffer);
    
    virtual Result doDeactivate();
    virtual Result doInit();
    virtual Result doInitOutput();
    virtual Result doActivate();
    
  private:
    Sock*         s_stream;
    SockAcceptor  s_accept;
    INetAddr      addr;
    bool is_connected;
    string local_host;
  };
} //end namepsace

#endif
