/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christian Gerstner <gerstner@graphics.cs.uni-sb.de>
 *                    Eric Peters <epeters@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef NMM_RFBPROTOCONTROLLER_HPP
#define NMM_RFBPROTOCONTROLLER_HPP NMM_RFBPROTOCONTROLLER_HPP


#include <sys/poll.h>
#include "nmm/utils/net/INetAddr.hpp"
#include "nmm/utils/net/Sock.hpp"
#include "nmm/utils/net/SockConnector.hpp"
#include "nmm/interfaces/net/IVNCImpl.hpp"

typedef unsigned long  CARD32;
typedef unsigned short CARD16;
typedef unsigned char  CARD8;

extern "C" {
#include "rfbproto.h"
#include "../../../utils/passwdtools/passwdtools.h"
#include <stdio.h>
}

typedef struct {
  bool shareDesktop;
  char* encodingsString;
  bool useBGR233;
  const char *passwordFile;
  unsigned char *enc_password;
  int copyRectDelay;
} AppData;

typedef struct _VncImage {
    int width, height;		/* size of image */
    char *data;			/* pointer to image data */
    int bitmap_pad;		/* 8, 16, 32 either XY or ZPixmap */
    int depth;			/* depth of image */
    int bytes_per_line;		/* accelarator to next line */
    int bits_per_pixel;		/* bits per pixel (ZPixmap) */
    bool hasNewImage;
} VncImage;

#define MAX_ENCODINGS 10
#define SERVER_PORT_OFFSET 5900

namespace NMM {
  
  class RFBProtoController : public NamedObject {

  public:
    
    enum Channelorder { rgb, bgr };
    enum Colorspace { rgb565, rgb24, rgb32, rgb00 };
    
    /** Constructor
     */
    RFBProtoController(IVNC::Mode mode = IVNC::REQUESTED);

    /** Destructor
     */
    ~RFBProtoController();
    
    Result connectToRFBServer();
    Result disconnectFromRFBServer();
    Result initialiseRFBConnection();
    void setHostAndPort(string, int);
    bool isConnected();
    void initDesktopAndColor();
    Result setFormatAndEncodings();
    int getFramebufferWidth();
    int getFramebufferHeight();
    int getBitsPerPixel();
    int getColordepth();
    int getColorspace();
    int getChannelorder();
    Result sendIncrementalFramebufferUpdateRequest();
    Result sendFramebufferUpdateRequest(int, int, int, int, bool);
    void setHostname(string);
    void setPort(int);
    void setTolerance(float);
    void setInterval(float);
    void setShared(bool);
    void setPasswdPath(const char*);
    void setPasswd(const string);
    float getTolerance() const;
    float getInterval() const;
    bool isShared();
    Result handleRFBServerMessage(char*);
    void sendPointerEvent(int, int, int);
    void sendKeyEvent(CARD32, bool); 
    bool hasNewImage();
    void setHasNewImage(bool);
    bool getTimeIsUp();
    void setTimeIsUp(bool);
    VncImage* getVncImage();
    IVNC::Mode getMode();
    bool hasRequested();
    void setRequested(bool);
    void setMode(IVNC::Mode mode);

  private:

    AppData          appData;
    //bool           errorMessageOnReadFailure;
    bool             is_connected;
    bool             shared;
    float            tolerance;
    float            interval;
    Sock*            s_stream;
    INetAddr         addr;
    SockConnector    s_connect;
    VncImage*        vncimage;
    rfbPixelFormat   myFormat;
    rfbServerInitMsg si;
    int              endianTest;
    IVNC::Mode             mode;
    bool             requested;
    bool             timeIsUp;
    char*            updateBuffer;
              
    void printPixelFormat(rfbPixelFormat* format);
    void setVisualAndCmap();
    void desktopInitBeforeRealization();
    int round(int, int);
    VncImage* createImage(int, int, int, int, char*, int);
    void copyDataToScreen(char*, int, int, int, int);
    void readIntoVNCImage(int, int, int, int);
    void read(char*, int);
    void write(char*, int);
    int swap16IfLE(int);
    int swap32IfLE(int);

  };
} // end namespace
#endif
      
