/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_MULTIPLEOSDNODE_HPP_
#define _NMM_MULTIPLEOSDNODE_HPP_

#include "nmm/base/GenericFilterNode.hpp"
#include "nmm/base/GenericSourceNode.hpp"
extern "C" {
#include "nmm/plugins/video/osd/font/font_load.h"
}

#define OSD_PLAY 0x01
#define OSD_PAUSE 0x02
#define OSD_STOP 0x03
#define OSD_REW 0x04
#define OSD_FFW 0x05
#define OSD_CLOCK 0x06
#define OSD_CONTRAST 0x07
#define OSD_SATURATION 0x08
#define OSD_VOLUME 0x09
#define OSD_BRIGHTNESS 0x0A
#define OSD_HUE 0x0B

#define OSD_PB_START 0x10
#define OSD_PB_0 0x11
#define OSD_PB_END 0x12
#define OSD_PB_1 0x13

#define RGBA32	0
#define YVA12   1

#include <png.h>

#include "nmm/nmm_hash_map.hpp"

namespace NMM {

  class OSDItem {
  public:
    size_t x_offset;
    size_t y_offset;
    size_t width;
    size_t height;
    size_t depth;
    bool enabled;

    OSDItem();

    virtual Result draw(char *data, int color, size_t width, size_t height) = 0;
  };

  class OSDImagePNG : public OSDItem {
  private:
    png_bytep *png_rows;               ///< stores the PNG image
    png_uint_32 png_width, png_height; ///< PNG image width and height

    int image_width;        ///< width of the image in the input stream
    int image_height;       ///< height of the image in the input stream
    Result preparePNGLogo(const char *filename);
    void setPixel(char *image, int x, int y, unsigned char r, unsigned char g, unsigned char b, unsigned char a);
    void paintPNGLogo(char *image, int x, int y);
  public:
    OSDImagePNG();
    Result load(const string url);
    virtual Result draw(char *data, int color, size_t width, size_t height);
  };

  class OSDText : public OSDItem {
  private:
    char *current_data;   

    int image_width;         
    int image_height;       
    int color;

    font_desc_t* osd_font;
    char* osd_text;
    int osd_progbar_type;
    int osd_progbar_value;   // 0..255
    int dxs;
    int dys;
    
    void draw_alpha_yv12(int w,int h, unsigned char* src, unsigned char *srca, int srcstride, unsigned char* dstbase,int dststride);
    void draw_alpha_rgb32(int w,int h, unsigned char* src, unsigned char *srca, int srcstride, unsigned char* dstbase,int dststride);
    void draw_alpha(int x0,int y0, int w,int h, unsigned char* src, unsigned char *srca, int stride);
    void draw_text_progbar();
    void draw_text_osd();
    void draw_text();

  public:
    OSDText();
    virtual Result draw(char *data, int color, size_t width, size_t height);
    Result setText(const string text);
    Result setFont( font_desc_t* osd_font );
  };

  class MultipleOSDNode : public GenericSourceNode {

  protected:
    typedef map<string, OSDItem *> items_t;
    items_t items;

    int image_width;         
    int image_height;       
    int color;
    int buff_size;

    font_desc_t* osd_font;

    ThreadMutex osd_mutex;
    ThreadCondition* osd_cond;
    bool new_text_flag;
  
    Result doActivate();
    Buffer *processBuffer(Buffer *);

  public:

    MultipleOSDNode(const char *name = "MultipleOSDNode",
		    StreamQueue::Mode = StreamQueue::MODE_SUSPEND,
		    int = StreamQueue::DEFAULT_SIZE);
    ~MultipleOSDNode();

    Result addText(const string id, const string text, const size_t x_offset, const size_t y_offset);
    Result addImage(const string id, const string url, const size_t x_offset, const size_t y_offset);
    Result setOffset(const string id, const size_t x_offset, const size_t y_offset);
    Result setText(const string id, const string text);
    Result setItemEnabled(const string id, const bool enable);
    Result DisableItems();
    Result update();

    Result setFont( char *font_name, float font_factor );
  };

}

#endif
