/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_OSDNODE_HPP_
#define _NMM_OSDNODE_HPP_

#include "nmm/base/GenericFilterNode.hpp"
#include "nmm/base/GenericSourceNode.hpp"
extern "C" {
#include "nmm/plugins/video/osd/font/font_load.h"
}

#include "nmm/interfaces/video/osd/IOSDImpl.hpp"

#define OSD_PLAY 0x01
#define OSD_PAUSE 0x02
#define OSD_STOP 0x03
#define OSD_REW 0x04
#define OSD_FFW 0x05
#define OSD_CLOCK 0x06
#define OSD_CONTRAST 0x07
#define OSD_SATURATION 0x08
#define OSD_VOLUME 0x09
#define OSD_BRIGHTNESS 0x0A
#define OSD_HUE 0x0B

#define OSD_PB_START 0x10
#define OSD_PB_0 0x11
#define OSD_PB_END 0x12
#define OSD_PB_1 0x13

#define RGBA32	0
#define YVA12   1


namespace NMM {
  class OSDNode : public GenericSourceNode,
                  public IOSDImpl {
  private:
    int image_width;         
    int image_height;       
    Buffer *curr_buffer;    // Current buffer

    font_desc_t* osd_font;
    string osd_text;
    int osd_progbar_type;
    int osd_progbar_value;   // 0..255
    int dxs;
    int dys;
    
    int buff_size;
    int color;
    
    ThreadMutex osd_mutex;
    ThreadCondition* osd_cond;
    bool new_text_flag;
    
    void draw_alpha_yv12(int w,int h, unsigned char* src, unsigned char *srca, int srcstride, unsigned char* dstbase,int dststride);
    void draw_alpha_rgb32(int w,int h, unsigned char* src, unsigned char *srca, int srcstride, unsigned char* dstbase,int dststride);
    void draw_alpha(int x0,int y0, int w,int h, unsigned char* src, unsigned char *srca, int stride);
    void draw_text_progbar();
    void draw_text_osd();
    void draw_text();
    
    void setupOutputFormats(Property* property);
    
  protected:
    virtual Result doInit();
    virtual Result doInitOutput();
    virtual Result doActivate();
    
    virtual Buffer *processBuffer(Buffer* buffer);
    
    virtual Result setText( const string& text );
    virtual Result setFont( const string& font_name, const float& font_factor );
    
  public:
    OSDNode(const char *name = "OSDNode",
	    StreamQueue::Mode = StreamQueue::MODE_SUSPEND,
	    int = StreamQueue::DEFAULT_SIZE);
    
    ~OSDNode();
  };

}

#endif
