/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "textinput.hpp"

namespace NMM{

    TextInput::TextInput( IOSDManager* manager, IWidgetFactory* factory, string _font ) 
      : OSDElement(manager, factory),
        currentKey(0),
        currentChar(0),
        clock(),
        withinTime(false),
        mode(ALPHA),
        font(_font)
    {
    setColors( 0xff000000, 0xff100000, 0xff707070, 0xff303030, 0xff101010 );
    
    text.reset(factory->createTextView());
    text->setFont(font);
    
    text->setSelectionColor( sel_inactive_color );
    
    plane.reset(factory->createPlaneDecorator( *text ));
    plane->setPlaneColor( win_inactive_color );
    border.reset(factory->createBorderDecorator( *plane ));
    header.reset(factory->createHeaderDecorator( *border ));
    header->setFont(font);
    
    win->setContent( *header );
    text->addLine("");

    /*<Initialize the default Mapping for keys to chars */
    keyChars.push_back(".,:-_"); //key 0
    keyChars.push_back(" ");   //key 1
    keyChars.push_back("abc"); //key 2
    keyChars.push_back("def"); //key 3
    keyChars.push_back("ghi"); //key 4
    keyChars.push_back("jkl"); //key 5
    keyChars.push_back("mno"); //key 6
    keyChars.push_back("pqrs");//key 7
    keyChars.push_back("tuv"); //key 8
    keyChars.push_back("wxyz");//key 9
    /*>End initialize Mapping */

    timeout = clock.getTime();
    maxTimeWait.sec = 2;
    maxTimeWait.nsec = 0;
  }
  
  TextInput::~TextInput(){
    deleteWidgets();
  }

  void TextInput::setColors( unsigned int _win_active_color, unsigned int _win_inactive_color,
    unsigned int _sel_active_color, unsigned int _sel_active_color2, unsigned int _sel_inactive_color ){
    win_active_color   = _win_active_color;
    win_inactive_color = _win_inactive_color;
    sel_active_color   = _sel_active_color;
    sel_active_color2  = _sel_active_color2;
    sel_inactive_color = _sel_inactive_color;

  }
  void TextInput::selectWindow(unsigned int mode){
    if ( mode == 1 ) {
      text->setSelectionColor( sel_active_color2 );
    } else {
      text->setSelectionColor( sel_active_color );
    }
    plane->setPlaneColor( win_active_color );

  }
  
  void TextInput::deSelectWindow(){
    text->setSelectionColor( sel_inactive_color );
    plane->setPlaneColor( win_inactive_color );

  }

  void TextInput::setHeader(string h){
    header->setText( h );
  }
  
  string TextInput::getContent(){
    return text->getLineContent(1);
  }

  void TextInput::addString(string s){
    string temp = text->getLineContent(1);
    text->deleteAllLines();
    if(withinTime){
      text->addLine(temp.substr(0, temp.size()-1)+s);
    }else{
      text->addLine(temp+s);
    }
  }
  
  void TextInput::clear(){
    text->deleteAllLines();
    text->addLine("");
  }
  
  void TextInput::erase(){
    string tmp = text->getLineContent(1);
    text->deleteAllLines();
    text->addLine(tmp.substr(0,tmp.size()-1));
  }

  void TextInput::setMode(mode_e m){
    mode = m;
  }

  Result TextInput::key0(){
    if(mode == NUMERIC){
      addString("0");
    }else{
      if(static_cast<unsigned int>(currentChar) >= keyChars[0].size()) currentChar = 0;
      addString(string(&(keyChars[0][currentChar]),1));
    }
    return SUCCESS;
  }
  Result TextInput::key1(){
    if(mode == NUMERIC){
      addString("1");
    }else{
      if(static_cast<unsigned int>(currentChar) >= keyChars[1].size()) currentChar = 0;
      addString(string(&(keyChars[1][currentChar]),1));
    }
    return SUCCESS;
  }
  Result TextInput::key2(){    
    if(mode == NUMERIC){
      addString("2");
    }else{
      if(static_cast<unsigned int>(currentChar) >= keyChars[2].size()) currentChar = 0;
      addString(string(&(keyChars[2][currentChar]),1));
    }
    return SUCCESS;
  }
  Result TextInput::key3(){
    if(mode == NUMERIC){
      addString("3");
    }else{
      if(static_cast<unsigned int>(currentChar) >= keyChars[3].size()) currentChar = 0;
      addString(string(&(keyChars[3][currentChar]),1));
    }
    return SUCCESS;
  }
  Result TextInput::key4(){
    if(mode == NUMERIC){
      addString("4");
    }else{
      if(static_cast<unsigned int>(currentChar) >= keyChars[4].size()) currentChar = 0;
      addString(string(&(keyChars[4][currentChar]),1));
    }
    return SUCCESS;
  }
  Result TextInput::key5(){
    if(mode == NUMERIC){
      addString("5");
    }else{
      if(static_cast<unsigned int>(currentChar) >= keyChars[5].size()) currentChar = 0;
      addString(string(&(keyChars[5][currentChar]),1));
    }
    return SUCCESS;
  }
  Result TextInput::key6(){
    if(mode == NUMERIC){
      addString("6");
    }else{
      if(static_cast<unsigned int>(currentChar) >= keyChars[6].size()) currentChar = 0;
      addString(string(&(keyChars[6][currentChar]),1));
    }
    return SUCCESS;
  }
  Result TextInput::key7(){
    if(mode == NUMERIC){
      addString("7");
    }else{
      if(static_cast<unsigned int>(currentChar) >= keyChars[7].size()) currentChar = 0;
      addString(string(&(keyChars[7][currentChar]),1));
    }
    return SUCCESS;
  }
  Result TextInput::key8(){    
    if(mode == NUMERIC){
      addString("8");
    }else{
      if(static_cast<unsigned int>(currentChar) >= keyChars[8].size()) currentChar = 0;
      addString(string(&(keyChars[8][currentChar]),1));
    }
    return SUCCESS;
  }
  Result TextInput::key9(){
    if(mode == NUMERIC){
      addString("9");
    }else{
      if(static_cast<unsigned int>(currentChar) >= keyChars[9].size()) currentChar = 0;
      addString(string(&(keyChars[9][currentChar]),1));
    }
    return SUCCESS;
  }


  Result TextInput::keyPressed(int k){


    //key pressed within 2 secs?
    if((clock.getTime() - timeout).sec < maxTimeWait.sec && k == currentKey){
      currentChar++;
      withinTime = true;
    }else{
      currentChar = 0;
      withinTime = false;
    }

    currentKey = k;    
    //set new timeout-value
    timeout = clock.getTime();

    switch(k){
    case 0:
      key0();
      break;
    case 1:
      key1();
      break;
    case 2:
      key2();
      break;
    case 3:
      key3();
      break;
    case 4:
      key4();
      break;
    case 5:
      key5();
      break;
    case 6:
      key6();
      break;
    case 7:
      key7();
      break;
    case 8:
      key8();
      break;
    case 9:
      key9();
      break;
    }
    
    return SUCCESS;
  }

  void TextInput::setNewKeyCharMapping(vector<string> v){
    keyChars = v;
  }
  vector<string> TextInput::getKeyCharMapping(){
    return keyChars;
  }


  void TextInput::deleteWidgets(){
    if ( text.get() ) {
      manager->deleteWidget(*text);
      text.reset(NULL);
    }

    if ( plane.get() ) {
      manager->deleteWidget(*plane);
      plane.reset(NULL);
    }

    if ( border.get() ) {
      manager->deleteWidget(*border);
      border.reset(NULL);
    }

    if ( header.get() ) {
      manager->deleteWidget(*header);
      header.reset(NULL);
    }

  }


}//end namespace
