/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "timeunit.hpp"

namespace NMM {
  TimeUnit::TimeUnit( IOSDManager* manager, IWidgetFactory* factory, string _font ) : OSDElement(manager, factory),
                                                                                      font(_font) {
    setColors( 0x80ff0000 );
    
    text.reset(factory->createTimeView());
    text->setFont(font);
    plane.reset(factory->createPlaneDecorator( *text ));
    plane->setPlaneColor( color1 );
    border.reset(factory->createBorderDecorator( *plane ));
    header.reset(factory->createHeaderDecorator( *border ));
    header->setFont(font);
    
    win->setDimension( manager->getMaxWidth(font)*4, manager->getMaxHeight(font)+24 );
    win->setContent( *header );
  }
  
  TimeUnit::~TimeUnit() {
    deleteWidgets();
  }

  void TimeUnit::deleteWidgets() {
    if ( text.get() ) {
      manager->deleteWidget(*text);
      text.reset(NULL);
    }

    if ( plane.get() ) {
      manager->deleteWidget(*plane);
      plane.reset(NULL);
    }
    
    if ( border.get() ) {
      manager->deleteWidget(*border);
      border.reset(NULL);
    }

    if ( header.get() ) {
      manager->deleteWidget(*header);
      header.reset(NULL);
    }
  }
  
  void TimeUnit::setHeader( string h ) {
    header->setText( h );
  }
  
  void TimeUnit::setColors( unsigned int _color1 ) {
    color1 = _color1;
  }
}
