/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_VIDEO_FILTER_OSDMANAGERNODE_HPP_
#define _NMM_VIDEO_FILTER_OSDMANAGERNODE_HPP_ _NMM_VIDEO_FILTER_OSDMANAGERNODE_HPP_

#include "nmm/base/GenericMultiplexerNode.hpp"

#include "nmm/interfaces/video/osd/manager/IOSDManagerImpl.hpp"
#include "nmm/interfaces/video/osd/manager/IWidgetFactoryImpl.hpp"
#include "nmm/interfaces/video/osd/manager/IBitmapReader.hpp" // XXX: (fw) currently the OSDManagerNode implements IBitmapReader

#include "OSDElements/messageunit.hpp"
#include "OSDElements/timeunit.hpp"
#include "OSDElements/progressbarunit.hpp"
#include "OSDElements/symbolunit.hpp"
#include "OSDElements/colorbarunit.hpp"
#include "window.hpp"

#define RGBA32 0
#define YVA12  1

namespace NMM {
  
  class OSDManagerNode : public GenericMultiplexerNode,
                         public IOSDManagerImpl,
                         public IWidgetFactoryImpl,
                         public IBitmapReaderImpl // XXX: (fw) currently the OSDManagerNode implements IBitmapReader
  {
    
    typedef map<int, Window*> windows_t;
    typedef map<int, Widget*> widgets_t;
    typedef map<string, BitmapFont*> fonts_t;
    
  private:
    Buffer*       lastBuffer;
    bool          paused;
    int           max_window_id;
    int           max_widget_id;
    windows_t     windows;      // map of window entries
    widgets_t     widgets;      // map of widgets
    fonts_t       fonts;
    BitmapReader* bmreader;
    bool          send_symbol_change;
    string        new_symbol;
    bool          doupdate;
    
    MessageUnit*     message;
    TimeUnit*        time;
    ProgressbarUnit* bar;
    ColorBarUnit*    color_bar;
    SymbolUnit*      symbol;
    bool             create_time;
    
    ThreadMutex      osd_mutex;
    int image_width;        ///< width of the image in the input stream
    int image_height;       ///< height of the image in the input stream
    int ratio_x;            ///< x-aspect-ratio of the image in the input stream
    int ratio_y;            ///< y-aspect-ratio of the image in the input stream
    int reference_width;    // this is the width we scale to
    int reference_height;   // this is the height we scale to
    double scale_factor_x;
    double scale_factor_y;
    int color;              ///< colorspace: RGBA32 or YVA12
    Timestamp old_timestamp;
    
    Buffer* out_buffer;
    bool send_format_changed;
    bool format_changed_sent;
    bool save_working_flag;
    Buffer* save_in_buffer;

  protected:
    virtual Result doInit();
    virtual Result doInitOutput();
    virtual Result doFlush();
    virtual Result doStart();
    virtual Result doStop();
    
    virtual Message* processBuffer(Buffer *in_buffer);
    
    /** IOSManager interface. */
    virtual Result loadFont(const string& id, const string& xmlfilename, const string& fontname);
    virtual list<string> getFontList() const;
    virtual Result update();
    virtual IWidget* getWidget(const int& id);
    virtual IWindow* getWindow(const int& id);
    virtual IWindow* createWindow();
    virtual void deleteWindow(const IWindow& window);
    virtual void deleteWidget(const IWidget& widget);
    virtual Result displayMessage(const string& header, const string& text, const string& label1, const string& label2 = "",
                                  const string& label3 = "", const unsigned int& sel = 0);
    virtual Result deleteMessage();
    virtual Result displayTime(const string& header = "", const unsigned int& x = 500, const unsigned int& y = 410);
    virtual Result createTime(const bool& ct);
    virtual Result resetTime();
    virtual Result hideTime();
    virtual Result setTime(const u_int64_t& time);
    virtual u_int64_t getTime();
    virtual Result displaySymbol(const string& symbol, const unsigned int& x = 600, const unsigned int& y = 50);
    virtual Result hideSymbol();
    virtual Result displayProgressBar(const string& header = "", const unsigned int& x = 50, const unsigned int& y = 470,
				      const unsigned int& width = 620);
    virtual Result hideProgressBar();
    virtual void setMaxProgress(const u_int64_t& max);
    virtual void setProgress(const u_int64_t& progress);
    virtual void updateProgress();

    virtual Result displayColorBar(const unsigned int& x = 80, const unsigned int& y = 480);
    virtual Result hideColorBar();
    virtual void setColorBarText(const string& red, const string& green, const string& yellow, const string& blue);
    virtual void setRedColorBarText(const string& text);
    virtual void setGreenColorBarText(const string& text);
    virtual void setYellowColorBarText(const string& text);
    virtual void setBlueColorBarText(const string& text);
    virtual string getRedColorBarText();
    virtual string getGreenColorBarText();
    virtual string getYellowColorBarText();
    virtual string getBlueColorBarText();

    virtual Result messageLeft();
    virtual Result messageRight();
    virtual string messageID();
    virtual void setPause(const bool& value);
    virtual void sendPauseEvent();
    virtual IBitmapReader* getBitmapReader();
    
    virtual void checkCharDimension(const string& id, const unsigned char& c, unsigned int& x, unsigned int& y) const;
    virtual unsigned int getMaxWidth(const string& id) const;
    virtual unsigned int getMaxHeight(const string& id) const;
    virtual void setLineSpace(const string& id, const unsigned int& space);
    virtual unsigned int getLineSpace(const string& id) const;
    
    /** IWidgetFactory interface. */
    virtual IWidgetComposite* createWidgetComposite();
    virtual IProgressBar* createProgressBar();
    virtual ITextView* createTextView();
    virtual ITimeView* createTimeView();
    virtual IImageView* createImageView();
    virtual IRadioButtons* createRadioButtons();
    virtual IButtons* createButtons();
    virtual IBorderDecorator* createBorderDecorator(const IWidget& internal);
    virtual IHeaderDecorator* createHeaderDecorator(const IWidget& internal);
    virtual IPlaneDecorator* createPlaneDecorator(const IWidget& internal);
    virtual IScrollDecorator* createScrollDecorator(const IWidget& internal);
    
    /** IBitmapReader interface */
    virtual bool loadImage(const string& id, const string& filename, const IBitmapReader::Format& f);
    virtual void getImage(const string& id, unsigned int& w, unsigned int& h);
    virtual bool createImage(const string& id, const unsigned int& width, const unsigned int& height,
                             const unsigned int& color1, const unsigned int& color2, const unsigned int& color3,
                             const IBitmapReader::Format& f);
    virtual void freeImage(const string& id);
    virtual void freeAll();
    
    /** IFormatChange interface */
    virtual Result setFormat(const Format& format);
    
    void sendSymbolChange(const string& sym);
    
  public:
    OSDManagerNode(const char *name = "OSDManagerNode",
                   const StreamQueue::Mode queue_mode = StreamQueue::MODE_SUSPEND,
                   const int queue_size = 1 );
    virtual ~OSDManagerNode();
    
    
    /**
     * Get own widget.
     * Returns a pointer to the local widget object from an interface.
     * This is done using the widget ID. We assume that the widget that the interface
     * refers to was created locally.
     */
    Widget* getOwnWidget(const IWidget& iwidget);
    
    /**
     * Get own window.
     * Returns a pointer to the local window object from an interface.
     * This is done using the window ID. We assume that the window that the interface
     * refers to was created locally.
     */
    Window* getOwnWindow(const IWindow& iwindow);
    
    /** Get own widget by id. */
    Widget* getOwnWidget(int id);

    /** Get own window by id. */
    Window* getOwnWindow(int id);
    
    
    /** Add a window. */
    void addWindow( Window* win );
    
    /** Remove a window. */
    void delWindow( Window* win );
    
    /** Add a widget. */
    void addWidget( Widget* win );
    
    /** Remove a widget. */
    void delWidget( Widget* win );
    
    
    void setFontList( fonts_t f ) { fonts = f; };
    void setBitmapReader( BitmapReader* br ) { bmreader = br; };
    
    /**
     * add font to list
     * \param id font identifier
     * \param font pointer to bitmap font
     */
    void setFont( string id, BitmapFont* font );
    
    /**
     * \returns pointer to Bitmapfont
     * \param id font identifier
     */
    BitmapFont* getFont( string id ) const;
  };
}

#endif //_NMM_VIDEO_FILTER_OSDMANAGERNODE_HPP_
