/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, 
 * USA.
 */


#ifndef NMM_RTJPEGDECODENODE_HPP
#define NMM_RTJPEGDECODENODE_HPP

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include "nmm/interfaces/general/IEncodeImpl.hpp"
#include "nmm/interfaces/video/IFrameEncodeModeImpl.hpp"
#include "nmm/base/GenericProcessorNode.hpp"
#include "nmm/utils/Benchmark.hpp"

extern "C" {
#include "RTjpeg/RTjpeg.h"
}

namespace NMM {

  /**
   * This Node uses RTJPEG-Decompression.
   */
  class RTJPEGDecodeNode :
    public GenericProcessorNode,
    public IFrameEncodeModeImpl,
    public IEncodeImpl
  {
  private:
    Benchmark benchmark;
    
  protected:
    unsigned char quality;
    size_t quality_size;
    
    IFrameEncodeMode::Mode mode;
    size_t mode_size;
    
    struct RTjpeg_struct *rtjpeg_struct;
    
    long unsigned int *rtjpeg_tables;
    size_t rtjpeg_tables_size;
    
    int image_width;
    int image_height;
    
    char *inter_frame_buffer;
    
    bool first_time;
    
  protected:
    virtual Result doInit();
    virtual Result doInitOutput();
    virtual Result doActivate();
    
    virtual Buffer *processBuffer(Buffer *in_buffer);
    
    /**
     * IEncode interface.
     * The bitrate parameter is not used. The quality is internally scaled from [0...1] to [0..255].
     */
    virtual void setQuality(const float& q);
    virtual float getQuality();
    virtual void setBitrate(const int& i);
    virtual int getBitrate();
    
    /**
     * IFrameEncodeMode interface.
     */
    virtual void setFrameEncodeMode(const IFrameEncodeMode::Mode &mode);
    virtual IFrameEncodeMode::Mode getFrameEncodeMode() const;
    
   public:
    
    RTJPEGDecodeNode(const char *name = "RTJPEGDecodeNode", 
		     const StreamQueue::Mode queue_mode = StreamQueue::MODE_SUSPEND, 
		     const int queue_size = 1 );
    virtual ~RTJPEGDecodeNode();
  };
}

#endif
