/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Florian Winter <fw@graphics.cs.uni-sb.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "FrequencyTable.hpp"
#include "nmm/utils/utils.hpp"

namespace NMM {
  
  FrequencyTable::FrequencyTable()
  {}
  
  FrequencyTable::FrequencyTable(const string& name)
    : __name(name)
  {}
  
  unsigned int FrequencyTable::getChannelFrequency(unsigned int channel) const
    throw (const Exception&) 
  {
    if (channel >= __channels.size()) {
      throw Exception(string("Invalid channel number ") + static_cast<int>(channel));
    }
    
    return __channels[channel].channel_frequency;
  }

  const string& FrequencyTable::getChannelName(unsigned int channel) const
    throw (const Exception&) 
  {
    if (channel >= __channels.size()) {
      throw Exception(string("Invalid channel number ") + static_cast<int>(channel));
    }
    
    return __channels[channel].channel_name;
  }
  
  int FrequencyTable::findChannel(const string& name) const {
    map<string, unsigned int>::const_iterator found(__map.find(name));
    if (found == __map.end()) {
      return -1;
    }
    
    return found->second;
  }
  
  void FrequencyTable::addChannel(const string& name, unsigned int frequency)
    throw (const Exception&)
  {
    map<string, unsigned int>::const_iterator found(__map.find(name));
    if (found != __map.end()) {
      throw Exception("channel " + name + " already exists");
    }
    
    unsigned int channel = __channels.size();
    __channels.push_back(Channel(name, frequency));
    __map[name] = channel;
  }
  
} // namespace NMM
