/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christoph Wellner <chwellner@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "WebServerProcessor.hpp"
#include "HttpConstants.hpp"

#include "nmm/utils/utils.hpp"

#include <iostream>

namespace NMM{
  WebServerProcessor::WebServerProcessor(unsigned int threadNr)
    :keepRunning(true),
     threadName("")
  {
    threadName = "serverProcess" + itoa(threadNr);
  }
  
  WebServerProcessor::~WebServerProcessor(){}
  
  void WebServerProcessor::processRequest(Socket *request){
    clientRequestPool.push_back(request);
    mutex.lock();
    tcond.notifyAll();
    mutex.unlock();

  }
  void WebServerProcessor::notifyProcessors(){
    mutex.lock();
    tcond.notifyAll();
    mutex.unlock();
    
  }
  
  void WebServerProcessor::registerHandler(WebRequestHandler* handler){
    handlers[handler->getURI()] = handler;
  }
  
  void WebServerProcessor::start(){
    activateThread();
  }
  void WebServerProcessor::stop(){
    keepRunning = false;
    tcond.notifyAll();
    deactivateThread();
  }

  void WebServerProcessor::run(){
    string ipaddress = "unknown";
    
    while(keepRunning){


      if(clientRequestPool.empty()){
        mutex.lock();
        tcond.wait();
      }

      if(clientRequestPool.empty()){
        mutex.unlock();
        continue;
      }else{

        accessMutex.lock();
        Socket *connection = clientRequestPool.front();
        clientRequestPool.pop_front();
        accessMutex.unlock();

        mutex.unlock();

        try{

          WebServerRequest *request = new WebServerRequest(connection);
          WebServerResponse * response = new WebServerResponse(connection);

          if(handlers[request->getRequestURI()]){
            response->sendOK();
            if(request->getMethod() == "post" || request->getMethod() == "POST"){
              ((WebRequestHandler*)handlers[request->getRequestURI()])->doPost(request, response);
            }else{
              ((WebRequestHandler*)handlers[request->getRequestURI()])->doGet(request, response);
            }
          }else{
            response->sendError(HTTP_NOT_FOUND, "Requested source not available");
          }
          response->close();
          
          if(request)  delete request;
          if(response) delete response;

          if(connection) {
            connection->close();
            delete connection;
          }

        }catch(const Exception &e){
          //cerr << "Exception : " << e << endl;
        }
      }
    }//end while
  }
}
 
//Initialize static variable
list<NMM::Socket*> NMM::WebServerProcessor::clientRequestPool;
map<string, NMM::WebRequestHandler*> NMM::WebServerProcessor::handlers;
NMM::ThreadMutex NMM::WebServerProcessor::mutex;
NMM::ThreadCondition NMM::WebServerProcessor::tcond(mutex);
  
