/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "EventRecorder.hpp"
#include "nmm/base/serialize/StdStream.hpp"
#include "nmm/base/Event.hpp"
#include "nmm/base/serialize/MagicNumberValueStream.hpp"

#include <fstream>

namespace NMM {
  EventRecorder::EventRecorder(const string& file)
    :  __value_stream (0), __std_o_stream (0), __o_stream (0)
  {
    /* create a copy of the desired value_stream */
    __value_stream = new MagicNumberOValueStream();

    /* create the __o_stream to write the events */
    __o_stream = new std::ofstream(file.c_str());
    if (! *__o_stream )
      throw Exception("Could not open file " + file );

    __std_o_stream = new StdOStream(__o_stream);

    /* set the partner stream */
    __value_stream->setOPartnerStream(__std_o_stream);

    /* write the type of the OValueStream into the first line (need for the EventPlayer)*/
    (*__o_stream) << __value_stream->getOValueStreamType() << endl;
  }

  EventRecorder::EventRecorder(const string& file, const OValueStream& ov)
    :  __value_stream (0), __std_o_stream (0), __o_stream (0)
  {
    /* create a copy of the desired value_stream */
    __value_stream = ov.copy();

    /* create the __o_stream to write the events */
    __o_stream = new std::ofstream(file.c_str());
    if (! *__o_stream )
      throw Exception("Could not open file " + file );

    __std_o_stream = new StdOStream(__o_stream);

    /* set the partner stream */
    __value_stream->setOPartnerStream(__std_o_stream);

    /* write the type of the OValueStream into the first line (need for the EventPlayer)*/
    (*__o_stream) << __value_stream->getOValueStreamType() << endl;
  }

  EventRecorder::~EventRecorder() {
    /* flush the __value_stream*/
    if (__value_stream )
      __value_stream->writeData();

    /* Begin> delete allocated objects */
    delete __value_stream;
    delete __std_o_stream;
    delete __o_stream;
    /* End< */
  }

  Result EventRecorder::receiveEvent(const CEvent& event, const IEventSender* ) {
    CEvent ce(event);
    return writeValue(ce);
  }
  Result EventRecorder::receiveEvent(const Event& event, const IEventSender*  ) {
    CEvent ce (event);
    return writeValue(ce);
  }

  Result EventRecorder::receiveEvent(const CEvent& event, const IEventSender* ) const {
    CEvent ce(event);
    return writeValue(ce);
  }
  Result EventRecorder::receiveEvent(const Event& event, const IEventSender* )  const {
    CEvent ce(event);
    return writeValue(ce);
  }


  Result EventRecorder::writeValue(CEvent& event) const {
    /* Set a timestamp if it is invalid */
    if ( ! event.getTimestamp().is_valid ) {
      Timestamp new_ts;
      new_ts.sync_time = getTime();
      new_ts.is_valid  = true;

      event.setTimestamp(new_ts);
    }

    /* write the event to the stream */
    (*__value_stream) << event;
    return SUCCESS;
  }
}
