/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 */

#include <stdio.h>
#include <string>
#include <iostream>
#include <fstream>
#include <sstream>
#include <list>
#include "ParserStructs.hpp"
//#include "ClicApplication.hpp"

#include "nmm/nmm_strstream.hpp"

#define YYSTYPE const char *

extern unsigned int lineno;
extern char *yytext;


namespace NMM {
  class NodeDescription;
}

extern "C"
{
  int yyparse(void);
  int yylex(void);
  int yywrap()
  {
    return 1;
  }

  void yyerror(const char *str)
  {
    fprintf(stderr,"error: in line %d: %s at '%s'\n",lineno+1, str, yytext);
    exit(1);
  }

}

extern int yydebug;

char* c_string(const string& s)
{
  char* p = new char[s.length() + 1];
  strcpy(p, s.c_str());
  return p;
}


void __replace(string& src, const string& what, const string& with)
{
  int n = 0;
  while((n = src.find(what, n + what.size())) > 0) {
    src.replace(n, what.size(), with);
  }
}

bool __remove_namespaces(string& s) 
{
  size_t pos = s.rfind("::");
  if(pos != string::npos) {
    s = s.substr(pos+2);
    return true;
  }
  else {
    return false; // no :: were found -> false
  }
}

string __get_extra_in_file(const string& filename, const string& extension)
{
  string extra_in_filename = filename;
  __replace(extra_in_filename, ".idl", extension);

  // Try to open in-file
  ifstream in(extra_in_filename.c_str());
  
  string s = "";
  string line = "";
  if(in) {
    while(getline(in, line)) {
      s += line + "\n";
    }
  }
  return s;
} 

#define __append(old, app) if(old) { ostringstream s; s << old << app; old = c_string(s.str()); } else { old = app; }
#define __set(old, app) ostringstream __s; __s << app; old = c_string(__s.str());

string filename = "";
string suffix   = "";

