/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef NMM_WORKERTHREAD_HPP
#define NMM_WORKERTHREAD_HPP

#include "ThreadedObject.hpp"



namespace NMM {
  class Task {
  public:
    virtual void run() = 0;
  };

  template<class T>
  class TTask : public Task {
  typedef void (T::*Method)();
  public:
    TTask(T* object, Method method) : __object(object), __method(method) {}
    virtual void run() { (__object->*__method)(); }

  private:
    T*     __object;
    Method __method;
  };

  class WorkerThread : public ThreadedObject
  {
  public:
    WorkerThread();
    ~WorkerThread();
    void setTask(Task* task) { __current_task = task; }
    void iterateTask(int count = 1);
    int getIterationCount() const { return __iteration_count; }
    bool getReleaseFlag()   const { return __release_flag; }
    void setReleaseFlag( bool release_flag ) { __release_flag = release_flag; }
  protected:
    void run();
  private:
    Task*             __current_task;
    ThreadMutex       __thread_mutex;
    ThreadCondition   __thread_condition;
    int               __iteration_count;
    bool              __is_running;
    bool              __release_flag;
  };

}


#endif // NMM_WORKERTHREAD_HPP
